// CodePages.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CodePages.h"

#include "ErrorString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCodePages

CCodePages::CCodePages()
{
}

CCodePages::~CCodePages()
{
}


BEGIN_MESSAGE_MAP(CCodePages, CListCtrlEx)
        //{{AFX_MSG_MAP(CCodePages)
                // NOTE - the ClassWizard will add and remove mapping macros here.
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCodePages message handlers

void CCodePages::PreSubclassWindow() 
   {
    CListCtrlEx::PreSubclassWindow();

    InsertColumnEx(0, IDS_CODEPAGE_CAPTION, LVCFMT_LEFT);
    InsertColumnEx(1, IDS_DESCRIPTION_CAPTION, LVCFMT_LEFT, -1);
   }

/****************************************************************************
*                        CCodePages::compare
* Inputs:
*       LPARAM lParam1: The LPARAM of the first item to compare
*       LPARAM lParam2: The LPARAM of the second item to compare
*       LPARAM data: ignored
* Result: int
*       <0 if lParam1 < lParam2
*       =0 if lParam1 == lParam2
*       >0 if lParam1 > lParam2
* Notes:
*       This requires that we have stored the CodePage number in the LPARAM
*       field
*       Note that we encode errors as negative values so we must use the
*       abs to get a correct sort
****************************************************************************/

/* static */ int CALLBACK CCodePages::compare(LPARAM lParam1, LPARAM lParam2, LPARAM)
    {
     return abs(lParam1) - abs(lParam2);
    } // CCodePages::compare

/****************************************************************************
*                      CCodePages::EnumCodePagesProc
* Inputs:
*       LPTSTR codepage:
* Result: BOOL
*       TRUE, always
* Effect: 
*       Adds a codepage to the list
* Notes:
*       THIS IS NOT THREAD-SAFE! Due to terminal brain-damage on the part
*       of Microsoft, it is impossible to pass an additional parameter
*       into this enumerator!
****************************************************************************/
static CCodePages * Self = NULL;

/* static */ BOOL CALLBACK CCodePages::EnumCodePagesProc(LPTSTR codepage)
   {
    UINT cp = _ttoi(codepage);
    int n = Self->GetItemCount();
    Self->InsertItem(n, codepage);
    CPINFOEX info;
    if(GetCPInfoEx(cp, 0, &info))
       { /* got it */
        Self->SetItemText(n, 1, info.CodePageName);
        Self->SetItemData(n, cp);
       } /* got it */
    else
        { /* failed */
        DWORD err = ::GetLastError();
        Self->SetItemText(n, 1, _T("(") + ErrorString(err) + _T(")"));
        Self->SetItemData(n, -(int)cp); // negative value means error
        } /* failed */
    return TRUE;
   } // CCodePages::EnumCodePagesProc

/****************************************************************************
*                              CCodePages::Load
* Inputs:
*       UINT flags: Flags to use
*       BOOL stock: TRUE to load stock pages, FALSE to not
* Result: void
*       
* Effect: 
*       Loads the list of code pages
****************************************************************************/

void CCodePages::Load(UINT flags, BOOL stock)
    {
     Self = this;

     DeleteAllItems();

     if(!EnumSystemCodePages(EnumCodePagesProc, flags ))
        { /* error */
         DWORD err = ::GetLastError();
         // .. handle the error here
        } /* error */     

#define ADDSTOCKPAGE(x) { \
                        CString s; \
                        s.Format(_T("%d"), x); \
                        int n = InsertItem(0, s); \
                        SetItemData(n, x); \
                        SetItemText(n, 1, _T(#x)); \
                       }
     if(stock)
        { /* add stock */
         ADDSTOCKPAGE(CP_ACP);
         ADDSTOCKPAGE(CP_MACCP);
         ADDSTOCKPAGE(CP_OEMCP);
         ADDSTOCKPAGE(CP_THREAD_ACP);
        } /* add stock */
     
     SortItems(compare, NULL);
    } // CCodePages::Load
