#include "stdafx.h"
#include "resource.h"
#include "CodeTemplate.h"
#include "ErrorString.h"

/****************************************************************************
*                         CodeTemplate::ReadTemplates
* Inputs:
*       const CString & filename: File name
* Result: BOOL
*       TRUE if successful
*       FALSE if any error occurred (use ::GetLastError)
* Effect: 
*       Reads in a file of templates and handles them by
*       creating an internal table of templates and patterns
****************************************************************************/

BOOL CodeTemplate::ReadTemplates(const CString & filename)
    {
     CStdioFile f;
     if(!f.Open(filename, CFile::modeRead))
        { /* failed */
         DWORD err = ::GetLastError();
         templates.RemoveAll();
         ::SetLastError(err);
         this->err = err;
         this->filename = filename;
         return FALSE;
        } /* failed */

     CodeTemplateInfo * cti = NULL;
     templates.RemoveAll();

     CString line;
     while(f.ReadString(line))
        { /* parse line */
         if(!line.IsEmpty())
            { /* see if section */
             if(line.GetLength() >= 2)
                { /* check comment */
                 if(line[0] == _T('*') && line[1] == _T('*'))
                    continue;
                } /* check comment */
             if(line[0] == _T('['))
                { /* decode token */
                 CString test = line;
                 test.MakeUpper();
                 test.TrimRight();
                 if(cti != NULL)
                    { /* put existing into table */
                     cti->pattern.TrimLeft();
                     cti->pattern.TrimRight();
                     templates.Add(cti);
                     cti = NULL;
                    } /* put existing into table */
                 cti = new CodeTemplateInfo;
                 cti->name = test;
                 continue;
                } /* decode token */
            } /* see if section */

         // It is not a [section] designatore
         if(cti != NULL)
            { /* append to target */
             cti->pattern += line + _T("\r\n");
            } /* append to target */
        } /* parse line */
     if(cti != NULL)
        { /* dump last */
         cti->pattern.TrimLeft();
         cti->pattern.TrimRight();
         templates.Add(cti);
         cti = NULL;
        } /* dump last */

     this->err = ERROR_SUCCESS; // indicate valid pattern
     this->filename = filename;

     return TRUE;
    } // CodeTemplate::ReadTemplates

/****************************************************************************
*                          CodeTemplate::GetTemplate
* Inputs:
*       CString name: Name of template
* Result: CString
*       The template value. If the template is not found, the empty string
*       is returned
****************************************************************************/

CString CodeTemplate::GetTemplate(CString name)
    {
     if(err != ERROR_SUCCESS)
        { /* failed to read pattern */
         CString s;
         CString fmt;
         fmt.LoadString(IDS_TEMPLATE_LOAD_FAILED);
         s.Format(fmt, filename, ErrorString(err));
         s.Replace(_T("%"), _T("")); // just in case user had silly name!
         return s;
        } /* failed to read pattern */
     CString result; 
     name.MakeUpper();  // canonical form uses all-upper-case named
     for(int i = 0; i < templates.GetSize(); i++)
        { /* check each */
         CodeTemplateInfo * t = templates[i];
         if(t->name == name)
            { /* found it */
             result = t->pattern;
             break;
            } /* found it */
        } /* check each */
     return result;
    } // CodeTemplate::GetTemplate

/****************************************************************************
*                         CodeTemplate::GetTemplates
* Result: CString
*       Text of all the templates
****************************************************************************/

#ifdef _DEBUG //----------------------------------------------------------------+
CString CodeTemplate::GetTemplates()                                         // |
    {                                                                        // |
     CString result;                                                         // |
                                                                             // |
     for(int i = 0; i < templates.GetSize(); i++)                            // |
        { /* get each */                                                     // |
         result += templates[i]->name;                                       // |
         result += _T("\r\n");                                               // |
         result += templates[i]->pattern;                                    // |
         result += _T("\r\n");                                               // |
        } /* get each */                                                     // |
     return result;                                                          // |
    } // CodeTemplate::GetTemplates                                          // |
#endif //-----------------------------------------------------------------------+
    
