#include "stdafx.h"
#include "DoEscape.h"


/****************************************************************************
*                                  DoEscape
* Inputs:
*       CString & s: String to convert
* Result: void
*       
* Effect: 
*       Converts \ sequences to the actual characters
****************************************************************************/

void DoEscape(CString & s)
    {
     CString result;
     enum {NORMAL, BACKSLASH, OCTAL, HEX} state = NORMAL;

     UINT n;
     // For UNICODE, the escape will be
     // octal: \0..\0177777
     // hex:   \x0..\xFFFF
     // For ANSI, the escape will be
     // octal: \0..\0377
     // hex:   \x0..\xFF
     // Because this is in C, we disallow the \0 character
     // If the value is too large for the representation mode, it will be truncated to the
     // low-order 8 bits

     for(int i = 0; i < s.GetLength(); i++)
        { /* scan and convert */
         switch(state)
            { /* state */
             case NORMAL:
                if(s[i] == _T('\\'))
                   { /* start escape */
                    state = BACKSLASH;
                    continue;
                   } /* start escape */
                result += s[i];
                continue;
             case BACKSLASH:
                switch(s[i])
                   { /* esc type */
                    case _T('a'):
                       result += _T('\a');
                       state = NORMAL;
                       continue;
                    case _T('b'):
                       result += _T('\b');
                       state = NORMAL;
                       continue;
                    case _T('f'):
                       result += _T('f');
                       state = NORMAL;
                       continue;
                    case _T('n'):
                       result += _T('\n');
                       state = NORMAL;
                       continue;
                    case _T('r'):
                       result += _T('\r');
                       state = NORMAL;
                       continue;
                    case _T('t'):
                       result += _T('\t');
                       state = NORMAL;
                       continue;
                    case _T('v'):
                       result += _T('\v');
                       state = NORMAL;
                       continue;
                    case _T('\\'):
                       result += _T('\\');
                       state = NORMAL;
                       continue;
                    case _T('0'):
                       n = 0;
                       state = OCTAL;
                       continue;
                    case _T('x'):
                       n = 0;
                       state = HEX;
                       continue;
                   } /* esc type */
                // Illegal escape sequence. Ignore it
                result += _T('\\');
                result += s[i];
                state = NORMAL;
                continue;
             case OCTAL:
                switch(s[i])
                   { /* octal */
                    case _T('0'):
                    case _T('1'):
                    case _T('2'):
                    case _T('3'):
                    case _T('4'):
                    case _T('5'):
                    case _T('6'):
                    case _T('7'):
                       n = n * 8 + (s[i] - _T('0'));
                       continue;
                    default:
                       if((TCHAR)n != 0)
                          { /* non-NUL */
                           result += (TCHAR)n;
                          } /* non-NUL */
                       result += s[i];
                       state = NORMAL;
                       continue;
                   } /* octal */
             case HEX:
                switch(s[i])
                   { /* hex */
                    case _T('0'):
                    case _T('1'):
                    case _T('2'):
                    case _T('3'):
                    case _T('4'):
                    case _T('5'):
                    case _T('6'):
                    case _T('7'):
                    case _T('8'):
                    case _T('9'):
                       n = n * 16 + (s[i] - _T('0'));
                       continue;
                    case _T('A'):
                    case _T('a'):
                    case _T('B'):
                    case _T('b'):
                    case _T('C'):
                    case _T('c'):
                    case _T('D'):
                    case _T('d'):
                    case _T('E'):
                    case _T('e'):
                    case _T('F'):
                    case _T('f'):
                       n = n * 16 + (_totupper(s[i]) - _T('A') + 10);
                       continue;
                    default:
                       if((TCHAR)n != 0)
                          { /* use hex */
                           result += (TCHAR)n;
                          } /* use hex */
                       result += s[i];
                       state = NORMAL;
                       continue;
                   } /* hex */
             default:
                ASSERT(FALSE); // impossible error
            } /* state */
        } /* scan and convert */
    } // DoEscape
