// EnumSystemCodePages.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "EnumSystemCodePages.h"
#include "Query.h"
#include "msg.h"
#include "ErrorString.h"
#include "LogWnd.h"
#include "RTLMap.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CEnumSystemCodePages, CFunction)
DECLARE_MESSAGE(UWM_LOG_DATA)

/////////////////////////////////////////////////////////////////////////////
// CEnumSystemCodePages dialog


CEnumSystemCodePages::CEnumSystemCodePages(CWnd* pParent /*=NULL*/)
        : CFunction(CEnumSystemCodePages::IDD, pParent)
{
        //{{AFX_DATA_INIT(CEnumSystemCodePages)
        //}}AFX_DATA_INIT
}


void CEnumSystemCodePages::DoDataExchange(CDataExchange* pDX)
{
        CFunction::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CEnumSystemCodePages)
        DDX_Control(pDX, IDC_DWFLAGS, x_dwFlags);
        DDX_Control(pDX, IDC_OUTPUT, c_Output);
        DDX_Control(pDX, IDC_CP_SUPPORTED, c_CP_SUPPORTED);
        DDX_Control(pDX, IDC_CP_INSTALLED, c_CP_INSTALLED);
        DDX_Control(pDX, IDC_CALLBACK_NAME, c_CallbackName);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEnumSystemCodePages, CFunction)
        //{{AFX_MSG_MAP(CEnumSystemCodePages)
        ON_EN_CHANGE(IDC_CALLBACK_NAME, OnChangeCallbackName)
        ON_BN_CLICKED(IDC_CP_INSTALLED, OnCpInstalled)
        ON_BN_CLICKED(IDC_CP_SUPPORTED, OnCpSupported)
        ON_WM_SIZE()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEnumSystemCodePages message handlers

/****************************************************************************
*                      CEnumSystemCodePages::Initialize
* Result: void
*       
* Effect: 
*       Sets the variable value if it is not set
****************************************************************************/

void CEnumSystemCodePages::Initialize()
    {
     variable = _T("cp"); 
    } // CEnumSystemCodePages::Initialize

/****************************************************************************
*                     CEnumSystemCodePages::OnInitDialog
* Result: BOOL
*       TRUE, always
* Effect: 
*       Initializes the control
****************************************************************************/

BOOL CEnumSystemCodePages::OnInitDialog() 
   {
    CFunction::OnInitDialog();
        
    // TODO: Add extra initialization here
    CheckRadioButton(IDC_CP_INSTALLED, IDC_CP_SUPPORTED, IDC_CP_INSTALLED);
        
#define COL_CODE_PAGE_BASE 0
    c_Output.InsertColumnEx(COL_CODE_PAGE_BASE, IDS_CODEPAGE_CAPTION, LVCFMT_RIGHT);
#define COL_MAXCHARSIZE    1
    c_Output.InsertColumnEx(COL_MAXCHARSIZE, _T("MaxCharSize"), LVCFMT_RIGHT);
#define COL_DEFAULTCHAR    2
    CString caption;
    caption.Format(_T("DefaultChar[%d]"), MAX_DEFAULTCHAR);
    c_Output.InsertColumnEx(COL_DEFAULTCHAR, caption, LVCFMT_LEFT);
#define COL_LEADBYTE       3
    caption.Format(_T("LeadByte[%d]"), MAX_LEADBYTES);
    c_Output.InsertColumnEx(COL_LEADBYTE, caption, LVCFMT_LEFT, _T("00 00 00 00 00 [000.000 000..000]"));
#define COL_UNICODEDEFAULTCHAR 4
    c_Output.InsertColumnEx(COL_UNICODEDEFAULTCHAR, _T("UnicodeDefaultChar"), LVCFMT_RIGHT);
#define COL_CODEPAGE       5
    c_Output.InsertColumnEx(COL_CODEPAGE, _T("CodePage"), LVCFMT_RIGHT);
#define COL_CODEPAGENAME   6
    c_Output.InsertColumnEx(COL_CODEPAGENAME, _T("CodePageName"), LVCFMT_LEFT, -1);

#define COL_ERROR          6
    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
   }

void CEnumSystemCodePages::OnChangeCallbackName() 
   {
    Refresh();
   }

/****************************************************************************
*                   CEnumSystemCodePages::EnumCodePagesProc
* Inputs:
*       LPTSTR codepage:
* Result: BOOL
*       TRUE, always
* Effect: 
*       Logs the code page
****************************************************************************/
CEnumSystemCodePages * CEnumSystemCodePages::LogWnd = NULL;

/* static */ BOOL CALLBACK CEnumSystemCodePages::EnumCodePagesProc(LPTSTR codepage)
   {
    if(LogWnd == NULL)
       return TRUE;
    LCID lcid = (LCID)LogWnd->TabParent->SendMessage(UWM_QUERY, (WPARAM)QUERY_LOCALE);
    int codepagenum = _ttoi(codepage);

    CString info = codepage;
    RTLMap(info, lcid);
    int n = LogWnd->c_Output.GetItemCount();
    int item = LogWnd->c_Output.InsertItem(n, codepage);
    UINT cp = _ttoi(codepage);

    // Store the code page number so the SortItems has something to work with
    LogWnd->c_Output.SetItemData(n, cp);

    CPINFOEX cpinfo;
    if(GetCPInfoEx(cp, 0, &cpinfo))
       { /* cpinfo */
        CString s;
        //----------------------------------------------------------------
        // MaxCharSize
        //----------------------------------------------------------------
        s.Format(_T("%d"), cpinfo.MaxCharSize);
        LogWnd->c_Output.SetItemText(n, COL_MAXCHARSIZE, s);

        //----------------------------------------------------------------
        // DefaultChar
        //----------------------------------------------------------------
        ASSERT(strlen((LPCSTR)cpinfo.DefaultChar) <= 1); // NOTE: strlen, we want 8-bit test here!

        s.Format(_T("0x%02x, '%c'"), cpinfo.DefaultChar[0], cpinfo.DefaultChar[0]);
        LogWnd->c_Output.SetItemText(n, COL_DEFAULTCHAR, s);

        //----------------------------------------------------------------
        // LeadByte
        //----------------------------------------------------------------
        /*
          Specifies a fixed-length array of lead-byte ranges, where the number
          of lead-byte ranges is variable. If there are no lead bytes in this
          code page, every element of the array is NULL. If there are lead
          bytes in this code page, the array contains a starting and ending
          value for each range. Ranges are inclusive. The maximum number of
          lead-byte ranges for any code page is five. The array uses 2 bytes to
          describe each range, with a double-byte null terminator after the
          last range. 
        */
        CString leadbytes;
        for(int i = 0; i < MAX_LEADBYTES; i+=2)
           { /* format in hex */
            CString t;
            t.Format(_T("%02x %02x "), cpinfo.LeadByte[i], cpinfo.LeadByte[i+1]);
            leadbytes += t;
            if(cpinfo.LeadByte[i] == 0 && cpinfo.LeadByte[i+1] == 0)
               break;
           } /* format in hex */

        CString prefix = _T("[");
        for(i = 0; ; i+= 2)
           { /* lead byte formatting */
            if(cpinfo.LeadByte[i] == 0 && cpinfo.LeadByte[i+1] == 0)
               break; // a double-0 terminate the string
            CString t;
            t.Format(_T("%d..%d"), cpinfo.LeadByte[i], cpinfo.LeadByte[i+1]);
            leadbytes += prefix;
            prefix = _T(" ");
            leadbytes += t;
           } /* lead byte formatting */
        prefix.TrimLeft(); // in case it was space
        if(prefix.IsEmpty()) // if we put anything there, close it off
           leadbytes += _T("]");

        LogWnd->c_Output.SetItemText(n, COL_LEADBYTE, leadbytes);

        //----------------------------------------------------------------
        // UnicodeDefaultChar
        //----------------------------------------------------------------
        s.Format(_T("U%04x"), cpinfo.UnicodeDefaultChar);
        LogWnd->c_Output.SetItemText(n, COL_UNICODEDEFAULTCHAR, s);

        //----------------------------------------------------------------
        // CodePage
        //----------------------------------------------------------------
        s.Format(_T("%d"), cpinfo.CodePage);
        LogWnd->c_Output.SetItemText(n, COL_CODEPAGE, s);
        
        //----------------------------------------------------------------
        // CodePageName
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_CODEPAGENAME, cpinfo.CodePageName);
       } /* cpinfo */
    else
       { /* failed */
        DWORD err = ::GetLastError();

        //----------------------------------------------------------------
        // MaxCharSize
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_MAXCHARSIZE, _T("---"));
        //----------------------------------------------------------------
        // DefaultChar
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_DEFAULTCHAR, _T("----"));
        //----------------------------------------------------------------
        // LeadByte
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_LEADBYTE, _T("--------"));
        //----------------------------------------------------------------
        // UnicodeDefaultChar
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_UNICODEDEFAULTCHAR, _T("------"));
        //----------------------------------------------------------------
        // CodePage
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_CODEPAGE, _T("----"));
        //----------------------------------------------------------------
        // CodePageName
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_CODEPAGENAME, _T("----------------"));
        //----------------------------------------------------------------
        // Error message
        // Note this is last so that it overwrites one of the previous
        // columns but we can change its actual assignment as we feel fit
        //----------------------------------------------------------------
        LogWnd->c_Output.SetItemText(n, COL_ERROR, ErrorString(err));
       } /* failed */
    return TRUE;
   }

/****************************************************************************
*                        CEnumSystemCodePages::compare
* Inputs:
*       LPARAM lParam1: The LPARAM of the first item to compare
*       LPARAM lParam2: The LPARAM of the second item to compare
*       LPARAM data: ignored
* Result: int
*       <0 if lParam1 < lParam2
*       =0 if lParam1 == lParam2
*       >0 if lParam1 > lParam2
* Notes:
*       This requires that we have stored the CodePage number in the LPARAM
*       field
****************************************************************************/

int CALLBACK CEnumSystemCodePages::compare(LPARAM lParam1, LPARAM lParam2, LPARAM)
    {
     return lParam1 - lParam2;
    } // CEnumSystemCodePages::compare

/****************************************************************************
*                      CEnumSystemCodePages::Refresh
* Result: void
*       
* Effect: 
*       Updates the codex example
****************************************************************************/

void CEnumSystemCodePages::Refresh()
    {
     if(!initialized)
        return;

     EnableQuery(QUERY_NOUSEROVERRIDE, FALSE);
     EnableQuery(QUERY_USE_CP_ACP, FALSE);
     EnableQuery(QUERY_DWORD, FALSE);

     CString defaultCallback;

     CString or = QueryTemplate(_T("[OR]"));

     CString fmt;
     fmt = QueryTemplate(_T("[EnumSystemCodePages:VARIABLE]"));
     defaultCallback = _T("EnumCodePagesProc");

     //----------------------------------------------------------------
     // %1: The LCID
     //----------------------------------------------------------------
     CString localeName = GetLocaleName();

     //----------------------------------------------------------------
     // %2: The DWORD as a string
     //----------------------------------------------------------------
     CString dwordparameter = _T(""); // todo

     //----------------------------------------------------------------
     // %3: The option flags
     //----------------------------------------------------------------
     CString options;
     DWORD flags = 0;
     GetOptionFlags(options, flags, or);

     if(c_CP_INSTALLED.GetCheck() == BST_CHECKED)
        { /* CP_INSTALLED */
         options = _T("CP_INSTALLED ") + options;
         flags |= CP_INSTALLED;
        } /* CP_INSTALLED */
     else
     if(c_CP_SUPPORTED.GetCheck() == BST_CHECKED)
        { /* CP_SUPPORTED */
         options = _T("CP_SUPPORTED ") + options;
         flags |= CP_SUPPORTED;
        } /* CP_SUPPORTED */

     //----------------------------------------------------------------
     // %4: Length: not used
     //----------------------------------------------------------------
     CString length;

     //----------------------------------------------------------------
     // %5: The variable name
     //----------------------------------------------------------------
     CString variable;
     variable = GetVariable();

     //----------------------------------------------------------------
     // %6: dwFlags as a decimal number
     //----------------------------------------------------------------
     CString dwFlagsDecimal;
     dwFlagsDecimal.Format(_T("%d"), flags);

     //----------------------------------------------------------------
     // %7: The OR operator for bitwise operations
     //----------------------------------------------------------------
     // This was set when the language choice was made

     //----------------------------------------------------------------
     // %8: The locale ID as a decimal integer
     //----------------------------------------------------------------
     LCID lcid = QueryLocale();
     CString localeDecimal;
     localeDecimal.Format(_T("%d"), lcid);

     //----------------------------------------------------------------
     // %9: Not used
     //----------------------------------------------------------------

     //----------------------------------------------------------------
     // %10: Not used
     //----------------------------------------------------------------

     //----------------------------------------------------------------
     // %11: The callback function name
     //----------------------------------------------------------------
     CString callbackname;
     c_CallbackName.GetWindowText(callbackname);
     callbackname.TrimLeft();
     callbackname.TrimRight();
     if(callbackname.IsEmpty())
        callbackname = defaultCallback;

     //-----------------------------------------------------------------------------
     // Create the actual text from the template
     //-----------------------------------------------------------------------------
     CString text;

     //-----------------------------------------------------------------------------------------------
     //                        %1           %2               %3       %4       %5           %6        %7     %8      %9     %10       %11
     text.FormatMessage(fmt, localeName,  dwordparameter, options, length, variable, dwFlagsDecimal,  or, _T(""), _T(""), _T(""), callbackname);

     //-----------------------------------------------------------------------------------------------
     SetCode(text, !variable.IsEmpty());

     c_Output.DeleteAllItems();
     LogWnd = this;

     if(!::EnumSystemCodePages(EnumCodePagesProc, flags))
        { /* failed */
         DWORD err = ::GetLastError();
         //c_Output.AddString(ErrorString(err));
        } /* failed */
     else
        { /* success */
         c_Output.SortItems(compare, NULL);
        } /* success */
    } // CEnumSystemCodePages::Refresh

void CEnumSystemCodePages::OnCpInstalled() 
   {
    Refresh();
   }

void CEnumSystemCodePages::OnCpSupported() 
   {
    Refresh();
   }

/****************************************************************************
*                       CEnumSystemCodePages::SetFontToControls
* Result: void
*       
* Effect: 
*       Sets the font in the relevant controls
****************************************************************************/

void CEnumSystemCodePages::SetFontToControls()
    {
#ifdef _UNICODE
     c_Output.SetFont(&UnicodeFont);
#endif
    } // CEnumSystemCodePages::SetFontToControls

/****************************************************************************
*                        CEnumSystemCodePages::OnSize
* Inputs:
*       UINT nType:
*       int cx: New client width
*       int cy: New client height
* Result: void
*       
* Effect: 
*       Resizes the output window
****************************************************************************/

void CEnumSystemCodePages::OnSize(UINT nType, int cx, int cy) 
   {
    CFunction::OnSize(nType, cx, cy);
        
    ExpandBottom(c_Output);

    ExtendColumn(c_Output);
   }

/****************************************************************************
*                       CEnumSystemCodePages::QuerySave
* Result: BOOL
*       TRUE if save is possible
*       FALSE if save not possible
****************************************************************************/

BOOL CEnumSystemCodePages::QuerySave()
    {
     return c_Output.GetItemCount() > 0;
    } // CEnumSystemCodePages::QuerySave

/****************************************************************************
*                         CEnumSystemCodePages::Save
* Inputs:
*       const CString & filename: File name
*       UINT mode: Save mode: 0 (default) 8 (UTF-8) 16 (UTF-16)
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Saves the contents of the list
****************************************************************************/

BOOL CEnumSystemCodePages::Save(const CString & filename, UINT mode)
    {
     return c_Output.Save(filename, mode);
    } // CEnumSystemCodePages::Save
