// EnumSystemLocales.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "EnumSystemLocales.h"
#include "Query.h"
#include "ErrorString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CEnumSystemLocales, CFunction)

/////////////////////////////////////////////////////////////////////////////
// CEnumSystemLocales dialog


CEnumSystemLocales::CEnumSystemLocales(CWnd* pParent /*=NULL*/)
        : CFunction(CEnumSystemLocales::IDD, pParent)
{
        //{{AFX_DATA_INIT(CEnumSystemLocales)
                // NOTE: the ClassWizard will add member initialization here
        //}}AFX_DATA_INIT
}


void CEnumSystemLocales::DoDataExchange(CDataExchange* pDX)
{
        CFunction::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CEnumSystemLocales)
        DDX_Control(pDX, IDC_0, c_LCID_0);
        DDX_Control(pDX, IDC_CALLBACK_NAME, c_CallbackName);
        DDX_Control(pDX, IDC_OUTPUT, c_Output);
        DDX_Control(pDX, IDC_LCID_SUPPORTED, c_LCID_SUPPORTED);
        DDX_Control(pDX, IDC_LCID_INSTALLED, c_LCID_INSTALLED);
        DDX_Control(pDX, IDC_LCID_ALTERNATE_SORTS, c_LCID_ALTERNATE_SORTS);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEnumSystemLocales, CFunction)
        //{{AFX_MSG_MAP(CEnumSystemLocales)
        ON_WM_SIZE()
        ON_BN_CLICKED(IDC_LCID_INSTALLED, OnLcidInstalled)
        ON_BN_CLICKED(IDC_LCID_SUPPORTED, OnLcidSupported)
        ON_BN_CLICKED(IDC_0, On0)
        ON_BN_CLICKED(IDC_LCID_ALTERNATE_SORTS, OnLcidAlternateSorts)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEnumSystemLocales message handlers

/****************************************************************************
*                     CEnumSystemLocales::EnumLocalesProc
* Inputs:
*       LPTSTR locale: Locale
* Result: BOOL
*       TRUE, always
* Effect: 
*       Enumerates the locales
****************************************************************************/

CEnumSystemLocales * CEnumSystemLocales::LogWnd = NULL;
        
BOOL CEnumSystemLocales::EnumLocalesProc(LPTSTR locale)
    {
     if(LogWnd == NULL)
        return TRUE;

     int n = LogWnd->c_Output.GetItemCount();
     LogWnd->c_Output.InsertItem(n, locale);

     LCID lcid = _tcstoul(locale, NULL, 16);

     // LOCALE_SLANGUAGE
     CString slanguage_data;
     { /* get LOCALE_SLANGUAGE */
      int length = GetLocaleInfo(lcid, LOCALE_SLANGUAGE, NULL, 0);
      LPTSTR p = slanguage_data.GetBuffer(length);
      VERIFY(::GetLocaleInfo(lcid, LOCALE_SLANGUAGE, p, length));
     } /* get LOCALE_SLANGUAGE */
     slanguage_data.ReleaseBuffer();
     // ... use slanguage_data here

     LogWnd->c_Output.SetItemText(n, 1, slanguage_data);
     return TRUE;
    } // CEnumSystemLocales::EnumLocalesProc

/****************************************************************************
*                      CEnumSystemLocales::OnInitDialog
* Result: BOOL
*       TRUE, always
* Effect: 
*       Initializes the dialog
****************************************************************************/

BOOL CEnumSystemLocales::OnInitDialog() 
   {
    CFunction::OnInitDialog();
        
    CheckRadioButton(IDC_LCID_INSTALLED, IDC_0, IDC_LCID_INSTALLED);
    
    c_Output.InsertColumnEx(0, _T("LCID"), LVCFMT_RIGHT, _T("00000000"));
    c_Output.InsertColumnEx(1, _T("LOCALE_SLANGUAGE"), LVCFMT_LEFT, -1);
        
    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
   }

/****************************************************************************
*                         CEnumSystemLocales::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Handles resizing
****************************************************************************/

void CEnumSystemLocales::OnSize(UINT nType, int cx, int cy) 
   {
    CFunction::OnSize(nType, cx, cy);
        
    ExpandBottom(c_Output);
   }

/****************************************************************************
*                         CEnumSystemLocales::Refresh
* Result: void
*       
* Effect: 
*       Refreshes the state and code
****************************************************************************/

void CEnumSystemLocales::Refresh()
    {
     if(!initialized)
         return;
     EnableQuery(QUERY_NOUSEROVERRIDE, FALSE);
     EnableQuery(QUERY_USE_CP_ACP, FALSE);
     EnableQuery(QUERY_DWORD, FALSE);

     CString or = QueryTemplate(_T("[OR]"));

     CString fmt = QueryTemplate(_T("[EnumSystemLocales:VARIABLE]"));

     CString defaultCallback = _T("EnumLocalesProc");

     //----------------------------------------------------------------
     // %1: The LCID
     //----------------------------------------------------------------
     CString localeName = GetLocaleName(); // not actually needed

     //----------------------------------------------------------------
     // %2: not used
     //----------------------------------------------------------------
     
     //----------------------------------------------------------------
     // %3: The option flags
     //----------------------------------------------------------------
     CString options;
     DWORD flags = 0;

     CString prefix;
     if(c_LCID_ALTERNATE_SORTS.GetCheck() == BST_CHECKED)
        { /* add something */
         options = _T("LCID_ALTERNATE_SORTS");
         prefix = _T(" ") + or + _T(" ");
         flags |= LCID_ALTERNATE_SORTS;
        } /* add something */

     if(c_LCID_INSTALLED.GetCheck() == BST_CHECKED)
        { /* installed */
         options += prefix + _T("LCID_INSTALLED");
         flags |= LCID_INSTALLED;
        } /* installed */
     else
     if(c_LCID_SUPPORTED.GetCheck() == BST_CHECKED)
        { /* supported */
         options += prefix + _T("LCID_SUPPORTED");
         flags |= LCID_SUPPORTED;
        } /* supported */
     else
     if(c_LCID_0.GetCheck() == BST_CHECKED)
        { /* 0 */
        } /* 0 */
     else
        ASSERT(FALSE); // should not happen

     if(options.IsEmpty())
         options = _T("0");

     //----------------------------------------------------------------
     // %4: Length: not used
     //----------------------------------------------------------------
     CString length;

     //----------------------------------------------------------------
     // %5: The variable name
     //----------------------------------------------------------------
     CString variable;
     variable = GetVariable();

     //----------------------------------------------------------------
     // %6: dwFlags as a decimal number
     //----------------------------------------------------------------
     CString dwFlagsDecimal;
     dwFlagsDecimal.Format(_T("%d"), flags);

     //----------------------------------------------------------------
     // %7: The OR operator for bitwise operations
     //----------------------------------------------------------------
     // already set

     //----------------------------------------------------------------
     // %8: The locale ID as a decimal integer
     //----------------------------------------------------------------
     LCID lcid = QueryLocale();
     CString localeDecimal;
     localeDecimal.Format(_T("%d"), lcid);

     //----------------------------------------------------------------
     // %9: Not used
     //----------------------------------------------------------------

     //----------------------------------------------------------------
     // %10: Not used
     //----------------------------------------------------------------

     //----------------------------------------------------------------
     // %11: The callback function name
     //----------------------------------------------------------------
     CString callbackname;
     c_CallbackName.GetWindowText(callbackname);
     callbackname.TrimLeft();
     callbackname.TrimRight();
     if(callbackname.IsEmpty())
        callbackname = defaultCallback;

     //-----------------------------------------------------------------------------
     // Create the actual text from the template
     //-----------------------------------------------------------------------------
     CString text;

     //-----------------------------------------------------------------------------------------------
     //                        %1           %2       %3      %4       %5            %6        %7        %8          %9     %10       %11
     text.FormatMessage(fmt, localeName,  _T(""), options, length, variable, dwFlagsDecimal,  or, localeDecimal, _T(""), _T(""), callbackname);

     //-----------------------------------------------------------------------------------------------
     SetCode(text, !variable.IsEmpty());
     c_Output.DeleteAllItems();

     LogWnd = this;
     if(!EnumSystemLocales(EnumLocalesProc, flags))
        { /* failed */
         DWORD err = ::GetLastError();
         c_Output.InsertItem(c_Output.GetItemCount(), ErrorString(err));
        } /* failed */
     else
        { /* success */
        } /* success */
    } // CEnumSystemLocales::Refresh

void CEnumSystemLocales::OnLcidInstalled() 
   {
    Refresh();
   }

void CEnumSystemLocales::OnLcidSupported() 
   {
    Refresh();
   }

void CEnumSystemLocales::On0() 
   {
    Refresh();
   }

void CEnumSystemLocales::OnLcidAlternateSorts() 
   {
    Refresh();
   }

/****************************************************************************
*                    CEnumSystemLocales::SetFontToControls
* Result: void
*       
* Effect: 
*       Sets the font
****************************************************************************/

void CEnumSystemLocales::SetFontToControls()
    {
#ifdef _UNICODE
     c_Output.SetFont(&UnicodeFont);
#endif
    } // CEnumSystemLocales::SetFontToControls

/****************************************************************************
*                        CEnumSystemLocales::QuerySave
* Result: BOOL
*       TRUE if save possible
*       FALSE if save not possible
* Effect: 
*       Saves the data
****************************************************************************/

BOOL CEnumSystemLocales::QuerySave()
    {
     return c_Output.GetItemCount() > 0;
    } // CEnumSystemLocales::QuerySave

/****************************************************************************
*                          CEnumSystemLocales::Save
* Inputs:
*       const CString & filename:
*       UINT mode:
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Saves the data
****************************************************************************/

BOOL CEnumSystemLocales::Save(const CString & filename, UINT mode)
    {
     return c_Output.Save(filename, mode);
    } // CEnumSystemLocales::Save
