// ExpandingTreeCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "ExpandingTreeCtrl.h"
#include "Help.h"
#include "lctypemsg.h"
#include "AllChildren.h"
#include "GetCaption.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT UWM_SELECT_LCTYPE = ::RegisterWindowMessage(UWM_SELECT_LCTYPE_MSG);
static UINT UWM_DESELECT_ALL  = ::RegisterWindowMessage(UWM_DESELECT_ALL_MSG);
/////////////////////////////////////////////////////////////////////////////
// CExpandingTreeCtrl

CExpandingTreeCtrl::CExpandingTreeCtrl()
   {
    expanded = FALSE;
    wantexpand = FALSE;
    selected = FALSE;
    HelpBrush.CreateSolidBrush(::GetSysColor(COLOR_INFOBK));
   }

CExpandingTreeCtrl::~CExpandingTreeCtrl()
{
}


BEGIN_MESSAGE_MAP(CExpandingTreeCtrl, CTreeCtrl)
        ON_REGISTERED_MESSAGE(UWM_DESELECT_ALL, OnDeselectAll)
        //{{AFX_MSG_MAP(CExpandingTreeCtrl)
        ON_NOTIFY_REFLECT(NM_KILLFOCUS, OnKillfocus)
        ON_NOTIFY_REFLECT(NM_SETFOCUS, OnSetfocus)
        ON_COMMAND(IDM_WHAT_IS_THIS, OnWhatIsThis)
        ON_WM_CONTEXTMENU()
        ON_WM_CTLCOLOR_REFLECT()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExpandingTreeCtrl message handlers

/****************************************************************************
*                       CExpandingTreeCtrl::OnKillfocus
* Inputs:
*       NMHDR * pNMHDR:
*       LRESULT * pResult:
* Result: void
*       
* Effect: 
*       If autoexpand is set and the window is expanded, make it small again
****************************************************************************/

void CExpandingTreeCtrl::OnKillfocus(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    if(expanded)
       ResumeNormalSize();
    *pResult = 0;
   }

/****************************************************************************
*                       CExpandingTreeCtrl::OnSetfocus
* Inputs:
*       NMHDR * pNMHDR:
*       LRESULT * pResult:
* Result: void
*       
* Effect: 
*       If autoexpand is set, forces window to expand
****************************************************************************/

void CExpandingTreeCtrl::OnSetfocus(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    GetParent()->SendMessage(UWM_DESELECT_ALL);
    if(!expanded && wantexpand)
       ExpandWindow();
    GetParent()->SendMessage(UWM_SELECT_LCTYPE, 0, (LPARAM)GetDlgCtrlID());
    selected = TRUE;
    *pResult = 0;
   }

/****************************************************************************
*                      CExpandingTreeCtrl::SetAutoExpand
* Inputs:
*       BOOL mode: Desired autoexpand mode
* Result: BOOL
*        Previous value
* Effect: 
*       Sets the autoexpand mode.
* Notes:
*       if the control already has the focus, and the mode is TRUE, expands it
*       If the control is already expanded, and the mode is FALSE, reduces it
****************************************************************************/

BOOL CExpandingTreeCtrl::SetAutoExpand(BOOL mode)
    {
     BOOL result = wantexpand;

     if(!result && expanded)
        { /* contract it */
         ResumeNormalSize();
        } /* contract it */
     if(m_hWnd == ::GetFocus() && mode)
        { /* expand it */
         ExpandWindow();
        } /* expand it */
     wantexpand = mode;
     return result;
    } // CExpandingTreeCtrl::SetAutoExpand

/****************************************************************************
*                    CExpandingTreeCtrl::PresubclassWindow
* Result: void
*       
* Effect: 
*       Stores original size for expand/reduce
****************************************************************************/

void CExpandingTreeCtrl::PreSubclassWindow() 
   {
    GetWindowRect(&original);
    anchor = GetWindow(GW_HWNDPREV); // anchor to previous caption
    CTreeCtrl::PreSubclassWindow();
   }

/****************************************************************************
*                    CExpandingTreeCtrl::ResumeNormalSize
* Result: void
*       
* Effect: 
*       Sets the window back to its original size
****************************************************************************/

void CExpandingTreeCtrl::ResumeNormalSize()
    {
     ManageOverlappedControls(SW_SHOW);
     SetWindowPos(anchor, 0, 0, original.Width(), original.Height(), SWP_NOMOVE);
     HTREEITEM selection = GetSelectedItem();
     if(selection != NULL)
        { /* had selection */
         EnsureVisible(selection);
        } /* had selection */
     expanded = FALSE;
    } // CExpandingTreeCtrl::ResumeNormalSize

/****************************************************************************
*                      CExpandingTreeCtrl::ExpandWindow
* Effect: 
*       Expands the window to cover a fair amount of the parent, vertically
****************************************************************************/

void CExpandingTreeCtrl::ExpandWindow()
    {
     CRect me;
     GetWindowRect(&me);
     GetParent()->ScreenToClient(&me);
     
     CRect parent;
     GetParent()->GetWindowRect(&parent);

     // expand to some size
#define absorption 0.75 // amount of parent window to absorb

     int height = (int)((parent.Height() - me.top) * absorption);

     // Before we make this topmost, we save its predecessor (caption)
     // window so we can retrieve it in GetCaption()
     if(::GetWindowLongPtr(m_hWnd, GWLP_USERDATA) == NULL)
        { /* update it */
         CWnd * prev = GetWindow(GW_HWNDPREV);
         ::SetWindowLongPtr(m_hWnd, GWLP_USERDATA, (LPARAM)prev->m_hWnd);
        } /* update it */

     SetWindowPos(&wndTop, 0, 0, original.Width(), height, SWP_NOMOVE);
     expanded = TRUE;
     ManageOverlappedControls(SW_HIDE);
    } // CExpandingTreeCtrl::ExpandWindow

/****************************************************************************
*                CExpandingTreeCtrl::ManageOverlappedControls
* Inputs:
*       UINT mode: Mode to set windows
* Result: void
*       
* Effect: 
*       For every window underneath the current window, hide it or show it
****************************************************************************/

void CExpandingTreeCtrl::ManageOverlappedControls(UINT mode)
    {
     CRect w;
     GetWindowRect(&w);
     GetParent()->ScreenToClient(&w);

     for(ALL_PARENT_CHILDREN(wnd))
        { /* scan child windows */
         if(wnd == this)
             continue;
         CRect child;
         wnd->GetWindowRect(&child);
         GetParent()->ScreenToClient(&child);
         CRect test;
         if(test.IntersectRect(w, child))
            { /* overlaps */
             wnd->ShowWindow(mode);
            } /* overlaps */
        } /* scan child windows */
    } // CExpandingTreeCtrl::ManageOverlappedControls

/****************************************************************************
*                      CExpandingTreeCtrl::OnWhatIsThis
* Result: void
*       
* Effect: 
*       Pops up a descriptive help text
****************************************************************************/

void CExpandingTreeCtrl::OnWhatIsThis() 
   {
    CString s = ::GetCaption(this);
    CHelp dlg;
    dlg.m_HelpName = s;
    dlg.DoModal();
   }

/****************************************************************************
*                      CExpandingTreeCtrl::OnContextMenu
* Inputs:
*       CWnd * pWnd:
*       CPoint point:
* Result: void
*       
* Effect: 
*       Pops up the context menu
****************************************************************************/

void CExpandingTreeCtrl::OnContextMenu(CWnd* pWnd, CPoint point) 
   {
    CMenu menu;

    CString caption = ::GetCaption(this);
    BOOL what = (::FindResource(AfxGetInstanceHandle(), CHelp::MakeHelpName(caption), _T("RTF")) != NULL);

    menu.LoadMenu(IDR_WHATS_THIS);
    menu.EnableMenuItem(IDM_WHAT_IS_THIS, MF_BYCOMMAND | (what ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    CMenu * popup = menu.GetSubMenu(0);

    popup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this); 
   }

/****************************************************************************
*                      CExpandingTreeCtrl::OnDeselectAll
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Deselects. If selected, forces a redraw
****************************************************************************/

LRESULT CExpandingTreeCtrl::OnDeselectAll(WPARAM, LPARAM)
    {
     if(!selected)
        return 0;
     selected = FALSE;
     Invalidate();
     return 0;
    } // CExpandingTreeCtrl::OnDeselectAll

/****************************************************************************
*                        CExpandingTreeCtrl::CtlColor
* Inputs:
*       CDC * pDC:
*       UINT nCltColor:
* Result: HBRUSH
*       NULL or the background brush indicating selection
****************************************************************************/

HBRUSH CExpandingTreeCtrl::CtlColor(CDC* pDC, UINT nCtlColor) 
   {
    if(selected)
       { /* change background */
        LOGBRUSH br;
        HelpBrush.GetLogBrush(&br);
        pDC->SetBkColor(br.lbColor);
        pDC->SetBkMode(TRANSPARENT);
        return (HBRUSH)HelpBrush;
       } /* change background */

    return NULL;
}
