// FontData.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "FontData.h"
#include "ErrorString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFontData

CFontData::CFontData()
{
}

CFontData::~CFontData()
{
}


BEGIN_MESSAGE_MAP(CFontData, CTreeCtrl)
        //{{AFX_MSG_MAP(CFontData)
                // NOTE - the ClassWizard will add and remove mapping macros here.
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFontData message handlers

/****************************************************************************
*                          CFontData::charFormat
* Inputs:
*       CString& s: String reference
*       TCHAR ch: Character to format
* Result: void
*       
* Effect: 
*       Formats the string in hex, decimal and character displays
****************************************************************************/

void CFontData::charFormat(CString & s, TCHAR ch)
    {
     s.Format(_T("0x%02x %d \"%c\""), ch, ch, ch);
    }

/****************************************************************************
*                           CFontData::loadABCWidths
* Inputs:
*       CDC * dc: DC to use for getting widths
*       CString smp: Sample string
*       HTREEITEM wd: Parent node of width
* Result: void
*       
* Effect: 
*       Computes the ABC widths, in floating point if possible, and displays
*       the integer or integer and floating point versions of the widths.
****************************************************************************/

void CFontData::loadABCWidths(CDC & dc, const CString & smp, HTREEITEM wd)
    {     
     HTREEITEM gabc = InsertItem(_T("GetCharABCWidths"), wd);
     CString s;

     for(int i = 0; i < smp.GetLength(); i++)
        { /* add ABC */
         ABC abc;
         ABCFLOAT abcf;
         BOOL flt;
         BOOL fixed;

         fixed = dc.GetCharABCWidths(smp[i], smp[i], &abc);
         flt = dc.GetCharABCWidths(smp[i], smp[i], &abcf);

         charFormat(s, smp[i]);


         HTREEITEM iabc = InsertItem(s, gabc);
         if(flt  && fixed)
            { /* floating */
#define FloatFormat _T("%c %d   %#0.4g")
             s.Format(FloatFormat, _T('A'), abc.abcA, abcf.abcfA);
             InsertItem(s, iabc);

             s.Format(FloatFormat, _T('B'), abc.abcB, abcf.abcfB);
             InsertItem(s, iabc);

             s.Format(FloatFormat, _T('C'), abc.abcC, abcf.abcfC);
             InsertItem(s, iabc);
            } /* floating */
         else
         if(fixed)
            { /* fixed only */
             s.Format(_T("A %d"), abc.abcA);
             InsertItem(s, iabc);

             s.Format(_T("B %d"), abc.abcB);
             InsertItem(s, iabc);

             s.Format(_T("C %d"), abc.abcC);
             InsertItem(s, iabc);
            } /* fixed only */
         else
         if(flt)
             { /* float only */
#define FloatOnly _T("%c %#0.4g")
              s.Format(FloatOnly, _T('A'), abcf.abcfA);
              InsertItem(s, iabc);
              s.Format(FloatOnly, _T('B'), abcf.abcfB);
              InsertItem(s, iabc);
              s.Format(FloatOnly, _T('C'), abcf.abcfC);
              InsertItem(s, iabc);
             } /* float only */
        } /* add ABC */
     
    }

/****************************************************************************
*                        CFontData::loadWidths
* Result: void
*       
* Effect: 
*       Adds the ABCWidths info to the dropdown list
****************************************************************************/

void CFontData::loadWidths(const CString & smp)
    {
     int i;
     CClientDC dc(this);

     DWORD FontLanguageInfo = ::GetFontLanguageInfo(dc);

     CString s;
     s.LoadString(IDS_WIDTHS);
     HTREEITEM wd = InsertItem(s);

     { /* GetFontLanguageInfo */
      CString s;
      s = _T("GetFontLanguageInfo");
      HTREEITEM flx = InsertItem(s, wd);
#define GFIflag(x) {if(FontLanguageInfo & (x)) { InsertItem(_T(#x), flx); }}
      GFIflag(GCP_DBCS);
      GFIflag(GCP_DIACRITIC);
      GFIflag(FLI_GLYPHS);
      GFIflag(GCP_GLYPHSHAPE);
      GFIflag(GCP_KASHIDA);
      GFIflag(GCP_LIGATE);
      GFIflag(GCP_USEKERNING);
      GFIflag(GCP_REORDER);
     } /* GetFontLanguageInfo */

     { /* GetTextExtentPoint32 */
      s.Format(_T("GetTextExtentPoint32"));
      HTREEITEM te = InsertItem(s, wd);

      SIZE sz = dc.GetTextExtent(smp);
      s.Format(_T("cx %d"), sz.cx);
      InsertItem(s, te);
      s.Format(_T("cy %d"), sz.cy);
      InsertItem(s, te);

      for(i = 0; i < smp.GetLength(); i++)
         { /* add each */
          charFormat(s, smp[i]);
          HTREEITEM charwd = InsertItem(s, te);
     
          // Note the following clumsy construction because &smp[i]
          // gets a compilation error:
          TCHAR c[2];
          c[0] = smp[i];
          c[1] = _T('\0');
          sz = dc.GetTextExtent(c, 1);

          s.Format(_T("cx %d"), sz.cx);
          InsertItem(s, charwd);
          s.Format(_T("cy %d"), sz.cy);
          InsertItem(s, charwd);
         } /* add each */
     } /* GetTextExtentPoint32 */

     { /* GetTextExtentExPoinrt */
      CSize sz;
      s.Format(_T("GetTextExtentExPoint"));
      HTREEITEM txp = InsertItem(s, wd);

      CArray<int, int> psw;

      psw.SetSize(smp.GetLength());
     
      BOOL ok = ::GetTextExtentExPoint((HDC)dc, smp, smp.GetLength(),
                                       0, NULL, // no length limit
                                       psw.GetData(),
                                       &sz);
      s.Format(_T("lpSize"));
      HTREEITEM xp = InsertItem(s, txp);
      s.Format(_T("cx %d"), sz.cx);
      InsertItem(s, xp);
      s.Format(_T("cy %d"), sz.cy);
      InsertItem(s, xp);

      s.LoadString(IDS_PARTIAL_STRING_WIDTHS);
      HTREEITEM px = InsertItem(s, txp);
      for(int i = 0; i < smp.GetLength(); i++)
         { /* lengths */
          // [%d] '%c' %d
          CString f;
          charFormat(f, smp[i]);
          s.Format(_T("[%d] %s %d"), i, f, psw[i]);
          InsertItem(s, px);
         } /* lengths */
     } /* GetTextExtentExPoinrt */

     { /* GetCharacterPlacement */
      int i;
      GCP_RESULTS gcp = {sizeof(GCP_RESULTS) };
      gcp.lpOutString = new TCHAR[smp.GetLength() + 1];
      gcp.lpOrder = new UINT[smp.GetLength()];
      gcp.lpDx = new int[smp.GetLength()];
      gcp.lpCaretPos = new int[smp.GetLength()];
      gcp.lpClass = new char[smp.GetLength()];
      ::ZeroMemory(gcp.lpClass, smp.GetLength() * sizeof(char));  // must be zero
      gcp.lpGlyphs = new WCHAR[smp.GetLength()];

      DWORD flags = 0;
      //flags |= (FontLanguageInfo & GCP_DIACRITIC);
      flags |= (FontLanguageInfo & GCP_REORDER);
      
      CString s = _T("GetCharacterPlacement");
      HTREEITEM gcx = InsertItem(s, wd);

      DWORD result = ::GetCharacterPlacement((HDC)dc, smp, smp.GetLength(), 0, &gcp, flags);

      if(result == 0)
         { /* failed */
          DWORD err = ::GetLastError();
          if(err != 0)
             InsertItem(ErrorString(err), gcx);
          else
             { /* failed for known reason */
              CString s;
              s.LoadString(IDS_GCP_FAILED);
              InsertItem(s, gcx);
             } /* failed for known reason */
         } /* failed */
      else
         { /* GetCharacterPlacement succeeded */
          CString t;
          t.LoadString(IDS_WIDTH);
          s.Format(_T("%s %d"), t, LOWORD(result));
          InsertItem(s, gcx);
          t.LoadString(IDS_HEIGHT);
          s.Format(_T("%s %d"), t, HIWORD(result));
          InsertItem(s, gcx);
          s = _T("GCP_RESULTS");

          HTREEITEM gcpr = InsertItem(s, gcx);
          //----------------------------------------------------------------
          // lStructSize
          //----------------------------------------------------------------
          s.Format(_T("lStructSize %d"), gcp.lStructSize);
          InsertItem(s, gcpr);

          //----------------------------------------------------------------
          // lpOutString
          //----------------------------------------------------------------
          HTREEITEM lpostr = InsertItem(_T("lpOutString"), gcpr);
          for(i = 0; i < smp.GetLength(); i++)
             { /* lpOutString */
              CString f;
              charFormat(f, smp[i]);
              s.Format(_T("[%d] %s"), i, f);
              InsertItem(s, lpostr);
             } /* lpOutString */

          //----------------------------------------------------------------
          // lpOrder
          //----------------------------------------------------------------
          HTREEITEM lpox = InsertItem(_T("lpOrder"), gcpr);
          for(i = 0; i < smp.GetLength(); i++)
             { /* show reorder */
              s.Format(_T("[%d] %d"), i, gcp.lpOrder[i]);
              InsertItem(s, lpox);
             } /* show reorder */

          //----------------------------------------------------------------
          // lpDx
          //----------------------------------------------------------------

          HTREEITEM lpdx = InsertItem(_T("lpDx"), gcpr);
          for(i = 0; i < smp.GetLength(); i++)
             { /* lpDX */
              s.Format(_T("[%d] %d"), i, gcp.lpDx[i]);
              InsertItem(s, lpdx);
             } /* lpDX */

          //----------------------------------------------------------------
          // lpCaretPos
          //----------------------------------------------------------------
          HTREEITEM lpcp = InsertItem(_T("lpCaretPos"), gcpr);
          for(i = 0; i < smp.GetLength(); i++)
             { /* lpCaretPos */
              s.Format(_T("[%d] %d"), i, gcp.lpCaretPos[i]);
              InsertItem(s, lpcp);
             } /* lpCaretPos */

          //----------------------------------------------------------------
          // lpClass
          //----------------------------------------------------------------
          HTREEITEM lpcx = InsertItem(_T("lpClass"), gcpr);
          for(i = 0; i < smp.GetLength(); i++)
             { /* lpClass */
              CString c;
              UINT f = gcp.lpClass[i];
              CString sep = _T("");
    #define GCP1(x) if(f & (x)) { c = _T(#x); f &= ~(x); sep = _T(" | "); }
              GCP1(GCPCLASS_ARABIC);
              GCP1(GCPCLASS_HEBREW);
              GCP1(GCPCLASS_LATIN);
              GCP1(GCPCLASS_LATINNUMBER);
              GCP1(GCPCLASS_LOCALNUMBER);
              if(f != 0)
                 { /* leftovers */
                  c += sep;
                  CString t;
                  t.Format(_T("0x%02x"), f);
                  c += t;
                 } /* leftovers */
             } /* lpClass */
          //----------------------------------------------------------------
          // lpGlyphs/nGlyphs
          //----------------------------------------------------------------
          s.Format(_T("lpGlyphs [%d]"), gcp.nGlyphs);
          HTREEITEM lpgx = InsertItem(s, gcpr);
          for(i = 0; i < (int)gcp.nGlyphs; i++)
             { /* lpGlyphs */
              s.Format(_T("[%d] %d 0x%02x"), i, gcp.lpGlyphs[i], gcp.lpGlyphs[i]);
              InsertItem(s, lpgx);
             } /* lpGlyphs */

          //----------------------------------------------------------------
          // nMaxFit
          //----------------------------------------------------------------
          s.Format(_T("nMaxFit %d"), gcp.nMaxFit);
          InsertItem(s, gcpr);
         } /* GetCharacterPlacement succeeded */

      delete [] gcp.lpOutString;
      delete [] gcp.lpOrder;
      delete [] gcp.lpDx;
      delete [] gcp.lpCaretPos;
      delete [] gcp.lpClass;
     } /* GetCharacterPlacement */
     loadABCWidths(dc, smp, wd);
    }


/****************************************************************************
*                               CFontData::Fill
* Inputs:
*       const CString & s: String to display
* Result: void
*       
* Effect: 
*       Displays character information in the control
****************************************************************************/

void CFontData::Fill(const CString & s)
    {
     DeleteAllItems();
     loadWidths(s);
    } // CFontData::Fill
