// FontMap.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "FontMap.h"
#include "lcids.h"
#include "Registry.h"
#include "SpecialFontDialog.h"
#include "fontname.h"
#include "strsafe.h"
#include "RegVars.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/****************************************************************************************
Manages the fontmap

HKEY_CURRENT_USER
    |
    +--Software
          |
          +--LocaleExplorer
                  |
                  +---FontMap
                         |
                         +--- 0x0409
                         |       |
                         |       +--- Font REG_SZ "Arial"
                         |       |
                         |       +--- Adjust REG_DWORD +2
                         |
                         +--- 0x0411
                         |       |
                         |       +--- Font REG_SZ "WP Japanese"
                         |       |
                         |       +--- Adjust REG_DWORD 0
                         |
                         +--- 0x040d  (comment: Hebrew)
                         |       |
                         |       +--- RTL REG_DWORD  0=LTR 1=RTL
                         :
****************************************************************************************/
/////////////////////////////////////////////////////////////////////////////
// CFontMap dialog


CFontMap::CFontMap(CWnd* pParent /*=NULL*/)
        : CDialog(CFontMap::IDD, pParent)
{
        //{{AFX_DATA_INIT(CFontMap)
                // NOTE: the ClassWizard will add member initialization here
        //}}AFX_DATA_INIT
}


void CFontMap::DoDataExchange(CDataExchange* pDX)
{
        CDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CFontMap)
        DDX_Control(pDX, IDC_RTL_DEFAULT, c_LTR_DEFAULT);
        DDX_Control(pDX, IDC_LTR, c_LTR);
        DDX_Control(pDX, IDC_BROWSEFONT, c_BrowseFont);
        DDX_Control(pDX, IDC_RTL, c_RTL);
        DDX_Control(pDX, IDC_LCIDHEX, c_HexLCID);
        DDX_Control(pDX, IDC_REMOVE, c_Remove);
        DDX_Control(pDX, IDC_ADD, c_Add);
        DDX_Control(pDX, IDOK, c_OK);
        DDX_Control(pDX, IDC_SPINSIZE, c_SpinSize);
        DDX_Control(pDX, IDC_SIZEADJUST, c_Size);
        DDX_Control(pDX, IDC_LCID, c_LCID);
        DDX_Control(pDX, IDC_FONTNAME, c_FontName);
        DDX_Control(pDX, IDC_FONTMAP, c_FontMap);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFontMap, CDialog)
        //{{AFX_MSG_MAP(CFontMap)
        ON_BN_CLICKED(IDC_BROWSEFONT, OnBrowsefont)
        ON_EN_CHANGE(IDC_FONTNAME, OnChangeFontname)
        ON_EN_CHANGE(IDC_SIZE, OnChangeSize)
        ON_CBN_SELENDOK(IDC_LCID, OnSelendokLcid)
        ON_BN_CLICKED(IDC_ADD, OnAdd)
        ON_BN_CLICKED(IDC_REMOVE, OnRemove)
        ON_NOTIFY(NM_DBLCLK, IDC_FONTMAP, OnDblclkFontmap)
        ON_BN_CLICKED(IDC_RTL_DEFAULT, OnRtlDefault)
        ON_BN_CLICKED(IDC_RTL, OnRtl)
        ON_BN_CLICKED(IDC_LTR, OnLtr)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFontMap message handlers

BOOL CFontMap::OnInitDialog() 
   {
    CDialog::OnInitDialog();
        
    c_LCID.AddStrings(LCIDs);
    c_LCID.Select(0x0000); // language neutral value

    c_SpinSize.SetRange32(-4, 4);
    c_SpinSize.SetPos32(0);
    
    CheckRadioButton(IDC_RTL, IDC_RTL_DEFAULT, IDC_RTL_DEFAULT);
    //-----------------------------------------------------------------------------
    // Load the registry keys
    //-----------------------------------------------------------------------------
    CStringArray keys;
    EnumRegistryKeys(HKEY_CURRENT_USER, IDS_REGISTRY_FONTMAP, keys);
    for(int i = 0; i < keys.GetSize(); i++)
       { /* load each key */
        RegFontInfo * info = new RegFontInfo;
        // Key names are of the form 0x0000
        if(!c_FontMap.QueryFontInfo(keys[i], *info))
           { /* load failed */
            ASSERT(FALSE); // should not happen. Put error stuff in here
            delete info;
            continue;
           } /* load failed */
        c_FontMap.AddFontInfo(info);
       } /* load each key */
    //-----------------------------------------------------------------------------
    updateControls();
    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
   }

/****************************************************************************
*                           CFontMap::OnBrowseFont
* Result: void
*       
* Effect: 
*       Allows selection of a new font
****************************************************************************/

void CFontMap::OnBrowsefont() 
   {
    LOGFONT lf = {0};

    CFont * stdfont = GetFont();
    stdfont->GetLogFont(&lf);

    int basesize = abs(lf.lfHeight);
    
    int delta = c_SpinSize.GetPos32();
    if(lf.lfHeight < 0)
       lf.lfHeight -= delta;
    else
       lf.lfHeight += delta;

#ifdef _UNICODE
    RegistryString defunicodefont(IDS_REGISTRY_DEFAULT_FONT);
    defunicodefont.load(DEFAULT_UNICODE_FONT_NAME);
    StringCchCopy(lf.lfFaceName, LF_FACESIZE, defunicodefont.value);
#endif
    CString fname;
    c_FontName.GetWindowText(fname);
    fname.TrimLeft();
    fname.TrimRight();
    if(!fname.IsEmpty())
       { /* copy it */
        StringCchCopy(lf.lfFaceName, LF_FACESIZE, fname);
       } /* copy it */

    CSpecialFontDialog dlg(&lf, CF_SCREENFONTS);
    if(dlg.DoModal() != IDOK)
       return;

    CString s = dlg.GetFaceName();
    c_FontName.SetWindowText(s);

    int size = dlg.GetSize() / 10; // size is in tenths of pints
    int newsize = abs(size) - abs(basesize);
    c_SpinSize.SetPos32(newsize);
   }

/****************************************************************************
*                         CFontMap::OnChangeFontName
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void CFontMap::OnChangeFontname() 
   {
    // TODO: Add your control notification handler code here
   }

/****************************************************************************
*                           CFontMap::OnChangeSize
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void CFontMap::OnChangeSize() 
   {
    // TODO: Add your control notification handler code here
   }

/****************************************************************************
*                          CFontMap::updateControls
* Result: void
*       
* Effect: 
*       Updates the controls
****************************************************************************/

void CFontMap::updateControls()
    {
     //-----------------------------------------------------------------------------
     // LCID display
     //-----------------------------------------------------------------------------
     int sel = c_LCID.GetCurSel();
     if(sel != CB_ERR)
        { /* get font id */
         LCID lcid = (LCID)c_LCID.GetItemData(sel);
         CString s;
         s.Format(_T("0x%04x"), lcid);
         c_HexLCID.SetWindowText(s);
        } /* get font id */

     //-----------------------------------------------------------------------------
     // Remove button
     //-----------------------------------------------------------------------------
     int n = c_FontMap.GetSelectedCount();
     c_Remove.EnableWindow(n > 0);

     //-----------------------------------------------------------------------------
     // Add button
     //-----------------------------------------------------------------------------
     BOOL enable = TRUE;
     enable &= sel != CB_ERR;
     c_Add.EnableWindow(enable);
     
    } // CFontMap::updateControls

/****************************************************************************
*                           CFontMap::OnSelenokLcid
* Result: void
*       
* Effect: 
*       Updates the controls to represent the new ID
****************************************************************************/

void CFontMap::OnSelendokLcid() 
   {
    updateControls();
   }

/****************************************************************************
*                               CFontMap::OnAdd
* Result: void
*       
* Effect: 
*       Adds the current selection to the table.
* Notes:
*       If the LCID already exists, the element is marked as "modified" and
*       replaced by the new data
*       If the LCID does not exist, the element is marked as "modified" and
*       a new entry is created
****************************************************************************/

void CFontMap::OnAdd() 
   {
    int langsel = c_LCID.GetCurSel();
    if(langsel == CB_ERR)
       return; // this should never happen!
    LCID lcid = (LCID)c_LCID.GetItemData(langsel);
    
    RegFontInfo * info = c_FontMap.FindFontInfo(lcid);
    if(info == NULL)
       info = new RegFontInfo;

    info->lcid = lcid; // may be redundant if already found
    c_LCID.GetLBText(langsel, info->LanguageName); // commentary! Not stored in Registry

    CString s;
    c_FontName.GetWindowText(s);
    info->fontname = s;

    info->direction = ControlsToDirection();
    info->size = c_SpinSize.GetPos32();

    c_FontMap.AddFontInfo(info);
   } // CFontMap::OnAdd

/****************************************************************************
*                             CFontMap::OnRemove
* Result: void
*       
* Effect: 
*       The currently selected element is removed
****************************************************************************/

void CFontMap::OnRemove() 
   {
    POSITION pos = c_FontMap.GetFirstSelectedItemPosition();
    if(pos == NULL)
       return; // should not have been here!

    while(pos != NULL)
       { /* delete each */
        int item = c_FontMap.GetNextSelectedItem(pos);
        c_FontMap.DeleteItem(item);
       } /* delete each */
    updateControls();
   } // CFontMap::OnRemove

/****************************************************************************
*                          CFontMap::OnDblclkFontmap
* Inputs:
*       NMHDR * pNMHDR:
*       LRESULT * pResult:
* Result: void
*       
* Effect: 
*       Transfers the data to the controls
****************************************************************************/

void CFontMap::OnDblclkFontmap(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    int n = c_FontMap.GetSelectedCount();
    if(n != 1)
       return; // can't handle more than one
        
    POSITION pos = c_FontMap.GetFirstSelectedItemPosition(); // the only item position!
                                                             // See above n != 1!
    int sel = c_FontMap.GetNextSelectedItem(pos);

    RegFontInfo * info = c_FontMap.GetFontInfo(sel);

    // Now set the values
    c_LCID.Select(info->lcid);
    c_FontName.SetWindowText(info->fontname);
    DirectionToControls(info->direction);
    c_SpinSize.SetPos32(info->size);
    updateControls();
    *pResult = 0;
}

/****************************************************************************
*                        CFontMap::DirectionToControls
* Inputs:
*       RegFontInfo::Direction dir: Direction indicator
* Result: void
*       
* Effect: 
*       Puts the direction value into the controls
****************************************************************************/

void CFontMap::DirectionToControls(RegFontInfo::Direction dir)
    {
     UINT id = 0;
     switch((int)dir)
        { /* dir */
         case RegFontInfo::Direction::Unknown:
               id = IDC_RTL_DEFAULT;
               break;
         case RegFontInfo::Direction::LTR:
               id = IDC_LTR;
               break;
         case RegFontInfo::Direction::RTL:
               id = IDC_RTL;
               break;
         default:
            ASSERT(FALSE);
        } /* dir */
     CheckRadioButton(IDC_RTL, IDC_RTL_DEFAULT, id);
    } // CFontMap::DirectionToControls

/****************************************************************************
*                        CFontMap::ControlsToDirection
* Result: Direction
*       Direction code
****************************************************************************/

RegFontInfo::Direction CFontMap::ControlsToDirection()
    {
     if(c_RTL.GetCheck() == BST_CHECKED)
         return RegFontInfo::Direction(RegFontInfo::Direction::RTL);
     if(c_LTR.GetCheck() == BST_CHECKED)
         return RegFontInfo::Direction(RegFontInfo::Direction::LTR);
     return RegFontInfo::Direction(RegFontInfo::Direction::Unknown);;
    } // CFontMap::ControlsToDirection

void CFontMap::OnRtlDefault() 
{
 updateControls();        
}

void CFontMap::OnRtl() 
{
 updateControls();        
}

void CFontMap::OnLtr() 
   {
    updateControls();
   }

/****************************************************************************
*                               CFontMap::OnOK
* Result: void
*       
* Effect: 
*       Stores the elements in the Registry
****************************************************************************/

void CFontMap::OnOK() 
   {
    RegFontInfo::DeleteAllEntries();
    for(int i = 0; i < c_FontMap.GetItemCount(); i++)
       { /* store all */
        RegFontInfo * info = c_FontMap.GetFontInfo(i);
        info->SetRegistryFontInfo();
       } /* store all */
    CDialog::OnOK();
   }
