// FontMapData.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "FontMapData.h"
#include "Registry.h"
#include "lcids.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define COLUMN_LANGUAGE_NAME 1
#define COLUMN_FONT_NAME     2
#define COLUMN_SIZEADJUST    3
#define COLUMN_DIRECTION     4
/////////////////////////////////////////////////////////////////////////////
// CFontMapData

CFontMapData::CFontMapData()
{
}

CFontMapData::~CFontMapData()
{
}


BEGIN_MESSAGE_MAP(CFontMapData, CListCtrl)
        //{{AFX_MSG_MAP(CFontMapData)
        ON_NOTIFY_REFLECT(LVN_DELETEITEM, OnDeleteitem)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFontMapData message handlers

/****************************************************************************
*                          CFontMapData::Initialize
* Result: void
*       
* Effect: 
*       Initializes the columns
****************************************************************************/

void CFontMapData::Initialize()
    {
    //---------------------------------------------------------------+
    //| LCID  | Language name   | Font name       | Size | Direction |
    //---------------------------------------------------------------+
    //|0x040d | Japanese        | WP Japanese     |  +1  |   RTL     |

     CString s;
     s.LoadString(IDS_CAPTION_LCID);
     SetColumn(0, s, CString(_T("0x0000")));

     s.LoadString(IDS_CAPTION_LANGUAGE_NAME);
     SetColumn(COLUMN_LANGUAGE_NAME, s, CString(_T("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")));

     s.LoadString(IDS_CAPTION_FONTNAME);
     SetColumn(COLUMN_FONT_NAME, s, CString(_T("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")));

     s.LoadString(IDS_CAPTION_SIZEADJUST);
     SetColumn(COLUMN_SIZEADJUST, s, CString(_T("+0")));

     CClientDC dc(this);
     s.LoadString(IDS_CAPTION_DIRECTION);
     SetColumn(COLUMN_DIRECTION, s, RegFontInfo::Direction::MaxString(dc));
    } // CFontMapData::Initialize

/****************************************************************************
*                           CFontMapData::SetColumn
* Inputs:
*       int col: Column #
*       const CString & text: Column heading string
*       const CString & widest: (= CString(_T(""))): Widest string
* Result: void
*       
* Effect: 
*       Adds the column
****************************************************************************/

void CFontMapData::SetColumn(int col, const CString & text, const CString & widest)
    {
     CClientDC dc(this);
     int width = max(dc.GetTextExtent(text).cx, dc.GetTextExtent(widest).cx);;
     SetColumn(col, text, width);
    } // CFontMapData::SetColumn

/****************************************************************************
*                           CFontMapData::SetColumn
* Inputs:
*       int col: Column #
*       const CString & text: text string
*       int width: Maximum width of string
* Result: void
*       
* Effect: 
*       Adds the column heading
****************************************************************************/

void CFontMapData::SetColumn(int col, const CString & text, int width)
    {
     int border = 6 * ::GetSystemMetrics(SM_CXBORDER);
     LV_COLUMN data;
     data.cx = width;
     data.cx += border;
     data.pszText = (LPTSTR)(LPCTSTR)text;
     data.fmt = LVCFMT_LEFT;
     data.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
     data.iSubItem = col;
     InsertColumn(col, &data);
    } // CFontMapData::SetColumn

/****************************************************************************
*                         CFontMapData::QueryFontInfo
* Inputs:
*       const CString & key: Key of the form 0xnnnn
*       RegFontInfo & info: Place to put the info
* Result: BOOL
*       TRUE if found
*       FALSE if error (use ::GetLastError to find out why)
* Effect: 
*       Fills int he RegFontInfo structure with data retrieved from the
*       Registry
****************************************************************************/

BOOL CFontMapData::QueryFontInfo(const CString & key, RegFontInfo & info)
    {
     return info.GetRegistryFontInfo(key);
    } // CFontMapData::QueryFontInfo

/****************************************************************************
*                         CFontMapData::QueryFontInfo
* Inputs:
*       LCID lcid: LCID
*       RegFontInfo & info: Place to put the info
* Result: BOOL
*       TRUE if found
*       FALSE if error (use ::GetLastError to find out why)
* Effect: 
*       Fills int he RegFontInfo structure with data retrieved from the
*       Registry
****************************************************************************/

BOOL CFontMapData::QueryFontInfo(LCID lcid, RegFontInfo & info)
    {
     CString s;
     s.Format(_T("0x%04x"), lcid);
     return QueryFontInfo(s, info);
    } // CFontMapData::QueryFontInfo

/****************************************************************************
*                          CFontMapData::AddFontInfo
* Inputs:
*       RegFontInfo * info: Font information
* Result: int
*       Index where it was added, -1 if an error occurred
* Effect: 
*       Adds the font info to the control
****************************************************************************/

int CFontMapData::AddFontInfo(RegFontInfo * info)
    {
     int n = LocateFontInfo(info->lcid);
     BOOL insert = n < 0;
     if(n < 0)
        n = GetItemCount();

     LVITEM item = {0};
     item.mask = LVIF_TEXT | LVIF_PARAM;
     item.iItem = n;
     CString id;
     id.Format(_T("0X%04X"), info->lcid);
     item.lParam = (LPARAM)info;
     item.pszText = (LPTSTR)(LPCTSTR)id;
     if(insert)
        n = InsertItem(&item);
     else
         SetItem(&item);

     if(info->LanguageName.IsEmpty())
        { /* fill it in */
         info->LanguageName = LCIDString(info->lcid);
        } /* fill it in */
     item.mask = LVIF_TEXT;
     item.iSubItem = COLUMN_LANGUAGE_NAME;
     item.pszText = (LPTSTR)(LPCTSTR)info->LanguageName;
     SetItem(&item);

     item.iSubItem = COLUMN_FONT_NAME;
     item.pszText = (LPTSTR)(LPCTSTR)info->fontname;
     SetItem(&item);

     CString sz;
     sz.Format(_T("%d"), info->size);
     item.iSubItem = COLUMN_SIZEADJUST;
     item.pszText = (LPTSTR)(LPCTSTR)sz;
     SetItem(&item);

     CString dir;
     dir = info->direction.toString();
     item.iSubItem = COLUMN_DIRECTION;
     item.pszText = (LPTSTR)(LPCTSTR)dir;
     SetItem(&item);
     
     return n;
    } // CFontMapData::AddFontInfo

/****************************************************************************
*                         CFontMapData::OnDeleteItem
* Inputs:
*       NMHDR * pNMHDR: Notification header
*       LRESULT * pResult: set to 0, always
* Result: void
*       
* Effect: 
*       Deletes the storage associated with the object
****************************************************************************/

void CFontMapData::OnDeleteitem(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
    RegFontInfo * info = (RegFontInfo *)pNMListView->lParam;
    pNMListView->lParam = NULL;
    delete info;
    *pResult = 0;
   }

/****************************************************************************
*                          CFontMapData::LocateInfo
* Inputs:
*       LCID lcid: Desired LCID
* Result: int
*       Existing index of a value, -1 if item not found
* Effect: 
*       
****************************************************************************/

int CFontMapData::LocateFontInfo(LCID lcid)
    {
     int items = GetItemCount();

     for(int i = 0; i < items; i++)
        { /* see if match */
         LVITEM item = {0};
         item.mask = LVIF_PARAM;
         item.iItem = i;
         GetItem(&item);
         RegFontInfo * t = (RegFontInfo *)item.lParam;
         if(t->lcid == lcid)
            { /* found it */
             return i;
            } /* found it */
        } /* see if match */
     return -1;
    } // CFontMapData::LocateInfo

/****************************************************************************
*                         CFontMapData::FindFontInfo
* Inputs:
*       LCID lcid: LCID value
* Result: RegFontInfo *
*       Existing LCID element, or NULL if not found
****************************************************************************/

RegFontInfo * CFontMapData::FindFontInfo(LCID lcid)
    {
     int n = LocateFontInfo(lcid);
     if(n < 0)
        return NULL;

     LVITEM item = {0};
     item.mask = LVIF_PARAM;
     item.iItem = n;
     GetItem(&item);
     return (RegFontInfo *)item.lParam;
    } // CFontMapData::FindFontInfo

/****************************************************************************
*                       CFontMapData::PreSubclassWindow
* Result: void
*       
* Effect: 
*       Initializes the control
****************************************************************************/

void CFontMapData::PreSubclassWindow() 
   {
    ModifyStyleEx(0, LVS_EX_FULLROWSELECT);
    CListCtrl::PreSubclassWindow();

    Initialize();
   }

/****************************************************************************
*                          CFontMapData::GetFontInfo
* Inputs:
*       int n: Index into control
* Result: RegFontInfo * 
*       The font info at that index, or NULL if the index is invalid
****************************************************************************/

RegFontInfo * CFontMapData::GetFontInfo(int n)
    {
     LVITEM item = {0};
     item.mask = LVIF_PARAM;
     item.iItem = n;
     if(!GetItem(&item))
        return NULL;
     return (RegFontInfo *)item.lParam;
    } // CFontMapData::GetFontInfo
