// Function.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "Function.h"
#include "MessagesToChildren.h"
#include "msg.h"
#include "query.h"
#include "fontname.h"
#include "RegFontInfo.h"
#include "lctypemsg.h"
#include "EnablePair.h"
#include "GetCaption.h"
#include "IsButton.h"
#include "IsStatic.h"
#include "IndexInfo.h"
#include "AllChildren.h"
#include "lcids.h"
#include "save.h"
#include "ErrorString.h"
#include "OpenFileEx.h"
#include "BiFile.h"
#include "TabView.h"
#include "RegVars.h"
#include "StrSafe.h"
#include "OpenFileEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

DECLARE_MESSAGE(UWM_REFRESH)
DECLARE_MESSAGE(UWM_INITIALIZE)
DECLARE_MESSAGE(UWM_SET_USER_FONT)
DECLARE_MESSAGE(UWM_SHOW_LOCALE)
DECLARE_MESSAGE(UWM_QUERY_SAVE)
DECLARE_MESSAGE(UWM_SAVE)
//****************************
// Messages received from children
//****************************
DECLARE_MESSAGE(UWM_DESELECT_ALL)

IMPLEMENT_DYNCREATE(CFunction, CDialog)

/////////////////////////////////////////////////////////////////////////////
// CFunction dialog


CFunction::CFunction(UINT idd, CWnd* pParent /*=NULL*/)
        : CDialog(idd, pParent)
   {
           //{{AFX_DATA_INIT(CFunction)
           // NOTE: the ClassWizard will add member initialization here
           //}}AFX_DATA_INIT
    TabParent = NULL;
    ScrollPos = 0;
    initialized = FALSE;
   }


void CFunction::DoDataExchange(CDataExchange* pDX)
{
        CDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CFunction)
                // NOTE: the ClassWizard will add DDX and DDV calls here
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFunction, CDialog)
        ON_REGISTERED_MESSAGE(UWM_INITIALIZE, OnInitialize)
        ON_REGISTERED_MESSAGE(UWM_REFRESH, OnRefresh)
        ON_REGISTERED_MESSAGE(UWM_SET_USER_FONT, OnSetUserFont)
        ON_REGISTERED_MESSAGE(UWM_DESELECT_ALL, OnDeselectAll)
        ON_REGISTERED_MESSAGE(UWM_QUERY_SAVE, OnQuerySave)
        ON_REGISTERED_MESSAGE(UWM_SAVE, OnSave)
        //{{AFX_MSG_MAP(CFunction)
        ON_WM_SHOWWINDOW()
        ON_WM_VSCROLL()
        ON_WM_SIZE()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFunction message handlers

/****************************************************************************
*                           CFunction::OnRefresh
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Updates the code
****************************************************************************/

LRESULT CFunction::OnRefresh(WPARAM, LPARAM)
    {
     Refresh();   // Virtual method
     return 0;
    } // CFunction::OnRefresh

/****************************************************************************
*                           CFunction::OnInitialize
* Inputs:
*       WPARAM: (WPARAM)(CWnd *) messaging parent
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Sets the messaging parent
****************************************************************************/

LRESULT CFunction::OnInitialize(WPARAM wParam, LPARAM)
    {
     ASSERT(wParam != NULL);
     TabParent = (CWnd *)wParam;
     initialized = TRUE;
     Initialize(); // virtual method
     Index();
     return 0;
    } // CFunction::OnInitialize

/****************************************************************************
*                            CFunction::OnSetUserFont
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Computes the size of the user font and applies it
****************************************************************************/

LRESULT CFunction::OnSetUserFont(WPARAM, LPARAM)
    {
#ifdef _UNICODE
     LCID lcid = (LCID)QueryLocale();
     if(lcid == MISSING_LCID)
        return 0;

     CFont * qf = GetFont(); // use dialog font as reference font

     LOGFONT baself;
     qf->GetLogFont(&baself);

     // Determine what the font should be

     
     RegistryFont defunicodefont(IDS_REGISTRY_DEFAULT_FONT);
     defunicodefont.load(NULL);

     LOGFONT lf;
     RegFontInfo info; 
     if(info.GetRegistryFontInfo(lcid))
        { /* has entry */
         int baseadjust = 2; // default adjustment
         CString basename = defunicodefont.value.lfFaceName;
         CString s = info.fontname;
         s.TrimLeft();
         s.TrimRight();
         if(!s.IsEmpty())
            basename = s;
         baseadjust = info.size;
         CFont * f = GetFont();
         f->GetLogFont(&lf);
         StringCchCopy(lf.lfFaceName, LF_FACESIZE, basename);

         // Note that we need to make the font a little larger
         if(lf.lfHeight < 0)
            lf.lfHeight -= baseadjust;
         else
            lf.lfHeight += baseadjust;
        } /* has entry */
     else
        { /* use default font */
         lf = defunicodefont.value;
        } /* use default font */

     if(UnicodeFont.GetSafeHandle() != NULL)
        UnicodeFont.DeleteObject();

     if(lf.lfHeight == 0)
        lf.lfHeight = -12; // random number to avoid weird situations
     
     UnicodeFont.CreateFontIndirect(&lf);

     SetFontToControls(); // virtual method of subclass
#endif
     return 0;
    } // CFunction::OnSetFont

/****************************************************************************
*                          CFunction::GetLocaleName
* Result: CString
*       The locale name based on the locale selection
****************************************************************************/

CString CFunction::GetLocaleName()
    {
     CString localeName;
     LCID lcid = QueryLocale();
     localeName = GetLCIDString(lcid);
     return localeName;
    } // CFunction::GetLocaleName

/****************************************************************************
*                          CFunction::GetOptionFlags
* Inputs:
*       CString & options:
*       DWORD & flags: Flags value
*       const CString & or: "OR" operator in language
* Result: void
*       
* Effect: 
*       Appends the printable form of the options to the string
*       ORs the binary form of the options into the flags value
****************************************************************************/

void CFunction::GetOptionFlags(CString & options, DWORD & flags, const CString & or)
    {
     if(QueryFlag(QUERY_USE_CP_ACP))
        { /* USE_CP_ACP */
         AppendFlags(options, _T("LOCALE_USE_CP_ACP"), or);
         flags |= LOCALE_USE_CP_ACP;
        } /* USE_CP_ACP */

     if(QueryFlag(QUERY_NOUSEROVERRIDE))
        { /* NOUSERVOERRIDE */
         AppendFlags(options, _T("LOCALE_NOUSEROVERRIDE"), or);
         flags |= LOCALE_NOUSEROVERRIDE;
        } /* NOUSERVOERRIDE */
    } // CFunction::GetOptionFlags

/****************************************************************************
*                           CFunction::AppendFlags
* Inputs:
*       CString & flags: Flag value
*       LPCTSTR newflag: Flag to append
*       const CString & or: Or-operator
* Result: void
*       
* Effect: 
*       Appends the newflag to the list of flags
****************************************************************************/

void CFunction::AppendFlags(CString & flags, LPCTSTR newflag, const CString & or)
    {
     if(!flags.IsEmpty())
        { /* append */
         flags += _T(" ") + or + _T(" ");
        } /* append */
     flags += newflag;
    } // CFunction::AppendFlags

/****************************************************************************
*                          CFunction::OnDeselectAll
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Broadcasts the message to all children
****************************************************************************/

LRESULT CFunction::OnDeselectAll(WPARAM, LPARAM)
    {
     SendMessageToDescendants(UWM_DESELECT_ALL);
     return 0;
    } // CFunction::OnDeselectAll

/****************************************************************************
*                           CFunction::OnShowWindow
* Inputs:
*       BOOL bShow: TRUE when being shown, FALSE when being hidden
*       UINT nStatus:
* Result: void
*       
* Effect: 
*       Forces the contents to be (re)computed in case the settings outside
*       the window have changed
****************************************************************************/

void CFunction::OnShowWindow(BOOL bShow, UINT nStatus) 
   {
    CDialog::OnShowWindow(bShow, nStatus);
        
    if(bShow)
       { /* activate */
        if(TabParent != NULL)
           SetVariable(variable);
       } /* activate */
    else
       { /* deactivate */
        variable = QueryVariable(); // do not use GetVariable here!
       } /* deactivate */
    
    if(bShow)
       { /* show it */
        SetScroll();
        if(TabParent != NULL)
           SetVariable(variable);
       } /* show it */

    if(bShow)
       OnSetActive();
    else
       OnKillActive();
    if(bShow)
       { /* show it */
        PostMessage(UWM_REFRESH);
       } /* show it */
   }

/****************************************************************************
*                              CFunction::OnVScroll
* Inputs:
*       UINT nSBCode:
*       UINT nPos:
*       CScrollBar * pScrollBar
* Result: void
*       
* Effect: 
*       Scrolls the dialog
****************************************************************************/

void CFunction::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
   {
    if(pScrollBar != NULL)
       { /* some other control */
        ASSERT(FALSE); // should not be here
        return;
       } /* some other control */

    int delta = 0;
    int MaxPos = OriginalClient.Height() - CurrentHeight;

    int line = 12; // Make this better after we play with it
    CRect client;
    GetClientRect(&client);
    int page = client.Height();
    UINT flags = SIF_POS;

    switch(nSBCode)
       { /* nSBCode */
        case SB_LINEDOWN:
           if(ScrollPos >= MaxPos)
              return;
           delta = min(line, MaxPos - ScrollPos);
           TRACE0("SB_LINEDOWN ");
           break;
        case SB_LINEUP:
           if(ScrollPos <= 0)
              return;
           delta = -min(line, ScrollPos);
           TRACE0("SB_LINEUP ");
           break;
        case SB_PAGEUP:
           if(ScrollPos <= 0)
              return;
           delta = -min(page, ScrollPos);
           TRACE0("SB_PAGEUP ");
           break;
        case SB_PAGEDOWN:
           if(ScrollPos >= MaxPos)
              return;
           delta = min(page, MaxPos - ScrollPos);
           TRACE0("SB_PAGEDOWN ");
           break;
        case SB_TOP:
           break;
        case SB_BOTTOM:
           break;
        case SB_THUMBTRACK:
           TRACE1("SB_THUMBTRACK    nPos=%3d ", nPos);
           delta = (int)nPos - ScrollPos;
           break;
        case SB_THUMBPOSITION:
           TRACE1("SB_THUMBPOSITION nPos=%3d ", nPos);
           delta = (int)nPos - ScrollPos;
           break;
        case SB_ENDSCROLL:
           TRACE0("SB_ENDSCROLL     ");
            //flags |= SIF_PAGE;
           break;
        default:
           return;
       } /* nSBCode */

    //TRACE2("SB %d, delta %d", nSBCode, delta);
    ScrollPos += delta;
    TRACE1(" ScrollPos now %d\n", ScrollPos);

    int ScrollMax = OriginalClient.Height() - 1;

    SetScrollPos(SB_VERT, ScrollPos, TRUE);
    ScrollWindow(0, -delta);
        
    CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
   }

/****************************************************************************
*                              CFunction::OnSize
* Inputs:
*       UINT nType: TYpe of resize
*       int cx: New client width
*       int cy: New client height
* Result: void
*       
* Effect: 
*       Forces a recomputation of the scrollbar
****************************************************************************/

void CFunction::OnSize(UINT nType, int cx, int cy) 
   {
    CDialog::OnSize(nType, cx, cy);
        
    SetScroll();        
}

/****************************************************************************
*                           CFunction::OnInitDialog
* Result: BOOL
*       TRUE, always
* Effect: 
*       Stores the original height
****************************************************************************/

BOOL CFunction::OnInitDialog() 
   {
    CDialog::OnInitDialog();
        
    GetWindowRect(&OriginalWindow);
    GetClientRect(&OriginalClient);
    
    SetScroll();
    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
   }

/****************************************************************************
*                            CFunction::SetScroll
* Result: void
*       
* Effect: 
*       Sets the scroll information based on the current window size
****************************************************************************/

void CFunction::SetScroll()
    {
     CRect c;
     GetClientRect(&c);

     SCROLLINFO si = {sizeof(SCROLLINFO) };
     si.fMask = SIF_ALL;
     si.nMin = 0;
     si.nMax = OriginalClient.Height() - 1;
     si.nPos = ScrollPos;
     si.nPage = c.Height();
     SetScrollInfo(SB_VERT, &si, TRUE);
    } // CFunction::SetScroll

/****************************************************************************
*                          CFunction::GetCaption
* Result: CString
*       The CALID value
* Notes: 
*       If the control is a button, it is the window text
*       If the control is an edit control, it is the text of the preceding
*               label
****************************************************************************/

CString CFunction::GetCaption(UINT id)
    {
     CString s;
     CWnd * wnd = GetDlgItem(id);
     if(wnd == NULL)
         return _T("");
     TCHAR type[MAX_PATH];
     GetClassName(wnd->m_hWnd, type, MAX_PATH);
     CString nm = type;
     nm.MakeUpper();

     if(nm == _T("BUTTON"))
        { /* button */
         wnd->GetWindowText(s);
         s.Replace(_T("&"), _T(""));
        } /* button */
     else
     if(nm == _T("EDIT"))
        { /* edit */
         s = ::GetCaption(wnd);
        } /* edit */
     return s;
    } // CFunction::GetCaption

/****************************************************************************
*                       CFunction::EnablePair
* Inputs:
*       BOOL enable: TRUE to enable, FALSE to disable
* Result: void
*       
* Effect: 
*       Enables/disables a window and its caption
****************************************************************************/

void CFunction::EnablePair(UINT id, BOOL enable)
    {
     CWnd * w = GetDlgItem(id);
     ::EnablePair(w, enable);
    } // CFunction::EnablePair

/****************************************************************************
*                           CFunction::ExpandBottom
* Inputs:
*       CWnd & wnd: Window reference
* Result: void
*       
* Effect: 
*       Expands the control to fill the bottom of the window
****************************************************************************/

void CFunction::ExpandBottom(CWnd & wnd)
    {
     if(wnd.GetSafeHwnd() != NULL)
        { /* has window */
         CRect r;
         GetClientRect(&r);
         CRect w;
         wnd.GetWindowRect(&w);
         ScreenToClient(&w);
         int height = r.Height() - w.top - 2;
#define LIMIT (3 * ::GetSystemMetrics(SM_CYCAPTION))
         if(height < LIMIT)
            height = LIMIT;

         wnd.SetWindowPos(NULL,
                          0, 0,                 // new position
                          r.Width() - w.left,   // new width
                          height,               // new height
                          SWP_NOMOVE | SWP_NOZORDER);
        } /* has window */
    } // CFunction::ExpandBottom

/****************************************************************************
*                           CFunction::ExtendColumn
* Inputs:
*       CListCtrl & wnd: Window to handle 
* Result: void
*       
* Effect: 
*       If the last column is shorter than the window width, extends the rightmost column
****************************************************************************/

void CFunction::ExtendColumn(CListCtrl & wnd)
    {
     if(wnd.GetSafeHwnd() == NULL)
        return;
     
     CHeaderCtrl * header = wnd.GetHeaderCtrl();
     int columns = header->GetItemCount();
     
     int total = 0;
     for(int i = 0; i < columns - 1; i++)
        { /* add width */
         total += wnd.GetColumnWidth(i);
        } /* add width */

     CRect c;
     GetClientRect(&c);

     // total is the first columns-1 column widths
     int last = wnd.GetColumnWidth(i); // i is the last column #

     if(total + last < c.Width())
        { /* widen last */
         int w = c.Width() - total;
         wnd.SetColumnWidth(i, w);
        } /* widen last */
    } // CFunction::ExtendColumn

/****************************************************************************
*                              CFunction::Index
* Result: void
*       
* Effect: 
*       Enumerates all the windows and indexes them
****************************************************************************/

void CFunction::Index()
    {
     for(ALL_CHILDREN(wnd))
        { /* children */
         BOOL ok = FALSE;
         ok |= IsButton(wnd);
         ok |= IsStatic(wnd);
         
         if(ok)
            { /* indexable control */
             if(!IndexOK(wnd)) // page-specific filters: return FALSE to skip indexing
                continue;
             CString s;
             wnd->GetWindowText(s);
             if(s.IsEmpty())
                continue;
             TabParent->SendMessage(UWM_REFERENCE, (WPARAM)new Reference(s, this, wnd));
             if(s[0] == _T('_'))
                { /* reindex */
                 CString prefix = s.SpanIncluding(_T("_"));
                 TabParent->SendMessage(UWM_REFERENCE, (WPARAM)new Reference(s, s.Mid(prefix.GetLength()), this, wnd));
                } /* reindex */
            } /* indexable control */
        } /* children */
    } // CFunction::Index

/****************************************************************************
*                           CFunction::OnQuerySave
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       (LRESULT)(BOOL) indicates if save is possible
* Effect: 
*       Calls the virtual method QuerySave
****************************************************************************/

LRESULT CFunction::OnQuerySave(WPARAM, LPARAM)
    {
     return (LRESULT)QuerySave();
    } // CFunction::OnQuerySave

/****************************************************************************
*                              CFunction::OnSave
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Performs a save. Uses the various virtual methods to handle the
*       details of opening the file
****************************************************************************/

LRESULT CFunction::OnSave(WPARAM, LPARAM)
    {
     if(!QuerySave())
        return 0;  // we shouldn't be here, nothing to save!

     CString name;

#ifdef _UNICODE
     UINT width = 8;
#else
     UINT width = 0; // default option
#endif
     if(!OpenFileEx(TRUE, name, _T(".lst"), width, IDS_LIST_FILTER))
        return 0; // failed

     if(!Save(name, width))
        { /* failed to save */
         DWORD err = ::GetLastError();
         CString fmt;
         fmt.LoadString(IDS_FILE_CREATE_ERROR);
         CString s;
         s.Format(fmt, name, ErrorString(err));
         AfxMessageBox(s, MB_ICONERROR);
        } /* failed to save */
     return 0;
    } // CFunction::OnSave

/****************************************************************************
*                          CFunction::WriteChildren
* Inputs:
*       CBiFile & f: File to write to
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Writes the contents of paired static/edit controls
****************************************************************************/

BOOL CFunction::WriteChildren(CBiFile & f)
    {
     try
        { /* try */
         int width = 0;
         for(CWnd * wnd = GetWindow(GW_CHILD); wnd != NULL; wnd = wnd->GetWindow(GW_HWNDNEXT))
            { /* save each */
             if(wnd->IsKindOf(RUNTIME_CLASS(CEdit)))
                { /* can show it */
                 CString caption = ::GetCaption(wnd);
                 if(caption.GetLength() > width)
                    width = caption.GetLength();
                } /* can show it */
            } /* save each */

         width++; // allow for one space
         for(wnd = GetWindow(GW_CHILD); wnd != NULL; wnd = wnd->GetWindow(GW_HWNDNEXT))
            { /* save each */
             if(wnd->IsKindOf(RUNTIME_CLASS(CEdit)))
                { /* can show it */
                 CString caption = ::GetCaption(wnd);
                 f.WriteString(caption);
                 CString text = CString(_T(' '), width - caption.GetLength());

                 f.WriteString(text);
                 text = _T(" \"");
                 f.WriteString(text);
                 CString contents;
                 wnd->GetWindowText(contents);
                 f.WriteString(contents);
                 text = _T("\"\n");
                 f.WriteString(text);
                } /* can show it */
            } /* save each */
        } /* try */
     catch(CFileException * e)
        { /* failed */
         e->Delete();
         return FALSE;
        } /* failed */
     return TRUE;
    } // CFunction::WriteChildren

/****************************************************************************
*                         CFunction::GetOutputHeader
* Result: CString
*       The current page header
****************************************************************************/

CString CFunction::GetOutputHeader()
    {
     CString s;
     CTabView * parent = (CTabView *)GetParent();
     parent->GetPageTitle(this, s);
     s += _T("\n");
     return s;
    } // CFunction::GetOutputHeader

/****************************************************************************
*                           CFunction::GetVariable
* Result: CString
*       Gets the variable.  If the variable string is empty, returns a
*       predefined string
****************************************************************************/

#define PREDEFINED_DEFAULT_VARIABLE _T("no_variable_name_specified")

CString CFunction::GetVariable()
    {
     CString variable;

     variable = QueryVariable(); // Explicit QueryVariable call

     variable.TrimLeft();
     variable.TrimRight();

     if(variable.IsEmpty())
         variable = PREDEFINED_DEFAULT_VARIABLE;

     return variable;
    } // CFunction::GetVariable
