// GeoMap.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "GeoMap.h"
#include "RegVars.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDC_OVERLAY 10000 // semi-random number
/////////////////////////////////////////////////////////////////////////////
// CGeoMap

CGeoMap::CGeoMap()
   {
    rop2 = R2_COPYPEN; // just in case
    rop2Force = FALSE;  // do not force ROP2
    colorForce = FALSE;  // do not force color
   }

CGeoMap::~CGeoMap()
{
}


BEGIN_MESSAGE_MAP(CGeoMap, CStatic)
        //{{AFX_MSG_MAP(CGeoMap)
        ON_WM_PAINT()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGeoMap message handlers

/****************************************************************************
*                         CGeoMap::PreSubclassWindow
* Result: void
*       
* Effect: 
*       Performs necessary initialization
****************************************************************************/

void CGeoMap::PreSubclassWindow() 
   {
    ClearCoordinates();
    Vertical.SetRectEmpty();
    Horizontal.SetRectEmpty();
    MapRect.SetRectEmpty();
    CStatic::PreSubclassWindow();
   }

/****************************************************************************
*                              CGeoMap::OnPaint
* Result: void
*       
* Effect: 
*       Draws the map
****************************************************************************/

void CGeoMap::OnPaint() 
    {
     CPaintDC dc(this); // device context for painting
        
     HBITMAP b = GetBitmap();
     CBitmap * bmp = CBitmap::FromHandle(b);

     CDC memDC;
     memDC.CreateCompatibleDC(&dc);
     memDC.SelectObject(bmp);
     BITMAP info;
     bmp->GetBitmap(&info);
     dc.BitBlt(0,0, info.bmWidth, info.bmHeight, &memDC, 0, 0, SRCCOPY);

     //=============================================================================
     //  -180                       0                         180
     //  +--------------------------+--------------------------+ 90
     //  |                          |                          |
     //  |                          |                          |
     //  |                          |                          |
     //  +--------------------------+--------------------------+ 0
     //  |                          |                          |
     //  |                          |                          |
     //  |                          |                          |
     //  +--------------------------+--------------------------+ -90
     if(!MapRect.IsRectEmpty() && c_Overlay.GetSafeHwnd() != NULL)
        { /* draw lines */
         int save = dc.SaveDC();

         CClientDC odc(&c_Overlay);

         odc.SetMapMode(MM_ANISOTROPIC);
         odc.SetWindowExt(m_WindowExt.cx, m_WindowExt.cy);
         odc.SetViewportExt(m_ViewportExt.cx, m_ViewportExt.cy);
         odc.SetViewportOrg(m_ViewportOrg.x, m_ViewportOrg.y);
         odc.SetWindowOrg(m_WindowOrg.x, m_WindowOrg.y);
         int x = (int)Longitude;
         int y = (int)Latitude;

         //****************
         // Compute vertical
         //****************
         Vertical.left = x;
         Vertical.right = x;
         Vertical.top = 90;
         Vertical.bottom = -90;
         odc.LPtoDP(&Vertical);
         c_Overlay.ClientToScreen(&Vertical);
         ScreenToClient(&Vertical);

         CSize c(15,15);
         odc.LPtoDP(&c);
         
         //****************
         // Compute horizontal
         //****************
         Horizontal.left = -180;
         Horizontal.right = 180;
         Horizontal.top = y;
         Horizontal.bottom = y;
         odc.LPtoDP(&Horizontal);
         c_Overlay.ClientToScreen(&Horizontal);
         ScreenToClient(&Horizontal);

         //****************
         // Draw the lines
         //****************
         RegistryInt linecolor(IDS_REGISTRY_MAPCOLOR);
         if(!colorForce)
            linecolor.load(RGB(255,255,0));
         else
            linecolor.value = color;
         
         CPen pen(PS_SOLID, 0, linecolor.value);
         dc.SelectObject(&pen);
         RegistryInt op(IDS_REGISTRY_MAPOP);
         if(!rop2Force)
            { /* use Registry value */
             op.load(R2_COPYPEN);
            } /* use Registry value */
         else
            { /* use forced value */
             op.value = rop2;
            } /* use forced value */

         dc.SetROP2(op.value);
     
         dc.MoveTo(Vertical.left, Vertical.top);
         dc.LineTo(Vertical.right, Vertical.bottom);

         dc.MoveTo(Horizontal.left, Horizontal.top);
         dc.LineTo(Horizontal.right, Horizontal.bottom);
     
         dc.SelectStockObject(HOLLOW_BRUSH);

         CRgn rgn;
         BOOL isrgn = rgn.CreateRectRgn(MapRect.left, MapRect.top, MapRect.right, MapRect.bottom);
#ifdef _DEBUG  //----------------------------------------------------------------+
         CRect r;                                                             // |
         BOOL rgntype = rgn.GetRgnBox(&r);                                    // |
         CRect r2;                                                            // |
         BOOL rgntype2 = dc.GetClipBox(&r2);                                  // |
#endif  //-----------------------------------------------------------------------+
         // int clip = dc.SelectClipRgn(&rgn, RGN_OR);
         // Note in the code below, cy appears for both x and y
         // This is because we want a circle in the device coordinates
         // Using cx gives a circle in transformed coordinates, which
         // comes out as an ellipse in device coordinates
         dc.Ellipse(Vertical.left - c.cy, Horizontal.top - c.cy,
                    Vertical.left + c.cy, Horizontal.top + c.cy);
#if 0
#ifdef _DEBUG// ----------------------------------------------------------------+
         dc.MoveTo(x - 30, y + 30);                                          // |
         dc.LineTo(x + 30, y - 30);                                          // |
         dc.MoveTo(x + 30, y + 30);                                          // |
         dc.LineTo(x - 30, y - 30);                                          // |
#endif //-----------------------------------------------------------------------+
#endif
         dc.RestoreDC(save);
        } /* draw lines */
     //=============================================================================
     // Do not call CStatic::OnPaint() for painting messages
    }

/****************************************************************************
*                            CGeoMap::SetLatitude
* Inputs:
*       double d: Desired latitude
* Result: void
*       
* Effect: 
*       Updates the latitude value
****************************************************************************/

void CGeoMap::SetLatitude(double d)
    {
     Latitude = d;
     Redraw();
    } // CGeoMap::SetLatitude

/****************************************************************************
*                            CGeoMap::SetLongitude
* Inputs:
*       double d: Desired new longitude
* Result: void
*       
* Effect: 
*       Sets the longitude
****************************************************************************/

void CGeoMap::SetLongitude(double d)
    {
     Longitude = d;
     Redraw();
    } // CGeoMap::SetLongitude

/****************************************************************************
*                           CGeoMap::SetCoordinates
* Inputs:
*       double lat: latitude
*       double lon: longitude
* Result: void
*       
* Effect: 
*       Sets te lat/long
****************************************************************************/

void CGeoMap::SetCoordinates(double lat, double lon)
   {
    Latitude = lat;
    Longitude = lon;
    Redraw();
   } // CGeoMap::SetCoordinates

/****************************************************************************
*                              CGeoMap::SetColor
* Inputs:
*       COLORREF c: Desired color
*       BOOL force: TRUE to force use of this color; FALSE to use Registry value
* Result: void
*       
* Effect: 
*       Updates the color of the lines
****************************************************************************/

void CGeoMap::SetColor(COLORREF c, BOOL force /* = TRUE */)
    {
     color = c;
     colorForce = force;
     Redraw();
    } // CGeoMap::SetColor

/****************************************************************************
*                               CGeoMap::Redraw
* Result: void
*       
* Effect: 
*       Invalidates the line regtangles
****************************************************************************/

void CGeoMap::Redraw()
    {
     //InvalidateRect(&Vertical);
     //InvalidateRect(&Horizontal);
     Invalidate();
    } // CGeoMap::Redraw

/****************************************************************************
*                          CGeoMap::ClearCoordinates
* Result: void
*       
* Effect: 
*       Sets the coordinates to 0,0
****************************************************************************/

void CGeoMap::ClearCoordinates()
    {
     Latitude = 0.0;
     Longitude = 0.0;
    } // CGeoMap::ClearCoordinates

/****************************************************************************
*                             CGeoMap::SetMapRect
* Inputs:
*       const CRect & r: new rectangle
* Result: void
*        
* Effect: 
*       Sets the map area rectangle
****************************************************************************/

void CGeoMap::SetMapRect(const CRect & r)
    {
     MapRect = r;
     if(c_Overlay.GetSafeHwnd() != NULL)
        { /* discard old */
         c_Overlay.DestroyWindow();
        } /* discard old */
     c_Overlay.CreateEx(WS_EX_TRANSPARENT,
                        _T("STATIC"),
                        NULL,
                        WS_CHILD,
                        r,
                        this,
                        IDC_OVERLAY);
     m_ViewportExt.cx = -r.Width() / 2;
     m_ViewportExt.cy = -r.Height();
     m_ViewportOrg.x = r.Width() / 2;
     m_ViewportOrg.y = r.Height() / 2;
     m_WindowExt.cx = -180;
     m_WindowExt.cy = 180;
     m_WindowOrg.x = 0;
     m_WindowOrg.y = 0;
     Redraw();
    } // CGeoMap::SetMapRect

/****************************************************************************
*                              CGeoMap::SetROP2
* Inputs:
*       UINT rop: Raster op for pen
*       BOOL force: TRUE to always use the rop value; FALSE to rever to Registry value
* Result: void
*       
* Effect: 
*       Sets the rop2 value.  If force is TRUE, this will be the value used to draw
****************************************************************************/

void CGeoMap::SetROP2(UINT rop, BOOL force /* = TRUE */)
    {
     if(force == rop2Force && rop == rop2)
        return; // already set
     rop2Force = force;
     rop2 = rop;
     Redraw();
    } // CGeoMap::SetROP2
