// GetGeoInfo.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "GetGeoInfo.h"
#include "Query.h"
#include "ErrorString.h"
#include "msg.h"
#include "GetGeoInfoData.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CGetGeoInfo, CFunction)
/////////////////////////////////////////////////////////////////////////////
// CGetGeoInfo dialog


CGetGeoInfo::CGetGeoInfo(CWnd* pParent /*=NULL*/)
        : CFunction(CGetGeoInfo::IDD, pParent)
{
        //{{AFX_DATA_INIT(CGetGeoInfo)
        //}}AFX_DATA_INIT
    SelectedGeoType = 0;
}


void CGetGeoInfo::DoDataExchange(CDataExchange* pDX)
{
        CFunction::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CGetGeoInfo)
        DDX_Control(pDX, IDC_GEO_TIMEZONES, c_GEO_TIMEZONES);
        DDX_Control(pDX, IDC_GEO_RFC1766, c_GEO_RFC1766);
        DDX_Control(pDX, IDC_GEO_OFFICIALNAME, c_GEO_OFFICIALNAME);
        DDX_Control(pDX, IDC_GEO_OFFICIALLANGUAGES, c_GEO_OFFICIALLANGUAGES);
        DDX_Control(pDX, IDC_GEO_LONGITUDE, c_GEO_LONGITUDE);
        DDX_Control(pDX, IDC_GEO_LCID, c_GEO_LCID);
        DDX_Control(pDX, IDC_GEO_LATITUDE, c_GEO_LATITUDE);
        DDX_Control(pDX, IDC_GEO_ISO3, c_GEO_ISO3);
        DDX_Control(pDX, IDC_GEO_ISO2, c_GEO_ISO2);
        DDX_Control(pDX, IDC_GEO_NATION, c_GEO_NATION);
        DDX_Control(pDX, IDC_RESULT, c_Result);
        DDX_Control(pDX, IDC_LANGID, c_LangID);
        DDX_Control(pDX, IDC_GEOID, c_GEOID);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGetGeoInfo, CFunction)
        ON_CONTROL_RANGE(BN_CLICKED, IDC_GEO_NATION, IDC_GEO_OFFICIALLANGUAGES, OnGeoType)
        //{{AFX_MSG_MAP(CGetGeoInfo)
        ON_NOTIFY(NM_CLICK, IDC_GEOID, OnClickGeoid)
        ON_NOTIFY(LVN_ITEMCHANGED, IDC_GEOID, OnItemchangedGeoid)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGetGeoInfo message handlers

/****************************************************************************
*                        CGetGeoInfo::EnumGeoInfoProc
* Inputs:
*       GEOID geoid: ID value
* Result: BOOL
*       TRUE, always
* Effect: 
*       Adds element to list of GEIODs
****************************************************************************/
CGetGeoInfo * CGetGeoInfo::LogWnd = NULL;

/* static */ BOOL CALLBACK CGetGeoInfo::EnumGeoInfoProc(GEOID geoid)
    {
     if(LogWnd == NULL)
         return TRUE;

     LCID lcid = (LCID)LogWnd->TabParent->SendMessage(UWM_QUERY, (WPARAM)QUERY_LOCALE);
     LANGID langid = LANGIDFROMLCID(lcid);

     CString gstr;
     gstr.Format(_T("%d"), geoid);

     int n = LogWnd->c_GEOID.GetItemCount();
     LogWnd->c_GEOID.InsertItem(n, gstr);

     CString friendlyname = Get_GEO_FRIENDLYNAME(geoid, langid);
     LogWnd->c_GEOID.SetItemText(n, 1, friendlyname);
     
     return TRUE;
    } // CGetGeoInfo::EnumGeoInfoProc

/****************************************************************************
*                          CGetGeoInfo::OnInitDialog
* Result: void
*       
* Effect: 
*       Initializes the dialog
****************************************************************************/

BOOL CGetGeoInfo::OnInitDialog() 
   {
    CFunction::OnInitDialog();
        
    CClientDC dc(&c_GEOID);
    CString caption;
    int width;
    int total = 0;
    
    caption = _T("GEOID");
    width = dc.GetTextExtent(caption).cx + dc.GetTextExtent(CString(_T("  "))).cx;
    c_GEOID.InsertColumnEx(0, caption, LVCFMT_RIGHT, width);
    total += width;
        
    CRect r;
    c_GEOID.GetClientRect(&r);
    caption = _T("GEO_FRIENDLYNAME");
    width = dc.GetTextExtent(caption).cx;
    c_GEOID.InsertColumnEx(1, caption, LVCFMT_LEFT, r.Width() - total);

    SelectedGeoType = IDC_GEO_NATION;
    CheckRadioButton(IDC_GEO_NATION, IDC_GEO_OFFICIALLANGUAGES, SelectedGeoType);

    return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

/****************************************************************************
*                            CGetGeoInfo::Refresh
* Result: void
*       
* Effect: 
*       Updates whatever needs to be updated
****************************************************************************/

void CGetGeoInfo::Refresh()
    {
     if(!initialized)
        return;
     
     EnableQuery(QUERY_NOUSEROVERRIDE, FALSE);
     EnableQuery(QUERY_USE_CP_ACP, FALSE);

     BOOL candraw = TRUE;
     int count = c_GEOID.GetSelectedCount();
     CString reason;
     
     if(count != 1)
        { /* nothing selected */
         candraw = FALSE;
         reason.LoadString(IDS_NEED_GEOID);
        } /* nothing selected */

     if(!candraw)
        { /* nothing to draw */
         SetCode(reason, FALSE);
         return;
        } /* nothing to draw */

     LCID lcid = QueryLocale();
     CString lcidstr;
     lcidstr.Format(_T("0x%04x"), lcid);

     LANGID langid = LANGIDFROMLCID(lcid);

     CString fmt;
     fmt.LoadString(IDS_LANGID);

     CString langidstr;
     langidstr.Format(_T("0x%04x"), langid);

     CString langidfrom;
     langidfrom.Format(fmt, lcidstr, langidstr);
     c_LangID.SetWindowText(langidfrom);

     //----------------------------------------------------------------
     // %1: The LCID
     //----------------------------------------------------------------
     CString localeName = GetLocaleName();

     //----------------------------------------------------------------
     // %2: The GEOTYPE as a string
     //----------------------------------------------------------------

     CString geotypestr = GetCaption(SelectedGeoType);
     
     //----------------------------------------------------------------
     // %3: GEOID symbolic
     //----------------------------------------------------------------

     CString GeoID; // GEOID
     POSITION pos =  c_GEOID.GetFirstSelectedItemPosition();
     int sel = c_GEOID.GetNextSelectedItem(pos);
     GeoID = c_GEOID.GetItemText(sel, 0);
     
     //----------------------------------------------------------------
     // %4: GEOID as a decimal integer
     //----------------------------------------------------------------

     CString GeoIDDecimal;
     GeoIDDecimal = GeoID;
     GEOID geoidnumeric = _ttoi(GeoID);
     
     //----------------------------------------------------------------
     // %5: The variable name
     //----------------------------------------------------------------
     CString variable;
     //     variable = QueryVariable();
     //     variable.TrimLeft();
     //     variable.TrimRight();
     //     if(variable.IsEmpty())
     //        variable = _T("?");
     variable = GetVariable();

     //----------------------------------------------------------------
     // %6: The GEOTYPE as a decimal integer
     //----------------------------------------------------------------

     GEOTYPE geotype = ControlsToGEOTYPE(SelectedGeoType); // Gets the GEOTYPE

     CString GeoTypeDecimal;
     GeoTypeDecimal.Format(_T("%d"), geotype);

     //----------------------------------------------------------------
     // %7: The OR operator
     //----------------------------------------------------------------
     // Already obtained, above

     //----------------------------------------------------------------
     // %8: The LCID as an integer
     //----------------------------------------------------------------
     CString localeDecimal;
     localeDecimal.Format(_T("%d"), lcid);

     //----------------------------------------------------------------
     // %9: The LANGID as a string
     //----------------------------------------------------------------
     langidstr.Format(_T("LANGIDFROMLCID(%s)"), localeName);

     //----------------------------------------------------------------
     // %10: The LANGID as a decimal integer 
     //----------------------------------------------------------------
     CString langidDecimal;
     langidDecimal.Format(_T("%d"), langid);

     //----------------------------------------------------------------
     // %11: Friendly name
     //----------------------------------------------------------------
     CString friendlyname = Get_GEO_FRIENDLYNAME(geoidnumeric, langid);

     CString or = QueryTemplate(_T("[OR]"));

     //----------------------------------------------------------------
     // Load the template
     //----------------------------------------------------------------

     switch(SelectedGeoType)
        { /* GEOTYPE */
         case IDC_GEO_NATION:
         case IDC_GEO_ISO2:
         case IDC_GEO_ISO3:
         case IDC_GEO_RFC1766:
         case IDC_GEO_LCID:
         case IDC_GEO_FRIENDLYNAME:
         case IDC_GEO_OFFICIALNAME:
         case IDC_GEO_TIMEZONES:
         case IDC_GEO_OFFICIALLANGUAGES:
            fmt = QueryTemplate(_T("[GetGeoInfo:VARIABLE]"));
            break;
         case IDC_GEO_LATITUDE:
         case IDC_GEO_LONGITUDE:
            fmt = QueryTemplate(_T("[GetGeoInfo:DOUBLE]"));
            break;
         default:
            fmt = _T("<<no GEOTYPE selected>>");
            break;
        } /* GEOTYPE */
     //-----------------------------------------------------------------------------
     // Create the actual text from the template
     //-----------------------------------------------------------------------------
     CString text;

     //-----------------------------------------------------------------------------------------------
     //                        %1           %2          %3       %4           %5           %6         %7       %8           %9          %10          %11
     text.FormatMessage(fmt, localeName,  geotypestr, GeoID, GeoIDDecimal, variable, GeoTypeDecimal,  or, localeDecimal, langidstr, langidDecimal, friendlyname);

     //-----------------------------------------------------------------------------------------------
     SetCode(text, !variable.IsEmpty());

#define geocase(x) case x: result = Get_##x(geoidnumeric, langid); break
     CString result;
     switch(geotype)
        { /* geotype */
         geocase(GEO_NATION);
         geocase(GEO_LATITUDE);
         geocase(GEO_LONGITUDE);
         geocase(GEO_ISO2);
         geocase(GEO_ISO3);
         geocase(GEO_RFC1766);
         geocase(GEO_LCID);
         geocase(GEO_FRIENDLYNAME);
         geocase(GEO_OFFICIALNAME);
         geocase(GEO_TIMEZONES);
         geocase(GEO_OFFICIALLANGUAGES);
         default:
            break;
        } /* geotype */
#undef geocase
     c_Result.SetWindowText(result);
    } // CGetGeoInfo::Refresh

/****************************************************************************
*                           CGetGeoInfo::Initialize
* Result: void
*       
* Effect: 
*       Initializes the contents
****************************************************************************/

void CGetGeoInfo::Initialize()
    {
     LogWnd = this;
     if(!EnumSystemGeoID(GEOCLASS_NATION, 0, EnumGeoInfoProc))
        { /* failed */
        } /* failed */
     Refresh();
    } // CGetGeoInfo::Initialize

/****************************************************************************
*                           CGetGeoInfo::OnGeoType
* Inputs:
*       UINT id: Control ID
* Result: void
*       
* Effect: 
*       Stores the selected ID for later use
****************************************************************************/

void CGetGeoInfo::OnGeoType(UINT id)
    {
     SelectedGeoType = id;
     CString variable = GetCaption(id);
     variable.Replace(_T("GEO_"), _T(""));
     variable.MakeLower();
     variable += _T("_data");
     SetVariable(variable);
     Refresh();
    } // CGetGeoInfo::OnGeoType

/****************************************************************************
*                        CGetGeoInfo::ControlsToGEOTYPE
* Inputs:
*       UINT id: Desired control ID
* Result: GEOID
*       The GEOID associated with the control
****************************************************************************/

GEOID CGetGeoInfo::ControlsToGEOTYPE(UINT id)
    {
#define geocase(x) case IDC_##x: return x
     switch(id)
        { /* id */
         geocase(GEO_NATION);
         geocase(GEO_LATITUDE);
         geocase(GEO_LONGITUDE);
         geocase(GEO_ISO2);
         geocase(GEO_ISO3);
         geocase(GEO_RFC1766);
         geocase(GEO_LCID);
         geocase(GEO_FRIENDLYNAME);
         geocase(GEO_OFFICIALNAME);
         geocase(GEO_TIMEZONES);
         geocase(GEO_OFFICIALLANGUAGES);
     default:
         return 0;
        } /* id */
#undef geocase
    } // CGetGeoInfo::ControlsToGEOTYPE

/****************************************************************************
*                          CGetGeoInfo::OnClickGeoid
* Inputs:
*       NMHDR * pNMHDR:
*       LRESULT * pResult:
* Result: void
*       
* Effect: 
*       Causes a refresh in case there was a selection change
****************************************************************************/

void CGetGeoInfo::OnClickGeoid(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    Refresh();
        
    *pResult = 0;
   }

/****************************************************************************
*                       CGetGeoInfo::SetFontToControls
* Result: void
*       
* Effect: 
*       Sets the unicode font to the controls
****************************************************************************/

void CGetGeoInfo::SetFontToControls()
    {
#ifdef _UNICODE
     c_Result.SetFont(&UnicodeFont);
#endif
    } // CGetGeoInfo::SetFontToControls

/****************************************************************************
*                       CGetGeoInfo::OnItemchangedGeoid
* Inputs:
*       NMHDR * pNMHDR: (NMHDR *)(NM_LISTVIEW *)
*       LPRESULT * pResult: set to 0, always
* Result: void
*       
* Effect: 
*       Refreshes the display based on the changed selection
****************************************************************************/

void CGetGeoInfo::OnItemchangedGeoid(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
    Refresh();
        
    *pResult = 0;
   }

/****************************************************************************
*                            CGetGeoInfo::IndexOK
* Inputs:
*       CWnd * wnd: Window to check
* Result: BOOL
*       TRUE to include window in index
*       FALSE to omit window
****************************************************************************/

BOOL CGetGeoInfo::IndexOK(CWnd * wnd)
    {
     if(wnd == &c_Result)
        return FALSE;
     if(wnd == &c_LangID)
        return FALSE;
     return TRUE;
    } // CGetGeoInfo::IndexOK
