#include "stdafx.h"
#include "ErrorString.h"

/****************************************************************************
*                               Get_GEO_NATION
* Inputs:
*       GEOID geoid:
*       LANGID langid:
* Result: CString
*       The nation ID, as a string
****************************************************************************/

CString Get_GEO_NATION(GEOID geoid, LANGID langid)
    {
     CString nation_data;
     int length = ::GetGeoInfo(geoid, GEO_RFC1766, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_NATION */
         LPTSTR p = nation_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_NATION, p, length, langid);
         nation_data.ReleaseBuffer();
        } /* has GEO_NATION */
     else
        { /* error GEO_NATION */
         DWORD err = ::GetLastError();
         nation_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_NATION */
     return nation_data;
    } // Get_GEO_NATION

/****************************************************************************
*                              Get_GEO_LATITUDE
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_LATITUDE string
****************************************************************************/

CString Get_GEO_LATITUDE(GEOID geoid, LANGID langid)
    {
     CString latitude_data;
     int length = ::GetGeoInfo(geoid, GEO_LATITUDE, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_LATITUDE */
         LPTSTR p = latitude_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_LATITUDE, p, length, langid);
         latitude_data.ReleaseBuffer();
        } /* has GEO_LATITUDE */
     else
        { /* error GEO_LATITUDE */
         DWORD err = ::GetLastError();
         latitude_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_LATITUDE */
     return latitude_data;
    } // Get_GEO_LATITUDE

/****************************************************************************
*                              Get_GEO_LONGITUDE
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_LONGITUDE value
****************************************************************************/

CString Get_GEO_LONGITUDE(GEOID geoid, LANGID langid)
    {
     CString longitude_data;
     int length = ::GetGeoInfo(geoid, GEO_LONGITUDE, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_LONGITUDE */
         LPTSTR p = longitude_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_LONGITUDE, p, length, langid);
         longitude_data.ReleaseBuffer();
        } /* has GEO_LONGITUDE */
     else
        { /* error GEO_LONGITUDE */
         DWORD err = ::GetLastError();
         longitude_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_LONGITUDE */
     return longitude_data;
    } // Get_GEO_LONGITUDE

/****************************************************************************
*                                Get_GEO_ISO2
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_ISO2 value as a string
****************************************************************************/

CString Get_GEO_ISO2(GEOID geoid, LANGID langid)
    {
     CString iso2_data;
     int length = ::GetGeoInfo(geoid, GEO_ISO2, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_ISO2 */
         LPTSTR p = iso2_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_ISO2, p, length, langid);
         iso2_data.ReleaseBuffer();
        } /* has GEO_ISO2 */
     else
        { /* error GEO_ISO2 */
         DWORD err = ::GetLastError();
         iso2_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_ISO2 */
     return iso2_data;
    } // Get_GEO_ISO2

/****************************************************************************
*                                Get_GEO_ISO3
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_ISO3 value as a string
****************************************************************************/

CString Get_GEO_ISO3(GEOID geoid, LANGID langid)
    {
     CString iso3_data;
     int length = ::GetGeoInfo(geoid, GEO_ISO3, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_ISO3 */
         LPTSTR p = iso3_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_ISO3, p, length, langid);
         iso3_data.ReleaseBuffer();
        } /* has GEO_ISO3 */
     else
        { /* error GEO_ISO3 */
         DWORD err = ::GetLastError();
         iso3_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_ISO3 */
     return iso3_data;
    } // Get_GEO_ISO3

/****************************************************************************
*                                Get_GEO_RFC1766
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_RFC1766 value as a string
****************************************************************************/

CString Get_GEO_RFC1766(GEOID geoid, LANGID langid)
    {
     CString rfc1766_data;
     int length = ::GetGeoInfo(geoid, GEO_RFC1766, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_RFC1766 */
         LPTSTR p = rfc1766_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_RFC1766, p, length, langid);
         rfc1766_data.ReleaseBuffer();
        } /* has GEO_RFC1766 */
     else
        { /* error GEO_RFC1766 */
         DWORD err = ::GetLastError();
         rfc1766_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_RFC1766 */
     return rfc1766_data;
    } // Get_GEO_RFC1766

/****************************************************************************
*                                Get_GEO_LCID
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_LCID value as a string
****************************************************************************/

CString Get_GEO_LCID(GEOID geoid, LANGID langid)
    {
     CString lcid_data;
     int length = ::GetGeoInfo(geoid, GEO_LCID, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_LCID */
         LPTSTR p = lcid_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_LCID, p, length, langid);
         lcid_data.ReleaseBuffer();
        } /* has GEO_LCID */
     else
        { /* error GEO_LCID */
         DWORD err = ::GetLastError();
         lcid_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_LCID */
     return lcid_data;
    } // Get_GEO_LCID

/****************************************************************************
*                            Get_GEO_FRIENDLYNAME
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_FRIENDLYNAME value
****************************************************************************/

CString Get_GEO_FRIENDLYNAME(GEOID geoid, LANGID langid)
    {
     CString friendlyname_data;
     int length = ::GetGeoInfo(geoid, GEO_FRIENDLYNAME, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_FRIENDLYNAME */
         LPTSTR p = friendlyname_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_FRIENDLYNAME, p, length, langid);
         friendlyname_data.ReleaseBuffer();
        } /* has GEO_FRIENDLYNAME */
     else
        { /* error GEO_FRIENDLYNAME */
         DWORD err = ::GetLastError();
         friendlyname_data = (err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err));
        } /* error GEO_FRIENDLYNAME */
     return friendlyname_data;
    } // Get_GEO_FRIENDLYNAME

/****************************************************************************
*                            Get_GEO_OFFICIALNAME
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_OFFICIALNAME value
****************************************************************************/

CString Get_GEO_OFFICIALNAME(GEOID geoid, LANGID langid)
    {
     CString officialname_data;
     int length = ::GetGeoInfo(geoid, GEO_OFFICIALNAME, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_OFFICIALNAME */
         LPTSTR p = officialname_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_OFFICIALNAME, p, length, langid);
         officialname_data.ReleaseBuffer();
        } /* has GEO_OFFICIALNAME */
     else
        { /* error GEO_OFFICIALNAME */
         DWORD err = ::GetLastError();
         officialname_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_OFFICIALNAME */
     return officialname_data;
    } // Get_GEO_OFFICIALNAME

/****************************************************************************
*                            Get_GEO_TIMEZONES
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_TIMEZONES value
****************************************************************************/

CString Get_GEO_TIMEZONES(GEOID geoid, LANGID langid)
    {
     CString timezones_data;
     int length = ::GetGeoInfo(geoid, GEO_TIMEZONES, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_TIMEZONES */
         LPTSTR p = timezones_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_TIMEZONES, p, length, langid);
         timezones_data.ReleaseBuffer();
        } /* has GEO_TIMEZONES */
     else
        { /* error GEO_TIMEZONES */
         DWORD err = ::GetLastError();
         if(err == ERROR_SUCCESS)
            timezones_data = _T("");
         else
            timezones_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_TIMEZONES */
     return timezones_data;
    } // Get_GEO_TIMEZONES

/****************************************************************************
*                            Get_GEO_OFFICIALLANGUAGES
* Inputs:
*       GEOID geoid:
*       LANGID langid: Language ID
* Result: CString
*       The GEO_OFFICIALLANGUAGES value
****************************************************************************/

CString Get_GEO_OFFICIALLANGUAGES(GEOID geoid, LANGID langid)
    {
     CString officiallanguages_data;
     int length = ::GetGeoInfo(geoid, GEO_OFFICIALLANGUAGES, NULL, 0, langid);
     if(length != 0)
        { /* has GEO_OFFICIALLANGUAGES */
         LPTSTR p = officiallanguages_data.GetBuffer(length);
         length = ::GetGeoInfo(geoid, GEO_OFFICIALLANGUAGES, p, length, langid);
         officiallanguages_data.ReleaseBuffer();
        } /* has GEO_OFFICIALLANGUAGES */
     else
        { /* error GEO_OFFICIALLANGUAGES */
         DWORD err = ::GetLastError();
         if(err == ERROR_SUCCESS)
            officiallanguages_data = _T("");
         else
            officiallanguages_data = err == ERROR_INVALID_PARAMETER ? _T("?") : ErrorString(err);
        } /* error GEO_OFFICIALLANGUAGES */
     return officiallanguages_data;
    } // Get_GEO_OFFICIALLANGUAGES
