// GetLocaleInfo.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "GetLocaleInfo.h"
#include "Query.h"
#include "FontMapData.h"
#include "RegVars.h"
#include "ErrorString.h"
#include "messagesToChildren.h"
#include "msg.h"
#include "lctypemsg.h"
#include "RTLMap.h"
#include "IsEditControl.h"
#include "AllChildren.h"
#include "GetCaption.h"
#include "BiFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//****************************
// Messages received from parent
//****************************
DECLARE_MESSAGE(UWM_SELECT_LCTYPE)
DECLARE_MESSAGE(UWM_SHOW_LOCALE)


//****************************
// Messages sent to parent
//****************************

//****************************
// Messages sent to children
//****************************
DECLARE_MESSAGE(UWM_DESELECT_ALL)


IMPLEMENT_DYNCREATE(CGetLocaleInfo, CFunction)

class GETLOCALEINFO_TABLE {
    public:
       LCTYPE lctype;
       UINT length;
       BOOL CanReturnNumber;
};

#define StringLength(n) ( (n) + 1)
const static GETLOCALEINFO_TABLE localeTable[] = {
    {LOCALE_ILANGUAGE,              StringLength(5), TRUE},  // language id
    {LOCALE_SLANGUAGE,              0,               FALSE}, // localized name of language (?)
    {LOCALE_SENGLANGUAGE,           0,               FALSE}, // English name of language   (?)
    {LOCALE_SABBREVLANGNAME,        StringLength(3), FALSE}, // abbreviated language name
    {LOCALE_SNATIVELANGNAME,        0,               FALSE}, // native name of language (?)
    {LOCALE_ICOUNTRY,               StringLength(6), TRUE},  // country code
    {LOCALE_SCOUNTRY,               0,               FALSE}, // localized name of country (?)
    {LOCALE_SENGCOUNTRY,            0,               FALSE}, // English name of country (?)
    {LOCALE_SABBREVCTRYNAME,        0,               FALSE}, // abbreviated country name (?)
    {LOCALE_SNATIVECTRYNAME,        0,               FALSE}, // native name of country
    {LOCALE_IDEFAULTLANGUAGE,       StringLength(5), TRUE},  // default language id
    {LOCALE_IDEFAULTCOUNTRY,        StringLength(6), TRUE},  // default country code
    {LOCALE_IDEFAULTCODEPAGE,       StringLength(6), TRUE},  // default oem code page
    {LOCALE_IDEFAULTANSICODEPAGE,   StringLength(6), TRUE},  // default ansi code page
    {LOCALE_IDEFAULTMACCODEPAGE,    StringLength(6), TRUE},  // default mac code page
    {LOCALE_SLIST,                  StringLength(4), FALSE}, // list item separator
    {LOCALE_IMEASURE,               StringLength(2), TRUE},  // 0 = metric, 1 = US
    {LOCALE_SDECIMAL,               StringLength(4), FALSE}, // decimal separator
    {LOCALE_STHOUSAND,              StringLength(4), FALSE}, // thousand separator
    {LOCALE_SGROUPING,              0,               FALSE}, // digit grouping (?)
    {LOCALE_IDIGITS,                StringLength(2), TRUE},  // number of fractional digits
    {LOCALE_ILZERO,                 StringLength(2), TRUE},  // leading zeros for decimal
    {LOCALE_INEGNUMBER,             StringLength(2), TRUE},  // negative number mode
    {LOCALE_SNATIVEDIGITS,          0,               FALSE}, // native ascii 0-9 (?)
    {LOCALE_SCURRENCY,              StringLength(6), FALSE}, // local monetary symbol
    {LOCALE_SINTLSYMBOL,            0,               FALSE}, // intl monetary symbol
    {LOCALE_SMONDECIMALSEP,         StringLength(4), FALSE}, // monetary decimal separator
    {LOCALE_SMONTHOUSANDSEP,        StringLength(4), FALSE}, // monetary thousand separator
    {LOCALE_SMONGROUPING,           0,               FALSE}, // monetary grouping (specified length of 4 has to be wrong!)
    {LOCALE_ICURRDIGITS,            StringLength(3), TRUE},  // # local monetary digits
    {LOCALE_IINTLCURRDIGITS,        StringLength(3), TRUE},  // # intl monetary digits
    {LOCALE_ICURRENCY,              StringLength(2), TRUE},  // positive currency mode
    {LOCALE_INEGCURR,               StringLength(2), TRUE},  // negative currency mode
    {LOCALE_SDATE,                  StringLength(4), FALSE},  // date separator
    {LOCALE_STIME,                  StringLength(4), FALSE},  // time separator
    {LOCALE_SSHORTDATE,             StringLength(80), FALSE}, // short date format string
    {LOCALE_SLONGDATE,              StringLength(80), FALSE}, // long date format string
    {LOCALE_STIMEFORMAT,            StringLength(80), FALSE},   // time format string
    {LOCALE_IDATE,                  StringLength(2), TRUE},   // short date format ordering
    {LOCALE_ILDATE,                 StringLength(2), TRUE},   // long date format ordering
//   LOCALE_ITIME,   // time format specifier
    {LOCALE_ITIMEMARKPOSN,          0,                TRUE},   // time marker position (?)
    {LOCALE_ICENTURY,               StringLength(2),  TRUE},   // century format specifier (short date)
    {LOCALE_ITLZERO,                StringLength(2),  TRUE},   // leading zeros in time field
    {LOCALE_IDAYLZERO,              StringLength(2),  TRUE},   // leading zeros in day field (short date)
    {LOCALE_IMONLZERO,              StringLength(2),  TRUE},   // leading zeros in month field (short date)
    {LOCALE_S1159,                  StringLength(9),  FALSE},  // AM designator
    {LOCALE_S2359,                  StringLength(9),  FALSE},  // PM designator
    {LOCALE_ICALENDARTYPE,          0,                TRUE}, // type of calendar specifier (?)
    {LOCALE_IOPTIONALCALENDAR,      StringLength(2),  TRUE}, // additional calendar types specifier
    {LOCALE_IFIRSTDAYOFWEEK,        StringLength(2),  TRUE}, // first day of week specifier
    {LOCALE_IFIRSTWEEKOFYEAR,       StringLength(2),  TRUE}, // first week of year specifier
    {LOCALE_SDAYNAME1,              0,                FALSE},  // long name for Monday (?)
    {LOCALE_SDAYNAME2,              0,                FALSE},  // long name for Tuesday (?)
    {LOCALE_SDAYNAME3,              0,                FALSE},  // long name for Wednesday (?)
    {LOCALE_SDAYNAME4,              0,                FALSE},  // long name for Thursday (?)
    {LOCALE_SDAYNAME5,              0,                FALSE},  // long name for Friday (?)
    {LOCALE_SDAYNAME6,              0,                FALSE},  // long name for Saturday (?)
    {LOCALE_SDAYNAME7,              0,                FALSE},  // long name for Sunday (?)
    {LOCALE_SABBREVDAYNAME1,        0,                FALSE},   // abbreviated name for Monday
    {LOCALE_SABBREVDAYNAME2,        0,                FALSE},   // abbreviated name for Tuesday
    {LOCALE_SABBREVDAYNAME3,        0,                FALSE},   // abbreviated name for Wednesday
    {LOCALE_SABBREVDAYNAME4,        0,                FALSE},   // abbreviated name for Thursday
    {LOCALE_SABBREVDAYNAME5,        0,                FALSE},   // abbreviated name for Friday
    {LOCALE_SABBREVDAYNAME6,        0,                FALSE},   // abbreviated name for Saturday
    {LOCALE_SABBREVDAYNAME7,        0,                FALSE},  // abbreviated name for Sunday
    {LOCALE_SMONTHNAME1,            0,                FALSE},  // long name for January (?)
    {LOCALE_SMONTHNAME2,            0,                FALSE},  // long name for February (?)
    {LOCALE_SMONTHNAME3,            0,                FALSE},  // long name for March (?)
    {LOCALE_SMONTHNAME4,            0,                FALSE},  // long name for April (?)
    {LOCALE_SMONTHNAME5,            0,                FALSE},  // long name for May (?)
    {LOCALE_SMONTHNAME6,            0,                FALSE},  // long name for June (?)
    {LOCALE_SMONTHNAME7,            0,                FALSE},  // long name for July (?)
    {LOCALE_SMONTHNAME8,            0,                FALSE},  // long name for August (?)
    {LOCALE_SMONTHNAME9,            0,                FALSE},  // long name for September (?)
    {LOCALE_SMONTHNAME10,           0,                FALSE},  // long name for October (?)
    {LOCALE_SMONTHNAME11,           0,                FALSE},  // long name for November (?)
    {LOCALE_SMONTHNAME12,           0,                FALSE},  // long name for December (?)
    {LOCALE_SMONTHNAME13,           0,                FALSE},  // long name for 13th month (if exists) (?)
    {LOCALE_SABBREVMONTHNAME1,      0,                FALSE},  // abbreviated name for January (?)
    {LOCALE_SABBREVMONTHNAME2,      0,                FALSE},  // abbreviated name for February (?)
    {LOCALE_SABBREVMONTHNAME3,      0,                FALSE},  // abbreviated name for March (?)
    {LOCALE_SABBREVMONTHNAME4,      0,                FALSE},  // abbreviated name for April (?)
    {LOCALE_SABBREVMONTHNAME5,      0,                FALSE},  // abbreviated name for May (?)
    {LOCALE_SABBREVMONTHNAME6,      0,                FALSE},  // abbreviated name for June (?)
    {LOCALE_SABBREVMONTHNAME7,      0,                FALSE},  // abbreviated name for July (?)
    {LOCALE_SABBREVMONTHNAME8,      0,                FALSE},  // abbreviated name for August (?)
    {LOCALE_SABBREVMONTHNAME9,      0,                FALSE},  // abbreviated name for September (?)
    {LOCALE_SABBREVMONTHNAME10,     0,                FALSE},  // abbreviated name for October (?)
    {LOCALE_SABBREVMONTHNAME11,     0,                FALSE},  // abbreviated name for November (?)
    {LOCALE_SABBREVMONTHNAME12,     0,                FALSE},  // abbreviated name for December (?)
    {LOCALE_SABBREVMONTHNAME13,     0,                FALSE},  // abbreviated name for 13th month (if exists)
    {LOCALE_SPOSITIVESIGN,           StringLength(5), FALSE},  // positive sign
    {LOCALE_SNEGATIVESIGN,           StringLength(5), FALSE}, // negative sign
    {LOCALE_IPOSSIGNPOSN,            StringLength(2), TRUE},  // positive sign position
    {LOCALE_INEGSIGNPOSN,            StringLength(2), TRUE},  // negative sign position
    {LOCALE_IPOSSYMPRECEDES,         StringLength(2), TRUE},  // mon sym precedes pos amt
    {LOCALE_IPOSSEPBYSPACE,          StringLength(2), TRUE},  // mon sym sep by space from pos amt
    {LOCALE_INEGSYMPRECEDES,         StringLength(2), TRUE},  // mon sym precedes neg amt
    {LOCALE_INEGSEPBYSPACE,          StringLength(2), TRUE},  // mon sym sep by space from neg amt
    {LOCALE_FONTSIGNATURE,           sizeof(LOCALESIGNATURE), FALSE}, // font signature
    {LOCALE_SISO639LANGNAME,         0,               FALSE}, // ISO abbreviated language name
    {LOCALE_SISO3166CTRYNAME,        0,               FALSE}, // ISO abbreviated country name
    {LOCALE_IDEFAULTEBCDICCODEPAGE, StringLength(6),  TRUE},  // default ebcdic code page
    {LOCALE_IPAPERSIZE,             StringLength(2),  TRUE},  // 0 = letter, 1 = a4, 2 = legal, 3 = a3  (?)
    {LOCALE_SENGCURRNAME,           StringLength(40), TRUE},  // english name of currency (?)
    {LOCALE_SNATIVECURRNAME,        0,                FALSE}, // native name of currency
    {LOCALE_SYEARMONTH,             0,                FALSE}, // year month format string (?)
    {LOCALE_SSORTNAME,              0,                FALSE},  // sort name (?)
    {LOCALE_IDIGITSUBSTITUTION,     StringLength(2),  TRUE},  // 0 = none, 1 = context, 2 = native digit (?)
    0 // EOT
};

/////////////////////////////////////////////////////////////////////////////
// CGetLocaleInfo dialog


CGetLocaleInfo::CGetLocaleInfo(CWnd* pParent /*=NULL*/)
        : CFunction(CGetLocaleInfo::IDD, pParent)
   {
    //{{AFX_DATA_INIT(CGetLocaleInfo)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    ItemSelection = 0; // no item selected
   }


void CGetLocaleInfo::DoDataExchange(CDataExchange* pDX)
{
        CFunction::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CGetLocaleInfo)
        DDX_Control(pDX, LOCALE_ILANGUAGE, c_LOCALE_ILANGUAGE);
        DDX_Control(pDX, LOCALE_SLANGUAGE, c_LOCALE_SLANGUAGE);
        DDX_Control(pDX, LOCALE_SENGLANGUAGE, c_LOCALE_SENGLANGUAGE);
        DDX_Control(pDX, LOCALE_SABBREVLANGNAME, c_LOCALE_SABBREVLANGNAME);
        DDX_Control(pDX, LOCALE_SNATIVELANGNAME, c_LOCALE_SNATIVELANGNAME);
        DDX_Control(pDX, LOCALE_ICOUNTRY, c_LOCALE_ICOUNTRY);
        DDX_Control(pDX, LOCALE_SCOUNTRY, c_LOCALE_SCOUNTRY);
        DDX_Control(pDX, LOCALE_SENGCOUNTRY, c_LOCALE_SENGCOUNTRY);
        DDX_Control(pDX, LOCALE_SABBREVCTRYNAME, c_LOCALE_SABBREVCTRYNAME);
        DDX_Control(pDX, LOCALE_SNATIVECTRYNAME, c_LOCALE_SNATIVECTRYNAME);
        DDX_Control(pDX, LOCALE_IDEFAULTLANGUAGE, c_LOCALE_IDEFAULTLANGUAGE);
        DDX_Control(pDX, LOCALE_IDEFAULTCOUNTRY, c_LOCALE_IDEFAULTCOUNTRY);
        DDX_Control(pDX, LOCALE_IDEFAULTCODEPAGE, c_LOCALE_IDEFAULTCODEPAGE);
        DDX_Control(pDX, LOCALE_IDEFAULTANSICODEPAGE, c_LOCALE_IDEFAULTANSICODEPAGE);
        DDX_Control(pDX, LOCALE_IDEFAULTMACCODEPAGE, c_LOCALE_IDEFAULTMACCODEPAGE);
        DDX_Control(pDX, LOCALE_SLIST, c_LOCALE_SLIST);
        DDX_Control(pDX, LOCALE_IMEASURE, c_LOCALE_IMEASURE);
        DDX_Control(pDX, LOCALE_SDECIMAL, c_LOCALE_SDECIMAL);
        DDX_Control(pDX, LOCALE_STHOUSAND, c_LOCALE_STHOUSAND);
        DDX_Control(pDX, LOCALE_SGROUPING, c_LOCALE_SGROUPING);
        DDX_Control(pDX, LOCALE_IDIGITS, c_LOCALE_IDIGITS);
        DDX_Control(pDX, LOCALE_ILZERO, c_LOCALE_ILZERO);
        DDX_Control(pDX, LOCALE_INEGNUMBER, c_LOCALE_INEGNUMBER);
        DDX_Control(pDX, LOCALE_SNATIVEDIGITS, c_LOCALE_SNATIVEDIGITS);
        DDX_Control(pDX, LOCALE_SCURRENCY, c_LOCALE_SCURRENCY);
        DDX_Control(pDX, LOCALE_SINTLSYMBOL, c_LOCALE_SINTLSYMBOL);
        DDX_Control(pDX, LOCALE_SMONDECIMALSEP, c_LOCALE_SMONDECIMALSEP);
        DDX_Control(pDX, LOCALE_SMONTHOUSANDSEP, c_LOCALE_SMONTHOUSANDSEP);
        DDX_Control(pDX, LOCALE_SMONGROUPING, c_LOCALE_SMONGROUPING);
        DDX_Control(pDX, LOCALE_ICURRDIGITS, c_LOCALE_ICURRDIGITS);
        DDX_Control(pDX, LOCALE_IINTLCURRDIGITS, c_LOCALE_IINTLCURRDIGITS);
        DDX_Control(pDX, LOCALE_ICURRENCY, c_LOCALE_ICURRENCY);
        DDX_Control(pDX, LOCALE_INEGCURR, c_LOCALE_INEGCURR);
        DDX_Control(pDX, LOCALE_SDATE, c_LOCALE_SDATE);
        DDX_Control(pDX, LOCALE_STIME, c_LOCALE_STIME);
        DDX_Control(pDX, LOCALE_SSHORTDATE, c_LOCALE_SSHORTDATE);
        DDX_Control(pDX, LOCALE_SLONGDATE, c_LOCALE_SLONGDATE);
        DDX_Control(pDX, LOCALE_STIMEFORMAT, c_LOCALE_STIMEFORMAT);
        DDX_Control(pDX, LOCALE_IDATE, c_LOCALE_IDATE);
        DDX_Control(pDX, LOCALE_ILDATE, c_LOCALE_ILDATE);
        DDX_Control(pDX, LOCALE_ITIMEMARKPOSN, c_LOCALE_ITIMEMARKPOSN);
        DDX_Control(pDX, LOCALE_ICENTURY, c_LOCALE_ICENTURY);
        DDX_Control(pDX, LOCALE_ITLZERO, c_LOCALE_ITLZERO);
        DDX_Control(pDX, LOCALE_IDAYLZERO, c_LOCALE_IDAYLZERO);
        DDX_Control(pDX, LOCALE_IMONLZERO, c_LOCALE_IMONLZERO);
        DDX_Control(pDX, LOCALE_S1159, c_LOCALE_S1159);
        DDX_Control(pDX, LOCALE_S2359, c_LOCALE_S2359);
        DDX_Control(pDX, LOCALE_ICALENDARTYPE, c_LOCALE_ICALENDARTYPE);
        DDX_Control(pDX, LOCALE_IOPTIONALCALENDAR, c_LOCALE_IOPTIONALCALENDAR);
        DDX_Control(pDX, LOCALE_IFIRSTDAYOFWEEK, c_LOCALE_IFIRSTDAYOFWEEK);
        DDX_Control(pDX, LOCALE_IFIRSTWEEKOFYEAR, c_LOCALE_IFIRSTWEEKOFYEAR);
        DDX_Control(pDX, LOCALE_SDAYNAME1, c_LOCALE_SDAYNAME1);
        DDX_Control(pDX, LOCALE_SDAYNAME2, c_LOCALE_SDAYNAME2);
        DDX_Control(pDX, LOCALE_SDAYNAME3, c_LOCALE_SDAYNAME3);
        DDX_Control(pDX, LOCALE_SDAYNAME4, c_LOCALE_SDAYNAME4);
        DDX_Control(pDX, LOCALE_SDAYNAME5, c_LOCALE_SDAYNAME5);
        DDX_Control(pDX, LOCALE_SDAYNAME6, c_LOCALE_SDAYNAME6);
        DDX_Control(pDX, LOCALE_SDAYNAME7, c_LOCALE_SDAYNAME7);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME1, c_LOCALE_SABBREVDAYNAME1);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME2, c_LOCALE_SABBREVDAYNAME2);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME3, c_LOCALE_SABBREVDAYNAME3);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME4, c_LOCALE_SABBREVDAYNAME4);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME5, c_LOCALE_SABBREVDAYNAME5);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME6, c_LOCALE_SABBREVDAYNAME6);
        DDX_Control(pDX, LOCALE_SABBREVDAYNAME7, c_LOCALE_SABBREVDAYNAME7);
        DDX_Control(pDX, LOCALE_SMONTHNAME1, c_LOCALE_SMONTHNAME1);
        DDX_Control(pDX, LOCALE_SMONTHNAME2, c_LOCALE_SMONTHNAME2);
        DDX_Control(pDX, LOCALE_SMONTHNAME3, c_LOCALE_SMONTHNAME3);
        DDX_Control(pDX, LOCALE_SMONTHNAME4, c_LOCALE_SMONTHNAME4);
        DDX_Control(pDX, LOCALE_SMONTHNAME5, c_LOCALE_SMONTHNAME5);
        DDX_Control(pDX, LOCALE_SMONTHNAME6, c_LOCALE_SMONTHNAME6);
        DDX_Control(pDX, LOCALE_SMONTHNAME7, c_LOCALE_SMONTHNAME7);
        DDX_Control(pDX, LOCALE_SMONTHNAME8, c_LOCALE_SMONTHNAME8);
        DDX_Control(pDX, LOCALE_SMONTHNAME9, c_LOCALE_SMONTHNAME9);
        DDX_Control(pDX, LOCALE_SMONTHNAME10, c_LOCALE_SMONTHNAME10);
        DDX_Control(pDX, LOCALE_SMONTHNAME11, c_LOCALE_SMONTHNAME11);
        DDX_Control(pDX, LOCALE_SMONTHNAME12, c_LOCALE_SMONTHNAME12);
        DDX_Control(pDX, LOCALE_SMONTHNAME13, c_LOCALE_SMONTHNAME13);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME1, c_LOCALE_SABBREVMONTHNAME1);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME2, c_LOCALE_SABBREVMONTHNAME2);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME3, c_LOCALE_SABBREVMONTHNAME3);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME4, c_LOCALE_SABBREVMONTHNAME4);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME5, c_LOCALE_SABBREVMONTHNAME5);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME6, c_LOCALE_SABBREVMONTHNAME6);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME7, c_LOCALE_SABBREVMONTHNAME7);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME8, c_LOCALE_SABBREVMONTHNAME8);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME9, c_LOCALE_SABBREVMONTHNAME9);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME10, c_LOCALE_SABBREVMONTHNAME10);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME11, c_LOCALE_SABBREVMONTHNAME11);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME12, c_LOCALE_SABBREVMONTHNAME12);
        DDX_Control(pDX, LOCALE_SABBREVMONTHNAME13, c_LOCALE_SABBREVMONTHNAME13);
        DDX_Control(pDX, LOCALE_SPOSITIVESIGN, c_LOCALE_SPOSITIVESIGN);
        DDX_Control(pDX, LOCALE_SNEGATIVESIGN, c_LOCALE_SNEGATIVESIGN);
        DDX_Control(pDX, LOCALE_IPOSSIGNPOSN, c_LOCALE_IPOSSIGNPOSN);
        DDX_Control(pDX, LOCALE_INEGSIGNPOSN, c_LOCALE_INEGSIGNPOSN);
        DDX_Control(pDX, LOCALE_IPOSSYMPRECEDES, c_LOCALE_IPOSSYMPRECEDES);
        DDX_Control(pDX, LOCALE_IPOSSEPBYSPACE, c_LOCALE_IPOSSEPBYSPACE);
        DDX_Control(pDX, LOCALE_INEGSYMPRECEDES, c_LOCALE_INEGSYMPRECEDES);
        DDX_Control(pDX, LOCALE_INEGSEPBYSPACE, c_LOCALE_INEGSEPBYSPACE);
        DDX_Control(pDX, LOCALE_SISO639LANGNAME, c_LOCALE_SISO639LANGNAME);
        DDX_Control(pDX, LOCALE_SISO3166CTRYNAME, c_LOCALE_SISO3166CTRYNAME);
        DDX_Control(pDX, LOCALE_IDEFAULTEBCDICCODEPAGE, c_LOCALE_IDEFAULTEBCDICCODEPAGE);
        DDX_Control(pDX, LOCALE_IPAPERSIZE, c_LOCALE_IPAPERSIZE);
        DDX_Control(pDX, LOCALE_SENGCURRNAME, c_LOCALE_SENGCURRNAME);
        DDX_Control(pDX, LOCALE_SNATIVECURRNAME, c_LOCALE_SNATIVECURRNAME);
        DDX_Control(pDX, LOCALE_SYEARMONTH, c_LOCALE_SYEARMONTH);
        DDX_Control(pDX, LOCALE_SSORTNAME, c_LOCALE_SSORTNAME);
        DDX_Control(pDX, LOCALE_IDIGITSUBSTITUTION, c_LOCALE_IDIGITSUBSTITUTION);
        DDX_Control(pDX, IDC_FONTSIGNATURE_CAPTION, c_FontSignatureCaption);
        DDX_Control(pDX, LOCALE_FONTSIGNATURE, c_LOCALE_FONTSIGNATURE);
        DDX_Control(pDX, IDC_AUTOEXPAND, c_AutoExpand);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGetLocaleInfo, CFunction)
        ON_REGISTERED_MESSAGE(UWM_SELECT_LCTYPE, OnSelectLCType)
        ON_BN_CLICKED(IDC_AUTOEXPAND, OnAutoexpand)
        //{{AFX_MSG_MAP(CGetLocaleInfo)
        ON_WM_LBUTTONDOWN()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGetLocaleInfo message handlers

BOOL CGetLocaleInfo::OnInitDialog() 
   {
    CFunction::OnInitDialog();
        
    RegistryInt autoexpand(IDS_REGISTRY_AUTOEXPAND);
    autoexpand.load();

    c_AutoExpand.SetCheck(c_LOCALE_FONTSIGNATURE.GetAutoExpand() ? BST_CHECKED : BST_UNCHECKED);

    c_LOCALE_FONTSIGNATURE.SetAutoExpand(autoexpand.value);
    c_AutoExpand.SetCheck(autoexpand.value ? BST_CHECKED : BST_UNCHECKED);

    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
   } // CGetLocaleInfo::OnInitDialog

/****************************************************************************
*                         CGetLocaleInfo::UpdateCode
* Result: void
*       
* Effect: 
*       Updates the code
****************************************************************************/

void CGetLocaleInfo::UpdateCode()
    {
     EnableQuery(QUERY_NOUSEROVERRIDE, TRUE);
     EnableQuery(QUERY_USE_CP_ACP, TRUE);
     if(ItemSelection == 0)
        { /* no selection */
         SetCode(_T(""), FALSE);
         EnableQuery(QUERY_DWORD, FALSE);
         return;
        } /* no selection */

     // Because of how we have laid out the windows, we know that the
     // caption window immediately precedes the window whose ID we have


     // %1:    the LCID as symbolic or hex
     // %2:    the LCTYPE name
     // %3:    the LCTYPE options (see below)
     // %4:    the fixed length (as a character string)
     // %5:    the variable name
     // %6:    The LCTYPE value, with flags, as a decimal integer (for languages like VB)
     // %7:    The bitwise OR operator
     // %8:    The LCID as a decimal number
     // %9:    not used for GetLocaleInfo
     //%10:    not used for GetLocaleInfo
     //%11:    not used for GetLocaleInfo
     // Template
     //
     //----------------------------------------------------------------
     // The LCID (%1) is one of
     //         LOCALE_SYSTEM_DEFAULT
     //         LOCALE_USER_DEFAULT
     //         0xnnnn
     //--------------------------------
     // The LCTYPESTRING is formed by ORing in various fields to the locale
     //         | LOCALE_NOUSEROVERRIDE
     //         | LOCALE_USE_CP_ACP
     //
     //
     //----------------------------------------------------------------
     //     Lang    Length  DWORD ok?       DWORD?  Format
     //----------------------------------------------------------------
     // A.  C       >0      No              n/a     IDS_FORMAT_C_FIXED
     // B.  C       0       No              n/a     IDS_FORMAT_C_VARIABLE
     // C.  C       >0      Yes             No      IDS_FORMAT_C_FIXED
     // D.  C       0       Yes             No      IDS_FORMAT_C_VARIABLE
     // E.  C       n/a     Yes             Yes     IDS_FORMAT_C_DWORD
     //----------------------------------------------------------------
     // F.  CPP     >0      No              n/a     IDS_FORMAT_CPP_FIXED
     // G.  CPP     0       No              n/a     IDS_FORMAT_CPP_VARIABLE
     // H.  CPP     >0      Yes             No      IDS_FORMAT_CPP_FIXED
     // I.  CPP     0       Yes             No      IDS_FORMAT_CPP_VARIABLE
     // J.  CPP     n/a     Yes             Yes     IDS_FORMAT_CPP_DWORD
     //----------------------------------------------------------------
     // K.  MFC     >0      No              n/a     IDS_FORMAT_MFC_FIXED
     // L.  MFC     0       No              n/a     IDS_FORMAT_MFC_VARIABLE
     // M.  MFC     >0      Yes             No      IDS_FORMAT_MFC_FIXED
     // N.  MFC     0       Yes             No      IDS_FORMAT_MFC_VARIABLE
     // O.  MFC     n/a     Yes             Yes     IDS_FORMAT_MFC_DWORD
     //----------------------------------------------------------------
     // P.  STL     >0      No              n/a     IDS_FORMAT_STL_FIXED
     // Q.  STL     0       No              n/a     IDS_FORMAT_STL_VARIABLE
     // R.  STL     >0      Yes             No      IDS_FORMAT_STL_FIXED
     // S.  STL     0       Yes             No      IDS_FORMAT_STL_VARIABLE
     // T.  STL     n/a     Yes             Yes     IDS_FORMAT_STL_DWORD
     //----------------------------------------------------------------
#define CODECASE(Len, dwok, dw) MAKELONG(MAKEWORD(0, Len), MAKEWORD(dwok, dw))

     LPCGETLOCALEINFO_TABLE localeInfo;
     localeInfo = FindLocaleInfo(ItemSelection);

     typedef enum {LengthUnknown, LengthKnown} LengthType;

     LengthType len = localeInfo->length == 0 ? LengthUnknown : LengthKnown; 
     BOOL dwok = localeInfo->CanReturnNumber;
     BOOL dw = QueryDWORD();

     EnableQuery(QUERY_DWORD, localeInfo->CanReturnNumber);

     CString or  = QueryTemplate(_T("[OR]"));

     CString fmt;
     DWORD dword_flag = 0; // 0 or LOCALE_RETURN_NUMBER

     switch(CODECASE(len, dwok, dw))
        { /* CODECASE */
         case CODECASE(LengthKnown,  FALSE, FALSE):
         case CODECASE(LengthKnown,  FALSE, TRUE):
            // A.
         case CODECASE(LengthKnown, TRUE, FALSE):
            // C.
            fmt = QueryTemplate(_T("[GetLocaleInfo:FIXED]"));
            break;
         case CODECASE(LengthUnknown, FALSE, FALSE):
         case CODECASE(LengthUnknown, FALSE, TRUE):
            // B.
         case CODECASE(LengthUnknown, TRUE, FALSE):
            // D.
            fmt = QueryTemplate(_T("[GetLocaleInfo:VARIABLE]"));
            break;
         case CODECASE(LengthKnown, TRUE, TRUE):
         case CODECASE(LengthUnknown, TRUE, TRUE):
            // E.
            fmt = QueryTemplate(_T("[GetLocaleInfo:DWORD]"));
            dword_flag = LOCALE_RETURN_NUMBER;
            break;
        } /* CODECASE */

     //-----------------------------------------------------------------------------
     // %1: The locale ID
     //-----------------------------------------------------------------------------
     CString localeName = GetLocaleName();

     //-----------------------------------------------------------------------------
     // %2: The LCTYPE value
     //-----------------------------------------------------------------------------
     CString lctype = GetCaption(ItemSelection);

     //-----------------------------------------------------------------------------     
     // %3: Option flags
     //-----------------------------------------------------------------------------

     CString options;   // %3
     DWORD flags = ItemSelection;
     GetOptionFlags(options, flags, or);
     
     //-----------------------------------------------------------------------------
     // %4: Length
     //-----------------------------------------------------------------------------

     CString length;
     if(ItemSelection == LOCALE_FONTSIGNATURE)
        length.Format(_T("sizeof(LOCALESIGNATURE) /*%d*/"), localeInfo->length);
     else
        length.Format(_T("%d"), localeInfo->length);

     if(QueryDWORD() && localeInfo->CanReturnNumber)
        length.Format(_T("%d"), sizeof(DWORD));

     //-----------------------------------------------------------------------------
     // %5: The name of the variable
     //-----------------------------------------------------------------------------
     CString variable;  // %5
     variable = GetVariable();
     
     //-----------------------------------------------------------------------------     
     // %6: The LCTYPE as a decimal integer
     //-----------------------------------------------------------------------------
     CString lcstr;
     lcstr.Format(_T("%d"), flags | dword_flag);

     //-----------------------------------------------------------------------------
     // %7: The OR operator for bitwise operations
     //-----------------------------------------------------------------------------
     // This was set when the choice was made between C, CPP, MFC, STL or USER

     //-----------------------------------------------------------------------------
     // %8: The locale ID as a decimal integer
     //-----------------------------------------------------------------------------
     LCID lcid = QueryLocale();
     CString localeDecimal;
     localeDecimal.Format(_T("%d"), lcid);
     
     //-----------------------------------------------------------------------------
     // Create the actual text from the template
     //-----------------------------------------------------------------------------
     CString text;

     //-----------------------------------------------------------------------------------------------
     //                        %1           %2      %3       %4       %5      %6    %7     %8          %9      %10     %11
     text.FormatMessage(fmt, localeName, lctype, options, length, variable, lcstr, or, localeDecimal, _T(""), _T(""), _T(""));
     //-----------------------------------------------------------------------------------------------
     SetCode(text, !variable.IsEmpty());
    } // CGetLocaleInfo::UpdateCode

/****************************************************************************
*                     CLocaleExplorerDlg::OnSelectLCType
* Inputs:
*       WPARAM: unused
*       LPARAM: (LPARAM)(GetDlgCtrlID())
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Updates the generated code
****************************************************************************/

LRESULT CGetLocaleInfo::OnSelectLCType(WPARAM, LPARAM lParam)
   {
    if(!initialized)
        return 0;
    if(TabParent == NULL)
        return 0;
    ItemSelection = lParam;
    CString caption;
    caption = GetCaption(lParam);
    CString variable = caption;
    variable.Replace(_T("LOCALE_"), _T(""));
    variable.MakeLower();
    variable += _T("_data");
    
    SetVariable(variable);
    UpdateCode();
    return 0;
   } // CGetLocaleInfo::OnSelectLCType

/****************************************************************************
*                        CGetLocaleInfo::Refresh
* Result: void
*       
* Effect: 
*       Displays the code for GetLocaleInfo
****************************************************************************/

void CGetLocaleInfo::Refresh()
    {   
     if(!initialized)
        return;
     
     DWORD flags = 0;
     flags |= QueryFlag(QUERY_NOUSEROVERRIDE);
     flags |= QueryFlag(QUERY_USE_CP_ACP);

     LCID lcid = QueryLocale();

     for(int i = 0; localeTable[i].lctype != 0; i++)
        { /* show each */
         LPTSTR data;
         CString text;
         int size;
         if(localeTable[i].length == 0)
            { /* need length */
             size = ::GetLocaleInfo(lcid, localeTable[i].lctype, NULL, 0);
             data = new TCHAR[size];
            } /* need length */
         else
            { /* length provided */
             data = new TCHAR[localeTable[i].length];
             size = localeTable[i].length;
            } /* length provided */

         int result = ::GetLocaleInfo(lcid, localeTable[i].lctype, data, size);
         if(result == 0)
            { /* failed */
             DWORD err = ::GetLastError();
             CString msg;
             if(err == ERROR_INVALID_PARAMETER)
                msg.LoadString(IDS_ERROR_VALUE);
             else
                msg = ErrorString(err);

             EnablePair(localeTable[i].lctype, FALSE);
             text = msg;
            } /* failed */
         else
            { /* succeeded */
             text = data;
             EnablePair(localeTable[i].lctype, TRUE);
            } /* succeeded */

         switch(localeTable[i].lctype)
            { /* special handling */
             case LOCALE_FONTSIGNATURE:
                ShowLocaleSignature((PLOCALESIGNATURE)data);
                break;
             default: // anything else is text
#ifdef _UNICODE
                RTLMap(text, lcid);
#endif
                SetDlgItemText(localeTable[i].lctype, text);
                break;
            } /* special handling */
         delete data;

        } /* show each */
     UpdateCode();
    } // CGetLocaleInfo::Refresh

/****************************************************************************
*                     CGetLocaleInfo::ShowLocaleSignature
* Inputs:
*       PLOCALESIGNATURE data: Local signature informatio
* Result: void
*       
* Effect: 
*       Displays the locale font signature information
****************************************************************************/

void CGetLocaleInfo::ShowLocaleSignature(LPLOCALESIGNATURE data)
    {
     c_LOCALE_FONTSIGNATURE.DeleteAllItems();

     HTREEITEM lsusb = c_LOCALE_FONTSIGNATURE.InsertItem(_T("DWORD lsUsb[4]"));
     ShowUnicodeSubsets(data, lsusb);

     HTREEITEM lscsbdefault = c_LOCALE_FONTSIGNATURE.InsertItem(_T("DWORD lsCsbDefault[2]"));
     ShowCodePages(data->lsCsbDefault, lscsbdefault);

     HTREEITEM lscsbsupported = c_LOCALE_FONTSIGNATURE.InsertItem(_T("DWORD lsCsbSupported[2]"));
     ShowCodePages(data->lsCsbSupported, lscsbsupported);
    } // CGetLocaleInfo::ShowLocaleSignature

/****************************************************************************
*                                 class bitarray
****************************************************************************/
// A bit vector is a sequence of values, e.g.,
// 0000 0001 0010 0011 0100 0101 0110 0111 1000 1001 1010 1011 1100 1101 1110 1111
class bitarray {
    public:
       BOOL GetAt(int n) {
          DWORD w = n / (sizeof(DWORD) * 8); // get DWORD offset
          DWORD s = n % (sizeof(DWORD) * 8); // get bit offset
          return (((LPDWORD)this)[w] >> s) & 1;
       }
       BOOL operator[](int n) { return GetAt(n); }
};
typedef bitarray *Pbitarray;


/****************************************************************************
*                   CGetLocaleInfo::ShowUnicodeSubsets
* Inputs:
*       LPLOCALESIGNATURE data: local signature structure
*       HTREEITEM root: Place to attach them
* Result: void
*       
* Effect: 
*       Creates the font signature display
****************************************************************************/

void CGetLocaleInfo::ShowUnicodeSubsets(LPLOCALESIGNATURE data, HTREEITEM root)
    {
     LPBYTE b = (LPBYTE)data;
     CString hex;
     hex.Format(_T("%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x"),
                b[15], b[14], b[13], b[12],
                b[11], b[10], b[9],  b[8],
                b[7],  b[6],  b[5],  b[4],
                b[3],  b[2],  b[1],  b[0]);
     HTREEITEM item = c_LOCALE_FONTSIGNATURE.InsertItem(hex, root);

     bitarray * bits = (Pbitarray)data->lsUsb;
     for(int i = 0; i < 128; i++)
        { /* get each */
         BOOL bit;
         bit = bits->GetAt(i);
         if(bit)
            { /* bit is set */
             CString s;
             s.LoadString(IDS_UNICODE_SUBSET_0 + i);
             CString sbit;
             sbit.Format(_T("%d"), i);
             HTREEITEM subroot = c_LOCALE_FONTSIGNATURE.InsertItem(sbit, root);
             // There are three forms of the contents
             // I.   Reserved
             // II.  xxxx - xxxx\tsomething
             // III. xxxx - xxxx;yyyy - yyyy;zzzz - zzzz\tsomethingx;somethingy;somethingz
             // These are represented as the following
             // I.   +--- Reserved
             // II.  +--- xxxx - xxxx something
             // III. +--- xxxx - xxxx somethingx
             //           yyyy - yyyy somethingy
             //           zzzz - zzzz somethingz
             int tab = s.Find(_T("\t"));
             if(tab < 0)
                { /* I. */
                 c_LOCALE_FONTSIGNATURE.InsertItem(s, subroot);
                 continue;
                } /* I. */
             int semi = s.Find(_T(";"));
             if(semi < 0)
                { /* II. */
                 CString desc = s;
                 desc.Replace(_T("\t"), _T(" "));
                 c_LOCALE_FONTSIGNATURE.InsertItem(desc, subroot);
                 continue;
                } /* II. */

             // III.
             CString ranges;
             CString descriptions;
             ranges = s.Left(tab);
             descriptions =  s.Mid(tab+1);
             // range    xxxx - xxxx;yyyy - yyyy;zzzz - zzzz
             // desc    somethingx;somethingy;somethingz
             while(TRUE)
                { /* scan each */
                 CString range;
                 CString description;
                 int punctuation = ranges.Find(_T(";"));
                 if(punctuation < 0)
                    { /* nothing left */
                     range = ranges;
                     ranges = _T("");
                     description = descriptions;
                     descriptions = _T("");
                    } /* nothing left */
                 else
                    { /* something left */
                     range = ranges.Left(punctuation);
                     ranges = ranges.Mid(punctuation + 1);
                     punctuation = descriptions.Find(_T(";"));
                     description = descriptions.Left(punctuation);
                     descriptions = descriptions.Mid(punctuation + 1);
                    } /* something left */
                 c_LOCALE_FONTSIGNATURE.InsertItem(range + _T(" ") + description, subroot);
                 if(descriptions.IsEmpty() || ranges.IsEmpty())
                    break;
                } /* scan each */
            } /* bit is set */
        } /* get each */
    } // CGetLocaleInfo::ShowUnicodeSubsets

/****************************************************************************
*                      CGetLocaleInfo::ShowCodePages
* Inputs:
*       LPDWORD codepagevector
*       HTREEITEM root: Place to attach them
* Result: void
*       
* Effect: 
*       Adds the code page information in
****************************************************************************/

void CGetLocaleInfo::ShowCodePages(LPDWORD codepagevector, HTREEITEM root)
    {
     LPBYTE b = (LPBYTE)codepagevector;
     CString hex;
     hex.Format(_T("%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x"), b[7], b[6], b[5], b[4], b[3], b[2], b[1], b[0]);
     HTREEITEM item = c_LOCALE_FONTSIGNATURE.InsertItem(hex, root);

     bitarray * bits = (Pbitarray)codepagevector;
     for(int i = 0; i < 64; i++)
        { /* get each */
         BOOL bit;
         bit = bits->GetAt(i);
         if(bit)
            { /* bit is set */
             CString sbit;
             sbit.Format(_T("%d"), i);
             HTREEITEM subroot = c_LOCALE_FONTSIGNATURE.InsertItem(sbit, root);

             CString s;
             s.LoadString(IDS_CODEPAGE_0 + i);
             c_LOCALE_FONTSIGNATURE.InsertItem(s, subroot);
            } /* bit is set */
        } /* get each */
    } // CGetLocaleInfo::ShowCodePages

/****************************************************************************
*                      CGetLocaleInfo::OnAutoexpand
* Effect: 
*       Sets the autoexpand mode
****************************************************************************/

void CGetLocaleInfo::OnAutoexpand() 
   {
    BOOL expand = c_AutoExpand.GetCheck() == BST_CHECKED;
    c_LOCALE_FONTSIGNATURE.SetAutoExpand(expand);
    RegistryInt autoexpand(IDS_REGISTRY_AUTOEXPAND);
    autoexpand.value = expand;
    autoexpand.store();
   }

/****************************************************************************
*                      CGetLocaleInfo::OnLButtonDown
* Inputs:
*       UINT nFlags:
*       CPoint point:
* Result: void
*       
* Effect: 
*       This is done to intercept any mouse click outside the possibly-dropped-down
*       font information box and cause it to lose the focus, thus folding back
*       up
****************************************************************************/

void CGetLocaleInfo::OnLButtonDown(UINT nFlags, CPoint point) 
   {
    for(ALL_CHILDREN(wnd))
       { /* find first edit */
        if(IsEditControl(wnd))
           wnd->SetFocus();
       } /* find first edit */
    SendMessageToDescendants(UWM_DESELECT_ALL);
    CFunction::OnLButtonDown(nFlags, point);
   }

/****************************************************************************
*                     CLocaleExplorerDlg::FindLocaleInfo
* Inputs:
*       LCTYPE item: Item to search for
* Result: LPLOCALE_EXPLORER_DATA
*       Pointer to table entry of match, or NULL if not found
****************************************************************************/

LPCGETLOCALEINFO_TABLE CGetLocaleInfo::FindLocaleInfo(LCTYPE item)
    {
     for(int i = 0; localeTable[i].lctype != 0; i++)
        { /* search */
         if(localeTable[i].lctype == item)
            return &localeTable[i];
        } /* search */
     return NULL;
    } // CGetLocaleInfo::FindLocaleInfo

/****************************************************************************
*                      CGetLocaleInfo::EnableDisplay
* Inputs:
*       BOOL enable: TRUE to enable, FALSE to disable
* Result: void
*       
* Effect: 
*       For all controls in the table, enables them and their caption
****************************************************************************/

void CGetLocaleInfo::EnableDisplay(BOOL enable)
    {
     for(int i = 0; localeTable[i].lctype != 0; i++)
        { /* load controls */
         CWnd * w = GetDlgItem(localeTable[i].lctype);
         if(w != NULL)
            w->EnableWindow(enable);
         
        } /* load controls */
    } // CGetLocaleInfo::EnableDisplay

/****************************************************************************
*                      CGetLocaleInfo::SetFontToControls
* Result: void
*       
* Effect: 
*       Given the UnicodeFont has been set in the superclass, this virtual
*       method sets the font in all the controls
****************************************************************************/

void CGetLocaleInfo::SetFontToControls()
    {
#ifdef _UNICODE
     for(int i = 0; localeTable[i].lctype != 0; i++)
        { /* load controls */
         if(localeTable[i].lctype == LOCALE_FONTSIGNATURE)
            continue;
         CWnd * w = GetDlgItem(localeTable[i].lctype);
         if(w != NULL)
            w->SetFont(&UnicodeFont);
        } /* load controls */
#endif
    } // CGetLocaleInfo::SetFontToControls


/****************************************************************************
*                          CGetLocaleInfo::QuerySave
* Result: BOOL
*       TRUE, always
****************************************************************************/

BOOL CGetLocaleInfo::QuerySave()
    {
     return TRUE;
    } // CGetLocaleInfo::QuerySave

/****************************************************************************
*                            CGetLocaleInfo::Save
* Inputs:
*       const CString & filename: File name
*       UINT mode: Save mode: 0 (default), 8 (UTF-8), 16 (UTF-16)
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Saves the information
****************************************************************************/

BOOL CGetLocaleInfo::Save(const CString & filename, UINT mode)
    {
     CBiFile f;
     if(!f.Open(filename, CFile::modeWrite | CFile::modeCreate, mode))
        { /* failed */
         return FALSE;
        } /* failed */

     try
        { /* try */
         f.WriteBOM();
         f.WriteString(GetOutputHeader());
        } /* try */
     catch (CFileException * e)
        { /* failed */
         e->Delete();
         f.Close();
         return FALSE;
        } /* failed */

     BOOL result = WriteChildren(f);
     f.Close();
     return result;
    } // CGetLocaleInfo::Save
