// Help.cpp : implementation file
//
// Copyright  2005, Joseph M. Newcomer, All Rights Reserved
// You are free to redistribute FlounderClock providing all copyrights are left intact
// You must record any changes you make to this source by putting // and your
// initials at the end of each line you change. You may not remove any lines
// from this source; instead, put // in front of them and your initials at the end.
// You may not charge any fee for the redistribution, nor incorporate this code in
// any commercial product. This is freeware, and must remain so.
//

#include "stdafx.h"
#include "resource.h"
#include "Help.h"
#include "ErrorString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

UINT UWM_REPOSITION = ::RegisterWindowMessage(_T("UWM_REPOSITION-{367C1465-286A-40f8-B194-BAD31D8F7B42}"));

class StreamPos {
    public:
       LPCSTR base;
       DWORD pos;
       DWORD len;
       StreamPos() { pos = 0; len = 0; base = NULL; }
    };

/////////////////////////////////////////////////////////////////////////////
// CHelp dialog


CHelp::CHelp(CWnd* pParent /*=NULL*/)
        : CDialog(CHelp::IDD, pParent)
{
        FirstFocus = TRUE;
        //{{AFX_DATA_INIT(CHelp)
        //}}AFX_DATA_INIT
}


void CHelp::DoDataExchange(CDataExchange* pDX)
{
        CDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CHelp)
        DDX_Control(pDX, IDC_TEXT, c_Text);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CHelp, CDialog)
        ON_NOTIFY(EN_LINK, IDC_TEXT, OnURLLink)
        ON_REGISTERED_MESSAGE(UWM_REPOSITION, OnPosition)
        //{{AFX_MSG_MAP(CHelp)
        ON_WM_SIZE()
        ON_NOTIFY(EN_SETFOCUS, IDC_TEXT, OnSetfocusText)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

typedef DWORD DWORD_PTR; // fix this later when new platform SDK installed
/////////////////////////////////////////////////////////////////////////////
// CHelp message handlers

/****************************************************************************
*                                MakeHelpName
* Inputs:
*       CString name: Name to be converted to help key
* Result: CString
*       The help key
* Notes: 
*       This involves stripping out any &, and prepending HELP_
****************************************************************************/

/* static */ CString CHelp::MakeHelpName(const CString & name)
    {
     CString result;

     result = _T("HELP_") + name;
     result.Replace(_T("&"), _T(""));
     result.Replace(_T(" "), _T("_"));
     result.Replace(_T("["), _T("_"));
     result.Replace(_T("]"), _T("_"));
     result.Replace(_T("."), _T("_DOT_"));
     result.Replace(_T(","), _T("_COMMA_"));
     result.Replace(_T("/"), _T("_SLASH_"));
     result.Replace(_T("'"), _T("_SQUOTE_"));
     result.Replace(_T("-"), _T("_DASH_"));
     result.Replace(_T(":"), _T("_COLON_"));
     result.Replace(_T("("), _T(""));
     result.Replace(_T(")"), _T(""));
     return result;
    } // MakeHelpName

/****************************************************************************
*                               StreamCallback
* Inputs:
*       DWORD_PTR cookie: The cookie object
*       LPBYTE pbuff: Pointer to the buffer
*       LONG cb: Length of pbuff
*       LPLONG bytesTransferred
* Result: DWORD
*       0 if success
*       nonzero if error
* Effect: 
*       Transfers a section of the data from the buffer referred to by the
*       cookie to the stream buffer
****************************************************************************/

DWORD CALLBACK StreamCallback(DWORD_PTR cookie, LPBYTE pbuff, LONG cb, LPLONG bytesTransferred)
    {
     StreamPos * stream = (StreamPos *)cookie;
     
     int len = stream->len - stream->pos;
     if(len == 0)
        { /* done */
         *bytesTransferred = 0;
         return 0;
        } /* done */

     if(len < cb)
        { /* transfer all */
         memcpy(pbuff, &stream->base[stream->pos], len);
         *bytesTransferred = len;
         stream->pos += len;
         return 0;
        } /* transfer all */
     else
        { /* transfer some */
         memcpy(pbuff, &stream->base[stream->pos], cb);
         *bytesTransferred = cb;
         stream->pos += cb;
         return 0;
        } /* transfer some */
     return 0; // 0 =>success
    } // StreamCallback


/****************************************************************************
*                             CHelp::LoadHelpText
* Result: BOOL
*       TRUE if success
*       FALSE if error
* Effect: 
*       Loads the help text into the rich edit control
* Notes:
*       This uses m_HelpName to indicate the help resource to be loaded
*       * If the string is empty, it uses the resource whose ID is IDR_HELP
*       * If the string starts with #, it is followed by the decimal digits
*         of an RTF resource stored as an integer value, e.g. #1112 would
*         be stored as resource ID MAKEINTRESOURCE(1112).  This is used to
*         associate IDs with controls that have no text
*       * Otherwise, it is the string name of a resource generated by
*         taking the text caption and applying CHelp::MakeHelpName to it
****************************************************************************/

BOOL CHelp::LoadHelpText()
    {
     // This is a piece of absolute insanity in the VS6 dialog editor
     // I give it names for which there are defined symbols, and instead
     // of using the symbolic names, such as LOCALE_WHATEVER, it looks
     // at the definition of LOCALE_WHATEVER, finds it is 0XNNNNNNNN, and
     // replaces the name with the string name 0XNNNNNNNN

     
     HRSRC resource;
     if(m_HelpName.IsEmpty())
        { /* use default help */
         resource = ::FindResource(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_HELP), _T("RTF"));
        } /* use default help */
     else
        { /* use selected ID */
         resource = FindHelpResource(m_HelpName);
        } /* use selected ID */

     if(resource == NULL)
        { /* failed */
         DWORD err = ::GetLastError();
         return FALSE;
        } /* failed */

     HGLOBAL h = ::LoadResource(AfxGetInstanceHandle(), resource);
     if(h == NULL)
        { /* failed load */
         DWORD err = ::GetLastError();
         return FALSE;
        } /* failed load */

     LPVOID p = ::LockResource(h);
     if(p == NULL)
        { /* failed lock */
         DWORD err = ::GetLastError();
         return FALSE;
        } /* failed lock */

     StreamPos cookie;
     cookie.len = ::SizeofResource(AfxGetInstanceHandle(), resource);
     cookie.base = (LPSTR)p;

     EDITSTREAM es = {
        (DWORD_PTR) &cookie,
        0,
        StreamCallback};

     c_Text.AutoURLDetect(TRUE);
     DWORD mask =c_Text.GetEventMask();
     c_Text.SetEventMask(mask | ENM_LINK);

     c_Text.StreamIn(SF_RTF, es);
     c_Text.SetSel(-1,0);
     return TRUE;
    } // CHelp::LoadHelpText


/****************************************************************************
*                             CHelp::OnInitDialog
* Result: BOOL
*       TRUE, always
* Effect: 
*       Initializes the dialog
****************************************************************************/

BOOL CHelp::OnInitDialog() 
   {
    CDialog::OnInitDialog();
        
    if(!LoadHelpText())
       { /* failed */
        DWORD err = ::GetLastError();
        CString fmt;
        fmt.LoadString(IDS_HELP_NOT_FOUND);

        CString msg;
        msg.Format(fmt, m_HelpName, ErrorString(err));
        c_Text.SetWindowText(msg);
       } /* failed */

    c_Text.SetFocus();
    c_Text.SetSel(-1, 0);
    PostMessage(UWM_REPOSITION);
    
    return FALSE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
}

/****************************************************************************
*                                CHelp::OnSize
* Inputs:
*       UINT nType: Sizing type
*       int cx: New client width
*       int cy: New client height
* Result: void
*       
* Effect: 
*       Resizes the text control to fill the window size
****************************************************************************/

void CHelp::OnSize(UINT nType, int cx, int cy) 
   {
    CDialog::OnSize(nType, cx, cy);
        
    if(c_Text.GetSafeHwnd() != NULL)
       { /* has window */
        CRect r;
        c_Text.GetWindowRect(&r);
        ScreenToClient(&r);
        c_Text.SetWindowPos(NULL, 0, 0, cx, cy - r.top,
                     SWP_NOMOVE| SWP_NOZORDER);
       } /* has window */
   }

/****************************************************************************
*                              CHelp::OnURLLink
* Inputs:
*       NMHDR * pNotifyStruct:
*       LRESULT * pResult: set to 0
* Result: void
*       
* Effect: 
*       Performs a link operation
****************************************************************************/

void CHelp::OnURLLink(NMHDR * pNotifyStruct, LRESULT * pResult)
    {
     SHORT state = GetKeyState(VK_LBUTTON);
     if(state < 0)
        { /* click on URL */
         ENLINK * enlink = (ENLINK *)pNotifyStruct;
         int start = enlink->chrg.cpMin;
         int end = enlink->chrg.cpMax;

         CString link;
         c_Text.GetTextRange(start, end, link);
         ShellExecute(m_hWnd, _T("open"), link, NULL, NULL, SW_RESTORE);
        } /* click on URL */

     *pResult = 0;  // tells control to handle it. This creates the little
                    // pointy-hand cursor (setting this to 1 suppresses
                    // the change of cursor shape)
    } // CHelp::OnURLLink

/****************************************************************************
*                               CHelp::OnNotify
* Inputs:
*       WPARAM wParam:
*       LPARAM lParam:
* Result: LRESULT
*       whatever
* Effect: 
*       Handles the hyperlink notifications
* Notes:
*       This was used for debugging the notification that is later sent
*       to OnURLLink, because the conditions under which it is executed
*       are not well-defined in the documentation
****************************************************************************/

BOOL CHelp::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT * result)
    {
#ifdef _DEBUG //-------------------------------------------------------------------------------------+
     static int msgnum = 0; // silly use of static variable, but this is just for debugging       // |
     NMHDR * hdr = (NMHDR *)lParam;                                                               // |
     if(hdr->idFrom != IDC_TEXT)                                                                  // |
        return CDialog::OnNotify(wParam, lParam, result);                                         // |
     CString s;                                                                                   // |
     CString msg;                                                                                 // |
     SHORT state = GetKeyState(VK_LBUTTON);                                                       // |
     switch(hdr->code)                                                                            // |
        { /* code */                                                                              // |
         case EN_LINK:                                                                            // |
            { /* EN_LINK */                                                                       // |
             ENLINK * enlink = (ENLINK *)hdr;                                                     // |
             msg.Format(_T("EN_LINK: %d..%d %s"), enlink->chrg.cpMin, enlink->chrg.cpMax, state < 0 ? _T("*") : _T(""));
            } /* EN_LINK */                                                                       // |
            break;                                                                                // |
         default:                                                                                 // |
            msg.Format(_T("%d"), hdr->code);                                                      // |
            break;                                                                                // |
        } /* code */                                                                              // |
                                                                                                  // |
     s.Format(_T("{%04d} WM_NOTIFY %s\n"), msgnum++, msg);                                        // |
     TRACE(s);                                                                                    // |
#endif //--------------------------------------------------------------------------------------------+
     return CDialog::OnNotify(wParam, lParam, result);
    } // CHelp::OnNotify

/****************************************************************************
*                            CHelp::OnSetfocusText
* Inputs:
*       NMHDR * pNMHDR:
*       LRESTUL * pResult:
* Result: void
*       
* Effect: 
*       A futile attempt to prevent selection from happening on initial startup
****************************************************************************/

void CHelp::OnSetfocusText(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    if(FirstFocus)
       c_Text.SetSel(0,0);
    FirstFocus = FALSE;
        
    *pResult = 0;
   }

/****************************************************************************
*                              CHelp::OnPosition
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Positions the window properly
****************************************************************************/

LRESULT CHelp::OnPosition(WPARAM, LPARAM)
    {
     HMONITOR monitor = ::MonitorFromWindow(m_hWnd, MONITOR_DEFAULTTOPRIMARY);
     MONITORINFO info = { sizeof(MONITORINFO) };
     CPoint pt;
     if(::GetMonitorInfo(monitor, &info))
        { /* got it */
         pt.x = info.rcMonitor.left;
         pt.y = info.rcMonitor.top;
        } /* got it */
     else
        { /* failed */
         pt.x = pt.y = 0;
        } /* failed */

     SetWindowPos(NULL, pt.x, pt.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
     int current = c_Text.GetFirstVisibleLine();
     if(current != 0)
        c_Text.LineScroll(-current, 0);
     return 0;
    } // CHelp::OnPosition

/****************************************************************************
*                           CHelp::FindHelpResource
* Inputs:
*       const CString & helpname: The name of the help, expressed as
*               "HELP_name"
*               "#ddddd"  for ddddd the resource ID in decimal
* Result: HRSRC
*       Resource handle to help resource
* Effect: 
*       Finds the help resource
****************************************************************************/

/* static */ HRSRC CHelp::FindHelpResource(const CString & helpname)
    {
     HRSRC resource = NULL;
     if(helpname.Left(1) == _T("#"))
        { /* decimal ID */
         UINT id = _ttoi(helpname.Mid(1));
         resource = ::FindResource(AfxGetInstanceHandle(), MAKEINTRESOURCE(id), _T("RTF"));
        } /* decimal ID */
     else
        { /* string name */
         resource = ::FindResource(AfxGetInstanceHandle(), MakeHelpName(helpname), _T("RTF"));
        } /* string name */
     return resource;
    } // FindHelpResource
