// HexEditCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "HexEditCtrl.h"
#include "mbtrace.h"
#include "GetCaption.h"
#include "Help.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const int CHexEditCtrl::default_widthW = 5;
const int CHexEditCtrl::default_widthA = 3;


/////////////////////////////////////////////////////////////////////////////
// CHexEditCtrl

CHexEditCtrl::CHexEditCtrl()
   {
#ifdef _UNICODE
    width = default_widthW;
#else
    width = default_widthA;
#endif
    OVR = FALSE;
    Sending = FALSE;
    Changing = FALSE;
   }

CHexEditCtrl::~CHexEditCtrl()
{
}


BEGIN_MESSAGE_MAP(CHexEditCtrl, CRichEditCtrlEx)
        ON_CONTROL_REFLECT_EX(EN_CHANGE, OnChange)
        ON_NOTIFY_REFLECT_EX(EN_PROTECTED, OnProtected)
        ON_CONTROL_REFLECT_EX(EN_SETFOCUS, OnSetfocus)
        ON_CONTROL_REFLECT_EX(EN_KILLFOCUS, OnKillfocus)
        ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
        ON_COMMAND(ID_EDIT_SELECTALL, OnEditSelectAll)
        ON_MESSAGE(WM_CUT, OnCut)
        ON_MESSAGE(WM_PASTE, OnPaste)
        ON_MESSAGE(WM_CLEAR, OnClear)
        ON_MESSAGE(WM_COPY, OnCopy)
        ON_COMMAND(IDM_WHAT_IS_THIS, OnWhatIsThis)
        //{{AFX_MSG_MAP(CHexEditCtrl)
        ON_WM_CHAR()
        ON_WM_KEYDOWN()
        ON_WM_KEYUP()
        ON_WM_CONTEXTMENU()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHexEditCtrl message handlers

/****************************************************************************
*                            CHexEditCtrl::OnChar
* Inputs:
*       UINT nChar: Character code
*       UINT nRepCnt: Repeat count
*       UINT nFlags: flags
* Result: void
*       
* Effect: 
*       Handles the character
* Notes:
*       If the character is Ctrl+C, for copy, the selection must be
*       nonempty.  In this case, the character is copied to the clipboard
*
*       If it is not a recognized control character operation, then the
*       selection must be empty to handle other cases
*
*       If the character is not in the set [0-9A-Fa-f], it is rejected
*
*       Otherwise, the character replaces the character just ahead of the
*       caret.
****************************************************************************/

void CHexEditCtrl::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
   {
    long low;
    long high;
    CRichEditCtrlEx::GetSel(low, high);
    MB_TRACE2("=>CHexEditCtrl::OnChar('%c' 0x%04x)", nChar, nChar);
    MB_TRACE2(" sel=[%d, %d]\n", low, high);

#define CTRL(x)  ((x) - _T('@'))
    switch(nChar)
       { /* check ctrl codes */
        case CTRL(_T('C')):
           if(low == high)
               { /* no selection */
                MessageBeep(0);
                return;
               } /* no selection */
           SendMessage(WM_COPY);
           return;
        } /* check ctrl codes */

     // Typing is illegal if there is a nonempty selection
    if(low != high)
       { /* no replace */
        MessageBeep(0);
        MB_TRACE0("<=CHexEditCtrl::OnChar (selection: rejected)\n");
        return;
       } /* no replace */
    
    switch(nChar)
       { /* character */
        case _T('0'):
        case _T('1'):
        case _T('2'):
        case _T('3'):
        case _T('4'):
        case _T('5'):
        case _T('6'):
        case _T('7'):
        case _T('8'):
        case _T('9'):
        case _T('a'):
        case _T('b'):
        case _T('c'):
        case _T('d'):
        case _T('e'):
        case _T('f'):
        case _T('A'):
        case _T('B'):
        case _T('C'):
        case _T('D'):
        case _T('E'):
        case _T('F'):
           break;
        default:
           MessageBeep(0);
           MB_TRACE0("<=CHexEditCtrl::OnChar (character: rejected)\n");
           return;
       } /* character */
        
    Changing = TRUE;
#ifdef _DEBUG // ----------------------------------------------------------------+
    DWORD mask = GetEventMask();                                              // |
    ASSERT(mask & ENM_CHANGE);                                                // |
#endif // -----------------------------------------------------------------------+
    CRichEditCtrlEx::OnChar(nChar, nRepCnt, nFlags);
    MB_TRACE0("<=CHexEditCtrl::OnChar (accepted)\n");
   }

BOOL CHexEditCtrl::OnChange() 
   {
    return FALSE; // propagate to parent
   }

/****************************************************************************
*                           CHexEditCtrl::SendInsert
* Result: void
*       
* Effect: 
*       Toggles the OVR mode on
****************************************************************************/

void CHexEditCtrl::SendInsert()
    {
     Sending = TRUE; // Indicate that the VK_INSERTs should pass thru
     SendMessage(WM_KEYDOWN, VK_INSERT, 0x00510001);
     SendMessage(WM_KEYUP, VK_INSERT, 0xC0510001);
    } // CHexEditCtrl::SendInsert

/****************************************************************************
*                          CHexEditCtrl::OnSetFocus
* Result: BOOL
*       
* Effect: 
*       Saves current OVR mode and sets it
****************************************************************************/

BOOL CHexEditCtrl::OnSetfocus() 
   {
    MB_TRACE1("=>CHexEditCtrl::OnSetFocus() OVR=%s\n", OVR ? _T("TRUE") : _T("FALSE"));
    if(!OVR)
       { /* needs to set OVR */
        SendInsert();
       } /* needs to set OVR */
    MB_TRACE0("<=CHexEditCtrl::OnSetfocus\n");
    return FALSE;
   } // CHexEditCtrl::OnSetfocus

/****************************************************************************
*                          CHexEditCtrl::OnKillfocus
* Result: BOOL
*       FALSE, always (allows parent to handle it)
* Effect: 
*       Restores the OVR state
****************************************************************************/

BOOL CHexEditCtrl::OnKillfocus() 
   {
    return FALSE;
   }

/****************************************************************************
*                           CHexEditCtrl::OnKeyDown
* Inputs:
*       
* Result: void
*       
* Effect: 
*       Keeps the user from changing the OVR state
****************************************************************************/

void CHexEditCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
   {
    MB_TRACE2("=>CHexEditCtrl::OnKeyDown(0x%04x,...) Sending=%s", nChar, Sending ? _T("TRUE") : _T("FALSE"));
#ifdef _DEBUG  //-------------------------------------------------------------+
    switch(nChar)                                                          // |
       { /* nChar */                                                       // |
        case _T('\r'):                                                     // |
           MB_TRACE0(" (\\r)");                                            // |
           break;                                                          // |
        case _T('\n'):                                                     // |
           MB_TRACE0(" (\\n)");                                            // |
           break;                                                          // |
        case VK_DELETE:                                                    // |
           MB_TRACE0(" (VK_DELETE)");                                      // |
           break;                                                          // |
        case VK_INSERT:                                                    // |
           MB_TRACE0(" (VK_INSERT)");                                      // |
           break;                                                          // |
        case VK_BACK:                                                      // |
           MB_TRACE0(" (VK_BACK)");                                        // |
           break;                                                          // |
        case 0x0058:                                                       // |
           MB_TRACE0(" ('X')");                                            // |
           break;                                                          // |
       } /* nChar */                                                       // |
    MB_TRACE0("\n");                                                       // |
#endif //---------------------------------------------------------------------+
    //----------------------------------------------------------------
    // If we are not in the special mode used to force recognition of VK_INSERT
    // reject the keystroke
    //----------------------------------------------------------------
    if(!Sending && nChar == VK_INSERT)
       { /* VK_INSERT */
        MB_TRACE0("<=CHexEditCtrl::OnKeyDown (reject VK_INSERT)\n");
        return; // do not allow INSERT key
       } /* VK_INSERT */
    //----------------------------------------------------------------
    // Reject other keys that would change the contents
    //----------------------------------------------------------------
    if(nChar == _T('\r'))
       { /* \r */
        MB_TRACE0("<=CHexEditCtrl::OnKeyDown (reject \\r)\n");
        return;
       } /* \r */
    if(nChar == _T('\n'))
       { /* \n */
        MB_TRACE0("<=CHexEditCtrl::OnKeyDown (reject \\n)\n");
        return;
       } /* \n */
    if(nChar == VK_DELETE)
       { /* VK_DELETE */
        MB_TRACE0("<=CHexEditCtrl::OnKeyDown (reject VK_DELETE)\n");
        return;
       } /* VK_DELETE */
    if(nChar == VK_BACK)
       { /* VK_BACK */
        MB_TRACE0("<=CHexEditCtrl::OnKeyDown (reject VK_BACK)\n");
        return;
       } /* VK_BACK */
    if(nChar == 0x0058)
       { /* X */
        MB_TRACE0("<=CHexEditCtrl::OnKeyDown (reject X)\n");
        return;
       } /* X */
    //----------------------------------------------------------------
    // Otherwise pass the key on through
    //----------------------------------------------------------------
    CRichEditCtrlEx::OnKeyDown(nChar, nRepCnt, nFlags);
    MB_TRACE0("<=CHexEditCtrl::OnKeyDown\n");
   } // CHexEditCtrl::OnKeyDown

/****************************************************************************
*                         CHexEditCtrl::SetWindowText
* Inputs:
*       const CString & s: String
*       long low: Low selection
*       long high: High selection
*       BOOL Notify = FALSE;
* Result: void
*       
* Effect: 
*       Sets the text which will be rendered as hex
****************************************************************************/

void CHexEditCtrl::SetWindowText(const CString & s, long low, long high, BOOL Notify /* = FALSE */)
    {
     MB_TRACE3("=>CHexEditCtrl::SetWindowText(\"%s\", %d, %d)\n", s, low, high);
     if(Changing && s.GetLength() == text.GetLength())
        { /* ignore change */
         Changing = FALSE;
         MB_TRACE0("<=CHexEditCtrl::SetWindowText (was Changing)\n");
         return;
        } /* ignore change */

#ifndef _DEBUG // Only optimize flicker issues in release version
     SetRedraw(FALSE);
#endif
     //----------------------------------------------------------------
     // Disable parent change notification if NoNotify is set
     //----------------------------------------------------------------
     DWORD omask = GetEventMask();
     if(!Notify)
        { /* no notification */
         SetEventMask(omask & ~ENM_CHANGE);
        } /* no notification */

     //----------------------------------------------------------------
     // Retain the text
     //----------------------------------------------------------------
     text = s;

     //----------------------------------------------------------------
     // If we are locally changing the contents of the edit control, before
     // we rewrite the data save the current selection
     //----------------------------------------------------------------
     if(!Changing)
        { /* record new selection */
         selstart = low;
         selend = high;
        } /* record new selection */

     //----------------------------------------------------------------
     // Because we are about to delete protected space, we have to turn
     // off the protected notifications so we can set the
     // text to all blank
     //----------------------------------------------------------------
     {
      DWORD pmask = GetEventMask();
      SetEventMask(pmask & ~ENM_PROTECTED);

      MB_TRACE0("=>CRichEditCtrlEx::SetWindowText(_T(\"\"))\n");
      CRichEditCtrl::SetWindowText(_T("")); // delete existing text
      MB_TRACE0("<=CRichEditCtrlEx::SetWindowText\n");

      SetEventMask(pmask);
     }
     
     //----------------------------------------------------------------
     // Note that for ANSI we use 2 bytes and Unicode we use 4
     // xx_
     // xxxx_
     //----------------------------------------------------------------

     //----------------------------------------------------------------
     // Format each value in hex, append to the text
     //----------------------------------------------------------------
     int pos = 0;
     LPTSTR p = text.GetBuffer(0); // Note that we use GetBuffer so we can get NUL
     for(int i = 0; i <= text.GetLength(); i++) // Note <= to get terminal NUL
        { /* scan string */
         CString t;
         if(width == default_widthW)
            t.Format(_T("%04x "), (TCHAR)p[i]);
         else
            t.Format(_T("%02x "), (BYTE)p[i]);

         MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", pos, pos);
         CRichEditCtrlEx::SetSel(pos, pos);
         MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");

         MB_TRACE1("=>CRichEditCtrlEx::ReplaceSel(\"%s\")\n", t);
         CRichEditCtrlEx::ReplaceSel(t);
         MB_TRACE0("<=CRichEditCtrlEx::ReplaceSel\n");
         
         // Now set the attributes
         CHARFORMAT fmt = { sizeof(CHARFORMAT) };
         fmt.dwMask = CFM_PROTECTED;
         fmt.dwEffects = CFE_PROTECTED;

         MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", pos + width - 1, pos + width);
         CRichEditCtrlEx::SetSel(pos + width - 1, pos + width);
         MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");

         MB_TRACE0("=>CRichEditCtrlEx::SetSelectionCharFormat(CFE_PROTECTED)\n");
         CRichEditCtrlEx::SetSelectionCharFormat(fmt);
         MB_TRACE0("<=CRichEditCtrlEx::SetSelectionCharFormat\n");
         
         pos += width;
        } /* scan string */
     text.ReleaseBuffer();

     //----------------------------------------------------------------
     // Now make the selection correspond
     //----------------------------------------------------------------
     SetSel(selstart, selend);

     //----------------------------------------------------------------
     // This should be handled better, but we don't.
     // Do not allow any text to be appended
     //----------------------------------------------------------------
     int n = GetWindowTextLength();
     LimitText(n);

     //----------------------------------------------------------------
     // Restore the original event mask
     //----------------------------------------------------------------
     SetEventMask(omask);

#ifndef _DEBUG // Only optimize flicker issues in release version
     SetRedraw(TRUE);
     Invalidate();
#endif
     MB_TRACE0("<=CHexEditCtrl::SetWindowText\n");
    } // CHexEditCtrl::SetWindowText

/****************************************************************************
*                     CHexEditCtrl::SetWindowTextNoNotify
* Inputs:
*       const CString & s: String
*       long low: Low selection
*       long high: High selection
* Result: void
*       
* Effect: 
*       Sets with no notification
****************************************************************************/

void CHexEditCtrl::SetWindowTextNoNotify(const CString & s, long low, long high)
    {
     SetWindowText(s, low, high, FALSE);
    } // CHexEditCtrl::SetWindowTextNoNotify

/****************************************************************************
*                            CHexEditCtrl::OnKeyUp
* Inputs:
*       UINT nChar:
*       UINT nRepCnt:
*       UINT nFlags:
* Result: void
*       
* Effect: 
*       If we are in the mode of toggling the overstrike mode, set the
*       OVR mode to TRUE, otherwise ignore any VK_INSERT.
****************************************************************************/

void CHexEditCtrl::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
   {
    MB_TRACE2("=>CHexEditCtrl::OnKeyUp(0x%04x,...) Sending=%s", nChar, Sending ? _T("TRUE") : _T("FALSE"));
#ifdef _DEBUG  // ----------------------------------------------------------------+
    Changing = FALSE;                                                          // |
    switch(nChar)                                                              // |
       { /* nChar */                                                           // |
        case VK_DELETE:                                                        // |
           MB_TRACE0(" (VK_DELETE)");                                          // |
           break;                                                              // |
        case VK_INSERT:                                                        // |
           MB_TRACE0(" (VK_INSERT)");                                          // |
           break;                                                              // |
        case VK_BACK:                                                          // |
           MB_TRACE0(" (VK_BACK)");                                            // |
           break;                                                              // |
       } /* nChar */                                                           // |
    MB_TRACE0("\n");                                                           // |
#endif //-------------------------------------------------------------------------+
    //----------------------------------------------------------------
    // If we were in the special sending mode to allow VK_INSERT
    // to go through, the WM_KEYUP has completed that cycle and
    // we clear the Sending flag, and set OVR to TRUE
    // Otherwise we reject the VK_INSERT key-up request
    //----------------------------------------------------------------
    if(Sending && nChar == VK_INSERT)
       { /* end send */
        Sending = FALSE;
        OVR = TRUE;
       } /* end send */
    else
    if(!Sending && nChar == VK_INSERT)
       { /* VK_INSERT */
        MB_TRACE0("<=CHexEditCtrl::OnKeyUp (rejecting VK_INSERT)\n");
        return;
       } /* VK_INSERT */

    //----------------------------------------------------------------
    // Reject delete and backspace
    //----------------------------------------------------------------
    if(nChar == VK_DELETE)
       { /* VK_DELETE */
        MB_TRACE0("<=CHexEditCtrl::OnKeyUp (rejecting VK_DELETE)\n");
        return;
       } /* VK_DELETE */

    if(nChar == VK_BACK)
       { /* VK_BACK */
        MB_TRACE0("<=CHexEditCtrl::OnKeyUp (rejecting VK_BACK)\n");
        return;
       } /* VK_BACK */

    //----------------------------------------------------------------
    // Pass it on
    //----------------------------------------------------------------
    CRichEditCtrlEx::OnKeyUp(nChar, nRepCnt, nFlags);
    MB_TRACE0("<=CHexEditCtrl::OnKeyUp\n");
   }

/****************************************************************************
*                       CHexEditCtrl::PreSubclassWindow
* Result: void
*       
* Effect: 
*       Creates and sets the font
****************************************************************************/

void CHexEditCtrl::PreSubclassWindow() 
   {
    LOGFONT lf;
    CFont * f;
    f = GetFont();
    if(f == NULL)
       { /* no font set */
        CFont tmp;
        tmp.CreateStockObject(DEFAULT_GUI_FONT);
        tmp.GetLogFont(&lf);
       } /* no font set */
    else
       { /* has font */
        f->GetLogFont(&lf);
       } /* has font */

    _tcscpy(lf.lfFaceName, _T("Courier New"));
    font.CreateFontIndirect(&lf);
    SetFont(&font);

    CRichEditCtrlEx::PreSubclassWindow();
   }

/****************************************************************************
*                            CHexEditCtrl::SetSel
* Inputs:
*       long low: start of selection relative to source text
*       long high: end of selection relative to source text
* Result: void
*       
* Effect: 
*       Highlights the corresponding text
****************************************************************************/

void CHexEditCtrl::SetSel(long low, long high)
    {
     MB_TRACE3("=>CHexEditCtrl::SetSel(%d, %d) %s\n", low, high, Changing ? _T("Changing") : _T(""));
     if(!Changing)
        { /* update selection */
         int start = low * width;
         int stop = high * width;

         MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", start, stop);
         CRichEditCtrl::SetSel(start, stop);
         MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");
        } /* update selection */
     MB_TRACE0("<=CHexEditCtrl::SetSel\n");
    } // CHexEditCtrl::SetSel

/****************************************************************************
*                         CHexEditCtrl::GetWindowText
* Inputs:
*       CString & s: String to fill in
* Result: void
*       
* Effect: 
*       Fills the string in by 
****************************************************************************/

void CHexEditCtrl::GetWindowText(CString & s)
   {
    MB_TRACE0("=>CHexEditCtrl::GetWindowText()\n");
    s = _T("");

    CString t;

    CRichEditCtrlEx::GetWindowText(t);
    while(!t.IsEmpty())
       { /* scan and convert */
        if(t.GetLength() >= width)
           { /* has one valid */
            CString num = t.Left(width);
            t = t.Mid(width);
            TCHAR ch = (TCHAR)_tcstoul(num, NULL, 16);
            s += ch;
           } /* has one valid */
        else
           { /* last not valid */
            s += _T("?"); // FIX THIS to use GetCPInfo!
            break;
           } /* last not valid */
       } /* scan and convert */
    MB_TRACE1("<=CHexEditCtrl::GetWindowText \"%s\"\n", s);
   } // CHexEditCtrl::GetWindowText

/****************************************************************************
*                          CHexEditCtrl::OnProtected
* Inputs:
*       NMHDR * pNMHDR: (NMHDR*)(ENPROTECTED *)
*       LRESULT * pResult: nonzero to lock out operation, zero to allow it
* Result: BOOL
*       TRUE to block parent handling (always)
*       FALSE to allow parent handling (the parent isn't supposed to know about
*               the use of protected fields, so this is never returned)
* Effect: 
*       Blocks editing of protected fields
****************************************************************************/

BOOL CHexEditCtrl::OnProtected(NMHDR * pNMHDR, LRESULT * pResult)
    {
     ENPROTECTED *pEnProtected = reinterpret_cast<ENPROTECTED *>(pNMHDR);
     // See where the selection is
     long low;
     long high;
     CRichEditCtrlEx::GetSel(low, high);

     MB_TRACE2("=>CHexEditCtrl::OnProtected(%d, %d)\n", low, high);
     int n = CRichEditCtrlEx::GetWindowTextLength();
     if(low == high && low < n && pEnProtected->msg == WM_CHAR)
        { /* move forward */

         MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", low + 1, low + 1);
         CRichEditCtrlEx::SetSel(low + 1, low + 1);
         MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");

         *pResult = 1;
         PostMessage(pEnProtected->msg, pEnProtected->wParam, pEnProtected->lParam);
         MB_TRACE1("<=CHexEditCtrl::OnProtected '%c'\n", pEnProtected->wParam);
         return TRUE;
        } /* move forward */
     *pResult = 1;
     MB_TRACE0("<=CHexEditCtrl::OnProtected\n");
     return TRUE;
    } // CHexEditCtrl::OnProtected

/****************************************************************************
*                           CHexEditCtrl::SetModeA
* Result: void
*       
* Effect: 
*       Sets the mode to interpret its string as 8-bit characters
****************************************************************************/

void CHexEditCtrl::SetModeA()
    {
     width = default_widthA;
     SetWindowText(text, selstart, selend);
    } // CHexEditCtrl::SetModeA

/****************************************************************************
*                           CHexEditCtrl::SetModeW
* Result: void
*       
* Effect: 
*       Sets the mode to interpret characters as wide characters
****************************************************************************/

void CHexEditCtrl::SetModeW()
    {
     width = default_widthW;
     SetWindowText(text, selstart, selend);
    } // CHexEditCtrl::SetModeW

/****************************************************************************
*                        CHexEditCtrl::SetSelNoNotify
* Inputs:
*       long low:
*       long high:
* Result: void
*       
* Effect: 
*       Sets the selection but suppresses notification
****************************************************************************/

void CHexEditCtrl::SetSelNoNotify(long low, long high)
    {
     DWORD mask = GetEventMask();
     SetEventMask(mask & ~ENM_SELCHANGE);
     SetSel(low, high);
     SetEventMask(mask);
    } // CHexEditCtrl::SetSelNoNotify

/****************************************************************************
*                           CHexEditCtrl::GetRawSel
* Inputs:
*       long & low:
*       long & high:
* Result: void
*       
* Effect: 
*       Gets the raw selection
****************************************************************************/

void CHexEditCtrl::GetRawSel(long & low, long & high)
    {
     MB_TRACE0("=>CHexEditCtrl::GetRawSel()\n");
     CRichEditCtrlEx::GetSel(low, high);
     MB_TRACE2("<=CHexEditCtrl::GetRawSel=[%d,%d]\n", low, high);
    } // CHexEditCtrl::GetRawSel

/****************************************************************************
*                           CHexEditCtrl::SetRawSel
* Inputs:
*       long low:
*       long high:
* Result: void
*       
* Effect: 
*       Sets the raw selection
****************************************************************************/

void CHexEditCtrl::SetRawSel(long low, long high)
    {
     MB_TRACE2("=>CHexEditCtrl::GetRawSel(%d,%d)\n", low, high);
     DWORD mask = GetEventMask();
     SetEventMask(mask & ~ENM_SELCHANGE);
     CRichEditCtrlEx::SetSel(low, high);
     SetEventMask(mask);
     MB_TRACE0("<=CHexEditCtrl::GetRawSel\n");
    } // CHexEditCtrl::SetRawSel

/****************************************************************************
*                             CHexEditCtrl::OnCut
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       0, always
* Effect: 
*       Causes cut to be ignored
****************************************************************************/

LRESULT CHexEditCtrl::OnCut(WPARAM, LPARAM)
    {
     MessageBeep(0);
     return 0;
    } // CHexEditCtrl::OnCut

/****************************************************************************
*                            CHexEditCtrl::OnPaste
* Inputs:
*       WPARAM wParam: ignored
*       LPARAM lParam: ignored
* Result: LRESULT
*       0, always
* Effect: 
*       Ignores OnPaste
****************************************************************************/

LRESULT CHexEditCtrl::OnPaste(WPARAM, LPARAM)
    {
     MessageBeep(0);
     return 0;
    } // CHexEditCtrl::OnPaste

/****************************************************************************
*                            CHexEditCtrl::OnCopy
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Copies the text to the clipboard
* Notes:
*       This is required because the normal "copy" handler won't do a copy
*       if a protected field is involved, so we have to override it to
*       ignore protected fields.
****************************************************************************/

LRESULT CHexEditCtrl::OnCopy(WPARAM, LPARAM)
    {
     long low;
     long high;
     GetSel(low, high);
     if(low == high)
        { /* nothing selected */
         MessageBeep(0);
         return 0;
        } /* nothing selected */

     CString s = GetSelText();
     HANDLE gbl = ::GlobalAlloc(GMEM_MOVEABLE, (s.GetLength() + 1) * sizeof(TCHAR));
     if(gbl == NULL)
        { /* no memory */
         MessageBeep(0);
         return 0;
        } /* no memory */

     LPTSTR p = (LPTSTR)::GlobalLock(gbl);
     if(p == NULL)
        { /* lock failed */
         ::GlobalFree(gbl);
         MessageBeep(0);
         return 0;
        } /* lock failed */

     memcpy(p, (LPVOID)(LPCTSTR)s, (s.GetLength() + 1) * sizeof(TCHAR));

     ::GlobalUnlock(gbl);
     
     if(!OpenClipboard())
        { /* open failed */
         MessageBeep(0);
         ::GlobalFree(gbl);
         return 0;
        } /* open failed */

     ::EmptyClipboard();

     UINT fmt;
#ifdef UNICODE
     fmt = CF_UNICODETEXT;
#else
     fmt = CF_TEXT;
#endif
     ::SetClipboardData(fmt, gbl);

     ::CloseClipboard();
     return 0;
    } // CHexEditCtrl::OnCopy

/****************************************************************************
*                          CHexEditCtrl::OnClear
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       0, always
* Effect: 
*       Means OnClear is never valid
****************************************************************************/

LRESULT CHexEditCtrl::OnClear(WPARAM, LPARAM)
    {
     return 0;
    } // CHexEditCtrl::OnClear

/****************************************************************************
*                            CHexEditCtrl::GetSel
* Inputs:
*       long & low:
*       long & high:
* Result: void
*       
* Effect: 
*       Gets the selection normalized to the text offset
****************************************************************************/

void CHexEditCtrl::GetSel(long & low, long & high)
    {
     long rawlow;
     long rawhigh;
     CRichEditCtrlEx::GetSel(rawlow, rawhigh);

     if(rawlow == rawhigh)
        { /* same */
         low = high = rawlow / width;
        } /* same */
     else
        { /* different */
         low = rawlow / width; 
         high = rawhigh / width; 
        } /* different */
    } // CHexEditCtrl::GetSel

/****************************************************************************
*                         CHexEditCtrl::SetTextColor
* Inputs:
*       int i: "Character" position to change color of
*       COLORREF color: Color to set it to
* Result: void
*       
* Effect: 
*       Changes the text color
****************************************************************************/

void CHexEditCtrl::SetTextColor(int i, COLORREF color)
    {
     MB_TRACE2("=>CHexEditCtrl::SetTextColor(%d, 0x%08x)\n", i, color);
     DWORD events = GetEventMask();
     SetEventMask(events & ~(ENM_PROTECTED | ENM_SELCHANGE));

     CHARFORMAT fmt;
     fmt.cbSize = sizeof(fmt);
     fmt.dwMask = CFM_BOLD | CFM_COLOR;
     fmt.crTextColor = color;
     fmt.dwEffects = CFE_BOLD;

     long low;
     long high;
     CRichEditCtrlEx::GetSel(low, high); // Get underlying selection

     SetSel(i, i + 1); // set character-relative selection
     BOOL ok = SetSelectionCharFormat(fmt);

     MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", low, high);
     CRichEditCtrlEx::SetSel(low, high); // Restore original selection
     MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");
     
     SetEventMask(events);
     
     MB_TRACE0("<=CHexEditCtrl::SetTextColor\n");
    } // CHexEditCtrl::SetTextColor

/****************************************************************************
*                         CHexEditCtrl::SetUnderline
* Inputs:
*       int i: Virtual character position
*       BOOL extend: TRUE to extend into space, FALSE to not extend
*       BOOL on: TRUE to turn on underline, FALSE to turn off underline
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void CHexEditCtrl::SetUnderline(int i, BOOL extend, BOOL on)
    {
     MB_TRACE3("=>CHexEditCtrl::SetUnderline(%d, %s, %s\n", i, extend ? _T("TRUE") : _T("FALSE"),
                                                          on ? _T("TRUE") : _T("FALSE"));
     DWORD events = GetEventMask();
     SetEventMask(events & ~(ENM_PROTECTED | ENM_SELCHANGE));
     
     CHARFORMAT fmt = { sizeof(CHARFORMAT) };
     fmt.dwMask = CFM_UNDERLINE;
     fmt.dwEffects = on ? CFE_UNDERLINE : 0;

     //----------------------------------------------------------------
     // Retrieve the original selection
     //----------------------------------------------------------------
     long olow;
     long ohigh;
     CRichEditCtrlEx::GetSel(olow, ohigh);

     //----------------------------------------------------------------
     // Set the virtual character selection
     //----------------------------------------------------------------
     SetSel(i, i + 1);

     //----------------------------------------------------------------
     // Retrieve the underlying raw selection
     // If extend is false, decrement the upper bound
     //----------------------------------------------------------------
     long rawlow;
     long rawhigh;
     CRichEditCtrlEx::GetSel(rawlow, rawhigh);
     if(!extend)
        rawhigh--;

     MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", rawlow, rawhigh);
     CRichEditCtrlEx::SetSel(rawlow, rawhigh);
     MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");
     //----------------------------------------------------------------
     // Underline the text
     //----------------------------------------------------------------
     MB_TRACE0("=>SetSelectionCharFormat(CFE_UNDERLINE)\n");
     BOOL ok = SetSelectionCharFormat(fmt);
     MB_TRACE0("<=SetSelectionCharFormat\n");

     //----------------------------------------------------------------
     // Restore the original selection
     //----------------------------------------------------------------
     MB_TRACE2("=>CRichEditCtrlEx::SetSel(%d, %d)\n", olow, ohigh);
     CRichEditCtrlEx::SetSel(olow, ohigh);
     MB_TRACE0("<=CRichEditCtrlEx::SetSel\n");
     
     SetEventMask(events);
     MB_TRACE0("<=CHexEditCtrl::SetUnderline\n");
    } // CHexEditCtrl::SetUnderline

/****************************************************************************
*                         CHexEditCtrl::OnContextMenu
* Inputs:
*       CWnd * wnd: ignored
*       CPoint point:
* Result: void
*       
* Effect: 
*       Pops up a context menu
****************************************************************************/

void CHexEditCtrl::OnContextMenu(CWnd* pWnd, CPoint point) 
   {
    CMenu menu;
    
    long low;
    long high;
    GetSel(low, high);
    BOOL hasSelection = (low < high);
    int length = GetWindowTextLength();
    BOOL empty = length == 0;
    BOOL allSelected = (low == 0 && high == length);

    CString caption = GetCaption(this);
    BOOL what = CHelp::FindHelpResource(caption) != NULL;

    menu.LoadMenu(IDR_EDIT_POPUP);
    menu.EnableMenuItem(ID_EDIT_UNDO,    MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
    menu.EnableMenuItem(ID_EDIT_CUT,     MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
    menu.EnableMenuItem(ID_EDIT_COPY,    MF_BYCOMMAND | (hasSelection ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(ID_EDIT_PASTE,   MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
    menu.EnableMenuItem(ID_EDIT_DELETE,  MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
    menu.EnableMenuItem(ID_EDIT_SELECTALL,MF_BYCOMMAND | (!empty    && !allSelected     ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(IDM_WHAT_IS_THIS, MF_BYCOMMAND | (what                          ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));

#ifndef _DEBUG
    menu.DeleteMenu(IDM_DEBUG_INFO, MF_BYCOMMAND);
    //menu.DeleteMenu(IDM_HEX, MF_BYCOMMAND);
#endif
    CMenu * popup = menu.GetSubMenu(0);

    popup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this); 
    
}

/****************************************************************************
*                         CHexEditCtrl::OnWhatIsThis
* Result: void
*       
* Effect: 
*       Pops up the help
****************************************************************************/

void CHexEditCtrl::OnWhatIsThis()
    {
     CString s = ::GetCaption(this);
     CHelp dlg;
     dlg.m_HelpName = s;
     dlg.DoModal();
    } // CHexEditCtrl::OnWhatIsThis

/****************************************************************************
*                          CHexEditCtrl::OnEditCopy
* Result: void
*       
* Effect: 
*       Performs a copy operation
****************************************************************************/

void CHexEditCtrl::OnEditCopy()
    {
     SendMessage(WM_COPY);
    } // CHexEditCtrl::OnEditCopy

/****************************************************************************
*                        CHexEditCtrl::OnEditSelectAll
* Result: void
*       
* Effect: 
*       Selects everything
****************************************************************************/

void CHexEditCtrl::OnEditSelectAll()
    {
     int length = GetWindowTextLength();
     SetSel(0, length);
    } // CHexEditCtrl::OnEditSelectAll
