// Index.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "Index.h"
#include "IndexInfo.h"
#include "Query.h"
#include "TabView.h"
#include "Help.h"
#include "WhatIsItButton.h"
#include "WhatIsItSpinButtonCtrl.h"
#include "WhatIsThisEdit.h"
#include "pageref.h"
#include "Initializing.h"
#include "BiFile.h"
#include "RegVars.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CIndex, CFunction)


/////////////////////////////////////////////////////////////////////////////
// CIndex dialog

CIndex::CIndex(CWnd* pParent /*=NULL*/)
        : CFunction(CIndex::IDD, pParent)
{
        //{{AFX_DATA_INIT(CIndex)
                // NOTE: the ClassWizard will add member initialization here
        //}}AFX_DATA_INIT
}


void CIndex::DoDataExchange(CDataExchange* pDX)
{
        CFunction::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CIndex)
        DDX_Control(pDX, IDC_INDEX, c_Index);
        DDX_Control(pDX, IDC_FASTINDEX, c_FastIndex);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIndex, CFunction)
        ON_REGISTERED_MESSAGE(UWM_INDEX, OnIndex)
        //{{AFX_MSG_MAP(CIndex)
        ON_WM_SIZE()
        ON_NOTIFY(EN_LINK, IDC_INDEX, OnLinkIndex)
        ON_NOTIFY(EN_LINK, IDC_FASTINDEX, OnLinkFastIndex)
        ON_WM_DESTROY()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIndex message handlers

/****************************************************************************
*                          CIndex::SetFontToControls
* Result: void
*       
* Effect: 
*       Does nothing. Required to be here in case a font is required
****************************************************************************/

void CIndex::SetFontToControls()
    {
     
    } // CIndex::SetFontToControls

/****************************************************************************
*                               CIndex::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Expands the window
****************************************************************************/

void CIndex::OnSize(UINT nType, int cx, int cy) 
   {
    CFunction::OnSize(nType, cx, cy);
    if(c_FastIndex.GetSafeHwnd() != NULL)
       { /* resize */
        CRect c;
        c_FastIndex.GetWindowRect(&c);
        ScreenToClient(&c);
        c_FastIndex.SetWindowPos(NULL, 0, 0, c.Width(), cy - c.top, SWP_NOMOVE | SWP_NOZORDER);
       } /* resize */
        
    ExpandBottom(c_Index);
   }

/****************************************************************************
*                               CIndex::OnIndex
* Inputs:
*       WPARAM: (WPARAM)(ReferenceTable *) A reference table
*       LPARAM: (LPARAM)(CInitializing *) The initializing window
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Handles the creation of the index contents page
****************************************************************************/

LRESULT CIndex::OnIndex(WPARAM wParam, LPARAM lParam)
    {
     CInitializing * progress = (CInitializing *)lParam;
     if(CanLoadFromCache() && RetrieveFromCache())
        { /* all done */
         progress->DestroyWindow();
         return 0; // we have loaded the index from the cache
        } /* all done */
     
     if(GetSafeHwnd() == NULL)
        { /* no indexing */
         progress->DestroyWindow();
         return 0; // do not index if page was not created
        } /* no indexing */
     
     ReferenceTable & refs = *(ReferenceTable *)wParam;

     progress->SetLimit(refs.GetSize() + 1); // add 1 for the sorting step
     progress->SetPos(0);
     CString s;
     s.LoadString(IDS_SORTING);
     progress->StepIt(s);
     progress->UpdateWindow(); // make sure window stays visible
     refs.Sort();

     CString last;
     CString lastsub;
     TCHAR lastLetter = _T('\0');
     CString text;
     CString sep;
     CString newEntry;
     CString indexing;
     indexing.LoadString(IDS_INDEXING);
     
     TRACE0("===========================starting display=============================\n");
     for(int i = 0; i < refs.GetSize(); i++)
        { /* show each */
         CString line;
         line = refs[i]->text;
         if(line.IsEmpty())
             continue;

#ifdef _DEBUG  // ----------------------------------------------------------------+
         // In debug mode, track each item being added                         // |
         progress->StepIt(refs[i]->text);                                      // |
#endif // ------------------------------------------------------------------------+
         //TRACE2("%3d: \"%s\",", i, refs[i]->text);
         //TRACE2("\"%s\", \"%s\")\n", refs[i]->textAs, CHelp::MakeHelpName(refs[i]->text));
         CString lineAs = refs[i]->textAs;
         if(lineAs != last)
            { /* new element */
             TCHAR thisletter = lineAs[0];

             // Force all numbers to sort to same category
             if(_istdigit(thisletter))
                thisletter = _T('0'); // force all numbers to same heading

             // Look for category change
             if(lastLetter != _totupper(thisletter))
                { /* add new caption */
                 lastLetter = _totupper(line[0]);
                 // The letter has changed
                 // End the current line, if there was one

#ifndef _DEBUG // in release mode, just add letter when it changes
                 indexing += lastLetter;
                 progress->StepIt(indexing);
#endif                 
                 c_Index.Append(newEntry);
                 c_FastIndex.Append(newEntry);
                 newEntry = _T("\r\n");

                 // Create a line which is the first letter of the index items
                 // that follow. Change its font and color to be heading font
                 // and color
                 // Note that if it is numeric, the category is always the
                 // same, so if this is the first number we set the caption
                 // to 0..9
                 CHARRANGE caption;
                 CString CaptionText;
                 if(_istdigit(thisletter))
                    CaptionText = _T("0..9");
                 else
                    CaptionText = _totupper(line[0]);

                 caption = c_Index.Append(CaptionText);
                 c_Index.Append(_T("\r\n"));
                 //TRACE2("Caption %d..%d\n", caption.cpMin, caption.cpMax);

                 CaptionText = _T(" ") + CaptionText + _T(" "); // make it big enough to click on!
                 CHARRANGE IndexCaption = c_FastIndex.Append(CaptionText);
                 fastlinks.Add(FastIndexItem(IndexCaption, caption));

                 // Center the text in the fast-access panel
                 c_FastIndex.SetSel(IndexCaption);
                 PARAFORMAT fast = { sizeof(PARAFORMAT) };
                 fast.dwMask = PFM_ALIGNMENT;
                 fast.wAlignment = PFA_CENTER;
                 c_FastIndex.SetParaFormat(fast);

                 // Now format the index entry and link it to the main panel
                 CHARFORMAT ixfmt = { sizeof(CHARFORMAT) } ;
                 c_FastIndex.SetSel(IndexCaption);
                 ixfmt.dwMask = CFM_BOLD | CFM_SIZE | CFM_UNDERLINE | CFM_LINK;
                 ixfmt.dwEffects = CFE_BOLD | CFE_UNDERLINE | CFE_LINK;
                 ixfmt.yHeight = 20 * 10;
                 c_FastIndex.SetSelectionCharFormat(ixfmt);
                 
                 // Format the heading
                 // Paragraph format centers it
                 c_Index.SetSel(caption);
                 PARAFORMAT para = {sizeof(PARAFORMAT)};
                 para.dwMask = PFM_ALIGNMENT;
                 para.wAlignment = PFA_CENTER;
                 c_Index.SetParaFormat(para);
                 // Character format makes it large and in the right color
                 CHARFORMAT hdrfmt = {sizeof(CHARFORMAT) };
                 hdrfmt.dwMask = CFM_BOLD | CFM_COLOR | CFM_SIZE;
                 hdrfmt.dwEffects = CFE_BOLD;
                 hdrfmt.crTextColor = RGB(0, 255, 0);
                 hdrfmt.yHeight = 20*14;
                 c_Index.SetSelectionCharFormat(hdrfmt);
                } /* add new caption */
             else
                 c_Index.Append(_T("\r\n"));

             // Append the item to the line
             CHARRANGE phrase = c_Index.Append(line);
             c_Index.Append(_T(" "));
             //TRACE3("Phrase \"%s\"%d..%d\n", line, phrase.cpMin, phrase.cpMax);
             last = lineAs;
             CHARFORMAT chars = {sizeof(CHARFORMAT)};
             chars.dwMask = CFM_BOLD;
             chars.dwEffects = CFE_BOLD;
             c_Index.SetSel(phrase);
             c_Index.SetSelectionCharFormat(chars);
             PARAFORMAT para = {sizeof(PARAFORMAT)};
             para.dwMask = PFM_ALIGNMENT;
             para.wAlignment = PFA_LEFT;
             c_Index.SetParaFormat(para);
             lastsub = _T("");
             sep = _T("");
#ifdef _DEBUG // ----------------------------------------------------------------+
             // In the debug version, we check to see that                    // |
             // all linkable items have proper links                          // |
             CWnd * w = refs[i]->wnd;                                         // |
             if(   w->IsKindOf(RUNTIME_CLASS(CWhatIsItButton))                // |
                || w->IsKindOf(RUNTIME_CLASS(CWhatIsThisEdit))                // |
                || w->IsKindOf(RUNTIME_CLASS(CWhatIsItSpinButtonCtrl))        // |
               )                                                              // |
                { /* questionable */                                          // |
                 // The cursor will display a "?" when it moves               // |
                 // over this item                                            // |
                 // If we can't find a link for its text, put it in           // |
                 // a different color                                         // |
                 if(CHelp::FindHelpResource(refs[i]->text) == NULL)           // |
                    { /* not found */                                         // |
                     c_Index.Append(_T("*******************"));               // |
                    } /* not found */                                         // |
                } /* questionable */                                          // |
#endif // _DEBUG //--------------------------------------------------------------+
            } /* new element */

         // Now add the hyperlinks, unless it is a duplicate
         CString linkname = GetPageName(refs[i]->page);
         refs[i]->index = GetPageIndex(refs[i]->page);
         if(linkname != lastsub)
            { /* new subitem */
             c_Index.Append(sep);
             c_Index.Append(_T(" "));
             CHARRANGE link = c_Index.Append(linkname);
             CHARFORMAT2 linkfmt;
             linkfmt.dwMask = CFM_COLOR | CFM_UNDERLINE | CFM_LINK;
             linkfmt.dwEffects = CFE_UNDERLINE | CFE_LINK;
             linkfmt.crTextColor = RGB(0, 0, 255);
             //linkfmt.crTextColor = RGB(0, 255, 0);
             c_Index.SetSel(link);
             c_Index.SetSelectionCharFormat(linkfmt);
             refs[i]->range = link;
             //TRACE1("%s ", line);
             hyperlinks.Add(refs[i]);
             refs[i] = NULL; // we have safely copied it across
             lastsub = linkname;
             sep = _T(",");
            } /* new subitem */
         else
            { /* duplicate subitem */
             delete refs[i]; // free up object
             refs[i] = NULL; // it no longer exists
             refs.RemoveAt(i); // remove duplicate
             i--;
            } /* duplicate subitem */
        } /* show each */

     c_Index.SetSel(0,0);
     c_FastIndex.SetSel(0,0);

     progress->DestroyWindow();

     //****************************************************************
     // Now, for any elements that were not copied across to the hyperlinks table,
     // remove the element from the array so we don't have a storage leak
     // Then remove the entire contents of the refs table
     
     for(i = 0; i < refs.GetSize(); i++)
        { /* scan for unfreed */
         delete refs[i];
         refs[i] = NULL;
        } /* scan for unfreed */

     refs.RemoveAll();
     //****************************************************************
     
     SaveToCache();

     return 0;
    } // CIndex::OnIndex

/****************************************************************************
*                               CIndex::Refresh
* Result: void
*       
* Effect: 
*       Handles various housekeeping tasks
****************************************************************************/

void CIndex::Refresh()
    {
     EnableQuery(QUERY_DWORD, FALSE);
     EnableQuery(QUERY_NOUSEROVERRIDE, FALSE);
     EnableQuery(QUERY_USE_CP_ACP, FALSE);
     
     QueryTemplate(_T(""));
     SetCode(_T(""), FALSE);
    } // CIndex::Refresh

/****************************************************************************
*                            ReferenceTable::Sort
* Result: void
*       
* Effect: 
*       Sorts the contents of the reference table
****************************************************************************/

void ReferenceTable::Sort()
    {
     qsort(GetData(), GetSize(), sizeof(Reference *), compare);
    } // ReferenceTable::Sort

/****************************************************************************
*                           ReferenceTable::compare
* Inputs:
*       const void * e1:
*       const void * e2:
* Result: int
*       <0 if e1 < e2
*       ==0 if e1 == e2
*       >0 if e1 > e2
****************************************************************************/

/* static */ int ReferenceTable::compare(const void * e1, const void * e2)
    {
     Reference * r1 = *(Reference **)e1;
     Reference * r2 = *(Reference **)e2;

     CString t1 = r1->textAs;
     CString t2 = r2->textAs;
     int n = lstrcmpi(t1, t2);
     if(n != 0)
        return n;

     CString link1 = GetPageName(r1->page);
     CString link2 = GetPageName(r2->page);
     n = lstrcmpi(link1, link2);
     return n;
    } // ReferenceTable::compare

/****************************************************************************
*                             CIndex::OnLinkIndex
* Inputs:
*       NMHDR * pNMHDR
* Result: void
*       
* Effect: 
*       Handles a link event
****************************************************************************/

void CIndex::OnLinkIndex(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    ENLINK * event = (ENLINK *)pNMHDR;
    if(event->msg == WM_LBUTTONDOWN)
       { /* do hyperlink */
#if 0 //----------------------------------------------------------------+
        CString s;                                                   // |
        s.Format(_T("%d, %d"), event->chrg.cpMin, event->chrg.cpMax);// |
        AfxMessageBox(s);                                            // |
#endif //---------------------------------------------------------------+
        for(int i = 0; i < hyperlinks.GetSize(); i++)
           { /* scan links */
            if(hyperlinks[i]->IsMatch(event->chrg))
               { /* hit it */
                //CTabView * parent = (CTabView *)hyperlinks[i]->page->GetParent();
                CTabView * parent = (CTabView *)GetParent(); // parent of all pages is same as parent of this page
                parent->ActivatePage(hyperlinks[i]->index);
                break;
               } /* hit it */
           } /* scan links */
       } /* do hyperlink */
    *pResult = 0;
   }

/****************************************************************************
*                           CIndex::OnLinkFastIndex
* Inputs:
*       NMHDR * pNMHDR: (NMHDR*)(ENLINK *) Link event
*       LPRESULT * pResult: Result location
* Result: void
*       
* Effect: 
*       Selects the letter
****************************************************************************/

void CIndex::OnLinkFastIndex(NMHDR* pNMHDR, LRESULT* pResult) 
    {
     ENLINK * event = (ENLINK *)pNMHDR;
     if(event->msg == WM_LBUTTONDOWN)
        { /* do hyperlink */
         for(int i = 0; i < fastlinks.GetSize(); i++)
            { /* scan links */
             if(fastlinks[i].IsMatch(event->chrg))
                { /* hit it */
                 CHARRANGE r = fastlinks[i].range;
                 c_Index.SetSel(r);
                 int pos = fastlinks[i].index; // selection.cpMin;
                 long line = c_Index.LineFromChar(pos);
                 int visible = c_Index.GetFirstVisibleLine();
                 // Scroll until line is visible
                 // We have the following cases
                 // line < visible
                 //     Scroll towards beginning until current line is visible
                 //     LineScroll(line - visible);
                 // line = visible
                 //     do nothing
                 // line > visible
                 //     Scroll towards end until current line is visible
                 //     LineScroll(line - visible);
                 c_Index.LineScroll(line - visible);
                 break;
                } /* hit it */
            } /* scan links */
        } /* do hyperlink */
     *pResult = 0;
    } // CIndex::OnLinkFastIndex

/****************************************************************************
*                              CIndex::QuerySave
* Result: BOOL
*       TRUE, always
****************************************************************************/

BOOL CIndex::QuerySave()
    {
     return TRUE;
    } // CIndex::QuerySave

/****************************************************************************
*                                CIndex::Save
* Inputs:
*       const CString & filename: File name
*       UINT mode: Save mode: 0 (defaullt), 8 (UTF-8), 16 (UTF-16)
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Saves the index
****************************************************************************/

BOOL CIndex::Save(const CString & filename, UINT mode)
    {
     CBiFile f;
     if(!f.Open(filename, CFile::modeWrite | CFile::modeCreate, mode))
        { /* failed */
         return FALSE;
        } /* failed */

     CString s;
     c_Index.GetWindowText(s);

     try
        { /* try */
         f.WriteBOM();
         f.WriteString(s);
        } /* try */
     catch (CFileException * e)
        { /* failed */
         e->Delete();
         f.Close();
         return FALSE;
        } /* failed */
     f.Close();
     return TRUE;
    } // CIndex::Save

/****************************************************************************
*                              CIndex::OnDestroy
* Result: void
*       
* Effect: 
*       Cleans up the data structures so we avoid memory leak reports
****************************************************************************/

void CIndex::OnDestroy() 
   {
    CFunction::OnDestroy();
        
    for(int i = 0; i < hyperlinks.GetSize(); i++)
       delete hyperlinks[i];
    
    hyperlinks.RemoveAll();
    fastlinks.RemoveAll();
   }

/****************************************************************************
*                           CIndex::StreamOutCallback
* Inputs:
*       DWORD_PTR cookie: The cookie
*       LPBYTE buffer: Buffer pointer
*       LONG count: Number of bytes
*       LLONG * pcb: Number of bytes actually written
* Result: DWORD
*       0
* Effect: 
*       Writes the RTF text to the stream.  On each operation, updates the
*       length in the first DWORD to represent the number of bytes in the file
****************************************************************************/

/* static */ DWORD CALLBACK CIndex::StreamOutCallback(DWORD_PTR cookie, LPBYTE buffer, LONG count, LONG * pcb)
    {
     StreamOutCookie * StreamOut = (StreamOutCookie *)cookie;

     StreamOut->file->Write(buffer, count);

     DWORD_PTR pos = StreamOut->file->GetPosition(); // get current position (files are all < 4.2GB)

     StreamOut->file->Seek(StreamOut->LengthOffset, CFile::begin);  // go back to length word

     try {
          StreamOut->file->Write(&pos, sizeof(DWORD));
         }
     catch(CFileException * e)
        { /* failed */
         StreamOut->err = e->m_lOsError;
         *pcb = 0;
         e->Delete();
         return 1;
        } /* failed */
     
     StreamOut->file->SeekToEnd();  // reposition at end

     *pcb = count;
     StreamOut->err = ERROR_SUCCESS;
     return 0;
    } // CIndex::StreamOutCallback

/****************************************************************************
*                          CIndex::StreamInCallback
* Inputs:
*       DWORD_PTR cookie: Cookie
*       LPBYTE buffer: Pointer to buffer
*       LONG count: Count of bytes in buffer
*       LONG * pcb: Count of bytes actually read
* Result: DWORD
*       0 to indicate successful completion
*       nonzero to indicate termination
* Effect: 
*       Reads bytes into the buffer
****************************************************************************/

/* static */ DWORD CALLBACK CIndex::StreamInCallback(DWORD_PTR cookie, LPBYTE buffer, LONG count, LONG * pcb)
    {
     StreamInCookie * streamIn = (StreamInCookie *)cookie;
     if(streamIn->remaining == 0)
        { /* all done */
         *pcb = 0;
         streamIn->err = ERROR_SUCCESS;
         return 1; // nonzero value terminates read
        } /* all done */

     DWORD bytesToRead = min(streamIn->remaining, (DWORD)count);

     UINT bytesRead;
     try {
          bytesRead = streamIn->file->Read(buffer, bytesToRead);
         }
     catch(CFileException * e)
         { /* catch */
          streamIn->err = e->m_lOsError;
          e->Delete();
          streamIn->remaining = 0;
          return 1;
         } /* catch */

     if(bytesRead == 0)
        { /* read error */
         streamIn->err = ::GetLastError();
         *pcb = bytesRead;
         return 1; // return nonzero to stop operation
        } /* read error */

     streamIn->remaining -= bytesRead;
     *pcb = bytesRead;
     streamIn->err = ERROR_SUCCESS;
     return 0;
    } // CIndex::StreamInCallback

/****************************************************************************
*                             CIndex::SaveToCache
* Result: void
*       
* Effect: 
*       Attempts to write the data to the cache
****************************************************************************/

void CIndex::SaveToCache() 
    {
     CString filename;
     if(!GetCacheFile(filename))
        return; // can't save to cache

     CFile output;
     if(!output.Open(filename, CFile::modeCreate | CFile::modeWrite))
        return;  // can't open cache

     try
        { /* try output */
         //----------------------------------------------------------------
         //  [TimeStamp] Timestamp of executable
         //  [eod   ]    Offset of end of RTF data ----+
         //  <...>       RTF data      <---------------+                
         //  [count    ] count of hyperlinks
         //  <...>       Hyperlink data
         //  [eod      ] Offset of end of RTF data ----+
         //  <...>       RTF hyperlink data  <---------+
         //  [count    ] Count of hyperlinks
         //  <...>       Hyperlink data
         //----------------------------------------------------------------

         //****************************************************************
         // [TimeStamp]
         //****************************************************************

         TimeStamp ZeroTimeStamp;
         output.Write(&ZeroTimeStamp, sizeof(ZeroTimeStamp));

         StreamOutCookie OutStream;
         OutStream.file = &output;
         OutStream.LengthOffset = output.GetPosition();
         OutStream.err = 0;

         //****************************************************************
         // [eod   ]  Offset of end-of-data for RTF data
         //****************************************************************
         DWORD zero = 0; // start out with 0 length
         output.Write(&zero, sizeof(DWORD));

         EDITSTREAM es;
         es.dwCookie = (DWORD_PTR)&OutStream;
         es.pfnCallback = StreamOutCallback;

         //****************************************************************
         // <...>   RTF data
         //****************************************************************
         c_Index.StreamOut(SF_RTF, es);

         if(OutStream.err != ERROR_SUCCESS)
            { /* get out */
             output.Close();
             return;
            } /* get out */

         //****************************************************************
         // [count    ] count of hyperlinks
         //****************************************************************

         DWORD len = hyperlinks.GetSize();
         output.Write(&len, sizeof(len));

         //****************************************************************
         // <...>      hyperlink data
         //****************************************************************

         for(int i = 0; i < hyperlinks.GetSize(); i++)
            { /* write each hyperlink */
             hyperlinks[i]->Write(output);
            } /* write each hyperlink */

         //****************************************************************
         // [eod    ]  End-of-data of RTF data
         //****************************************************************

         OutStream.LengthOffset = output.GetPosition();
         output.Write(&zero, sizeof(DWORD));  // write out 0, will update in stream handler

         //****************************************************************
         // <...>      RTF data for fastindex
         //****************************************************************
         c_FastIndex.StreamOut(SF_RTF, es);

         //****************************************************************
         // [count] Count of hyperlinks
         //****************************************************************
         len = fastlinks.GetSize();
         output.Write(&len, sizeof(len));

         //****************************************************************
         // <...>  hyperlink data
         //****************************************************************

         for(i = 0; i < fastlinks.GetSize(); i++)
            { /* write each fastlink */
             fastlinks[i].Write(output);
            } /* write each fastlink */

         //****************************************************************
         // I/O is complete, write the timestamp
         //****************************************************************

         TimeStamp t;
         if(!GetExeTimeStamp(t))
            { /* failed */
             output.Close();
             return;
            } /* failed */

         output.SeekToBegin();  // overwrite dummy timestamp with valid one
         output.Write(&t, sizeof(t));
        } /* try output */
     catch(CFileException * e)
        { /* failed */
         // TODO: do we want to notify user here, or fail silently?
         // For now, fail silently
         e->Delete();
        } /* failed */

     output.Close();
    } // CIndex::SaveToCache

/****************************************************************************
*                              CIndex::MarkLink
* Inputs:
*       CRichEditCtrlEx & ctl:
*       CHARRANGE & range:
* Result: void
*       
* Effect: 
*       Marks the hyperlink by setting the CFE_LINK bit
****************************************************************************/

void CIndex::MarkLink(CRichEditCtrlEx & ctl, CHARRANGE & range)
    {
     CHARFORMAT2 linkfmt;

     ctl.SetSel(range);

     linkfmt.cbSize = sizeof(CHARFORMAT2);
     linkfmt.dwMask = CFM_LINK;
     //c_Index.GetSelectionCharFormat(linkfmt);                            // |
     linkfmt.dwEffects = CFE_LINK;
     linkfmt.dwMask = CFM_LINK;
     ctl.SetSelectionCharFormat(linkfmt);
    } // CIndex::MarkLink

/****************************************************************************
*                          CIndex::RetrieveFromCache
* Result: BOOL
*       TRUE if successful
*       FALSE if cannot be found
* Effect: 
*       Attempts to retrieve the data from the cache
****************************************************************************/

BOOL CIndex::RetrieveFromCache()
    {
     CString filename;
     if(!GetCacheFile(filename))
        return FALSE;
     
     CFile f;
     
     if(!f.Open(filename, CFile::modeRead))
        return FALSE; // open failed

     try { /* try read */
          StreamInCookie streamIn; //
          streamIn.file = &f;

          //****************************************************************
          // [Timestamp]
          //****************************************************************

          TimeStamp ft;
          if(f.Read(&ft, sizeof(TimeStamp)) == 0)
             { /* failed */
              ResetAllContent();
              f.Close();
              return FALSE;
             } /* failed */

          //****************************************************************
          //  [eod]   End of data position for RTF data
          //****************************************************************

          if(f.Read(&streamIn.remaining, sizeof(DWORD)) == 0)
             { /* failed */
              ResetAllContent();
              f.Close();
              return FALSE;
             } /* failed */

          // Now convert from offset to length
          streamIn.remaining -= f.GetPosition();

          //****************************************************************
          // <...> RTF data
          //****************************************************************
          EDITSTREAM es;
          es.dwCookie = (DWORD_PTR)&streamIn;
          es.pfnCallback = StreamInCallback;
          c_Index.StreamIn(SF_RTF, es); 

          if(streamIn.err != ERROR_SUCCESS)
             { /* failed */
              f.Close();
              ResetAllContent();
              return FALSE;
             } /* failed */

          //****************************************************************
          // Read the hyperlink length
          //****************************************************************
          DWORD len;
          if(f.Read(&len, sizeof(DWORD)) == 0)
             { /* failed to get links */
              f.Close();
              ResetAllContent();
              return FALSE;
             } /* failed to get links */

          hyperlinks.SetSize(len);

          //****************************************************************
          // Read the hyperlink data
          //****************************************************************

          for(DWORD i = 0; i < len; i++)
             { /* read each */
              hyperlinks[i] = new Reference;
              if(!hyperlinks[i]->Read(f))
                 { /* failed */
                  if(::GetLastError() == ERROR_HANDLE_EOF)
                     break; // "failure" is EOF (see spec on Reference::Read)
                  f.Close();
                  ResetAllContent();
                  return FALSE;
                 } /* failed */
              MarkLink(c_Index, hyperlinks[i]->range);
             } /* read each */

          //****************************************************************
          // [eod] Read EOD for Fastlink RTF data
          //****************************************************************

          if(f.Read(&streamIn.remaining, sizeof(DWORD)) == 0)
             { /* failed */
              ResetAllContent();
              f.Close();
              return FALSE;
             } /* failed */

          streamIn.remaining -= f.GetPosition(); // convert from offset to length

          //****************************************************************
          // <...> RTF data
          //****************************************************************
          c_FastIndex.StreamIn(SF_RTF, es);

          if(streamIn.err != ERROR_SUCCESS)
             { /* failed */
              ResetAllContent();
              f.Close();
              return FALSE;
             } /* failed */

          //****************************************************************
          // Read the hyperlink length
          //****************************************************************

          if(f.Read(&len, sizeof(DWORD)) == 0)
             { /* failed to get links */
              ResetAllContent();
              f.Close();
              return FALSE;
             } /* failed to get links */

          fastlinks.SetSize(len);

          //****************************************************************
          // Read the hyperlink data
          //****************************************************************

          for(i = 0; i < len; i++)
             { /* read each */
              if(!fastlinks[i].Read(f))
                 { /* failed */
                  if(::GetLastError() == ERROR_HANDLE_EOF)
                     break;
                  f.Close();
                  ResetAllContent();
                  return FALSE;
                 } /* failed */
              MarkLink(c_FastIndex, fastlinks[i].range);
             } /* read each */
          //****************************************************************
         } /* try read */
     catch(CFileException * e)
        { /* read error */
         e->Delete();
         ResetAllContent();
         f.Close();
         return FALSE;
        } /* read error */
     f.Close();
     c_Index.SetSel(0,0);
     c_FastIndex.SetSel(0,0);
     return TRUE;
    } // CIndex::RetrieveFromCache

/****************************************************************************
*                          CIndex::CanLoadFromCache
* Result: BOOL
*       TRUE if the cache is valid
*       FALSE if the cache does not exist or is not valid
****************************************************************************/

BOOL CIndex::CanLoadFromCache()
    {
     TimeStamp exetime;
     GetExeTimeStamp(exetime);
#ifdef _DEBUG
     LONGLONG lt = exetime.GetTime();
     TRACE(_T("%016I64x\n"), lt);
#endif
     CString cachename;
     if(!GetCacheFile(cachename))
        return FALSE;

     CFile f;
     if(!f.Open(cachename, CFile::modeRead))
        { /* failed */
         return FALSE;
        } /* failed */

     FILETIME ft;
     DWORD bytesRead;
     try {
        bytesRead = f.Read(&ft, sizeof(ft));
     }
     catch(CFileException * e)
        { /* read failed */
         f.Close();
         e->Delete();
         return FALSE;
        } /* read failed */

     if(bytesRead != sizeof(ft))
        { /* wrong length */
         f.Close();
         return FALSE;
        } /* wrong length */
     f.Close();

     TimeStamp cachetime(ft); 
#ifdef _DEBUG
     lt = cachetime.GetTime();
     TRACE(_T("%016I64x\n"), lt);
#endif

#ifdef _DEBUG //----------------------------------------------------------------+
     CString exetimeString = exetime.Format();                               // |
     CString cachetimeString = cachetime.Format();                           // |
     TCHAR exepath[MAX_PATH];                                                // |
     ::GetModuleFileName(NULL, exepath, MAX_PATH);                           // |
                                                                             // |
     CString msg;                                                            // |
     msg = exepath;                                                          // |
     msg += _T("\r\n");                                                      // |
     msg += exetimeString;                                                   // |
     msg += _T("\r\n");                                                      // |
     msg += cachename;                                                       // |
     msg += _T("\r\n");                                                      // |
     msg += cachetimeString;                                                 // |
#if 0                                                                        // |
     AfxMessageBox(msg, MB_ICONINFORMATION);                                 // |
#else                                                                        // |
     TRACE(_T("%s\n"), msg);                                                 // |
#endif                                                                       // |
#endif //-----------------------------------------------------------------------+

     if(exetime != cachetime)
        return FALSE;

     return TRUE; // cache is valid
    } // CIndex::CanLoadFromCache

/****************************************************************************
*                            CIndex::GetCacheFile
* Inputs:
*       CSting & name: Place to write cache name
* Result: BOOL
*       TRUE if the cache name is valid
*       FALSE if the cache name is not valid
* Effect: 
*       Computes the full path name of the cache file
****************************************************************************/

BOOL CIndex::GetCacheFile(CString & name)
    {
     RegistryString path(IDS_REGISTRY_CACHE_PATH);
     BOOL found = path.load();
     
     if(!found || path.value.IsEmpty())
        { /* use default path */
         TCHAR exepath[MAX_PATH];
         if(!::GetModuleFileName(NULL, exepath, MAX_PATH))
            return FALSE;
     
         TCHAR drive[MAX_PATH];
         TCHAR dir[MAX_PATH];

         _tsplitpath(exepath, drive, dir, NULL, NULL);

         _tmakepath(exepath, drive, dir, _T("index"), _T(".dat"));
         name = exepath;
        } /* use default path */
     else
        { /* use specified path */
         path.value.TrimRight();
         if(path.value.Right(1) != _T("\\"))
            path.value += _T("\\");
         path.value += _T("index.dat");
         name = path.value;
        } /* use specified path */

     return TRUE;
    } // CIndex::GetCacheFile

/****************************************************************************
*                           CIndex::GetExeTimeStamp
* Inputs:
*       TimeStamp & t: Timestmap value
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Sets the timestamp value
* Notes:
*       This has to use ::CreateFile because there is no way for CFile to
*       specify there is no intention of opening the file for read or write
*
*       The ::GetFileTime has to look at the write time, because incremental
*       compilation (compile-and-go) does not change the creation time,
*       just the write time
****************************************************************************/

BOOL CIndex::GetExeTimeStamp(TimeStamp & t)
    {
     TCHAR exepath[MAX_PATH];
     ::GetModuleFileName(NULL, exepath, MAX_PATH);
     HANDLE h = ::CreateFile(exepath, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
     ASSERT(h != INVALID_HANDLE_VALUE);
     if(h == INVALID_HANDLE_VALUE)
        return FALSE;
#ifdef _DEBUG
     FILETIME ft;
     ::GetFileTime(h, NULL, NULL, &ft); // note use of write time 
     TRACE(_T("::GetFileTime => %08x %08x\n"), ft.dwHighDateTime, ft.dwLowDateTime);
#endif
     BOOL result = t.GetFileTime(h);
     ::CloseHandle(h);
     return result;
    } // CIndex::GetExeTimeStamp

/****************************************************************************
*                           CIndex::ResetAllContent
* Result: void
*       
* Effect: 
*       Clears all the index data
****************************************************************************/

void CIndex::ResetAllContent()
    {
     c_Index.SetWindowText(_T(""));
     c_FastIndex.SetWindowText(_T(""));
     for(int i = 0; i < hyperlinks.GetSize(); i++)
        delete hyperlinks[i];

     hyperlinks.SetSize(0);
     fastlinks.SetSize(0);
    } // CIndex::ResetAllContent
