#include "stdafx.h"
#include "IndexInfo.h"

/****************************************************************************
*                            Reference::Canonicalize
* Inputs:
*       CString & s: String
* Result: void
*       
* Effect: 
*       Changes the string to have numeric values expanded to six digits
*       This means that numbers will sort in numeric order instead of
*       alphabetical order.       
****************************************************************************/

/* static */ void Reference::Canonicalize(CString & s)
   {
    if(s.IsEmpty())
       return;
    if(!_istdigit(s[0]))
       return;
    CString num = s.SpanIncluding(_T("0123456789"));
    CString can;
    can.Format(_T("%06d"), _ttoi(can));
    s = can + s.Mid(num.GetLength());
   } // ReferenceTable::Canonicalize


/****************************************************************************
*                            Reference::Reference
* Inputs:
*       CString s: Text string which is the key
*       CFunction * p: Page on which it is located
*       CWnd * w: Window in page
* Effect: 
*       Creates an Reference entry. Normalizes the text
****************************************************************************/

Reference::Reference(CString s, CFunction * p, CWnd * w)
    {
     Set(s, s, p, w);
    } // Reference::Reference


/****************************************************************************
*                            Reference::Reference
* Inputs:
*       CString s: Text string which is the key
*       CString as: Text string on which we sort
*       CFunction * p: Page on which it is located
*       CWnd * w: Window in page
* Effect: 
*       Constructor
****************************************************************************/

Reference::Reference(CString s, CString as, CFunction * p, CWnd * w)
    {
     Set(s, as, p, w);
    } // Reference::Reference

/****************************************************************************
*                               Reference::Set
* Inputs:
*       const CString & s: String to display
*       const CString & as: String to sort on
* Result: void
*       
* Effect: 
*       Initializes the structure; called from the two different constructors
****************************************************************************/

void Reference::Set(const CString & s, const CString & as, CFunction * p, CWnd * w)
    {
     text = s;
     text.Replace(_T("&"), _T(""));
     textAs = as;
     Canonicalize(textAs);
     textAs.Replace(_T("&"), _T(""));
     page = p;
     wnd = w; 
     //TRACE2("Set(\"%s\", \"%s\")\n", text, textAs);
     LinkItem::Clear();
    } // Reference::Set
