#include "stdafx.h"
#include "OpenFileEx.h"
#include "resource.h"

/****************************************************************************
*                               OpenFileExHook
* Inputs:
*       HWND hDlg: Dialog reference
*       UINT msg: message ID
*       WPARAM wParam:
*       LPARAM lParam:
* Result: UINT_PTR
*       
* Effect: 
*       
****************************************************************************/

static UINT_PTR CALLBACK OpenFileExHook(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
    {
     switch(msg)
        { /* msg */
         case WM_INITDIALOG:
            { /* WM_INITIDIALOG */
             TRACE0("Hook: WM_INITDIALOG\n");

             ::SetWindowLongPtr(hDlg, DWLP_USER, lParam);
             
             LPOPENFILENAME ofn = (LPOPENFILENAME)lParam;
             if(ofn->lCustData == 0)
                { /* no mode */
                 ::ShowWindow(hDlg, SW_HIDE);
                } /* no mode */
             else
                { /* select mode */
                 UINT mode;
                 if(ofn->lCustData == 8)
                    mode = IDC_UTF8;
                 else
                 if(ofn->lCustData == 16)
                    mode = IDC_UNICODE;

                 ::CheckRadioButton(hDlg, IDC_UNICODE, IDC_UTF8, mode);
                 ofn->lCustData = mode;
                } /* select mode */
            } /* WM_INITIDIALOG */
            break;
         case WM_NOTIFY:
#ifdef _DEBUG //----------------------------------------------------------------+
            { /* debugging */                                                // |
             CString code;                                                   // |
             LPOFNOTIFY info = (LPOFNOTIFY)lParam;                           // |
             switch(info->hdr.code)                                          // |
                { /* code */                                                 // |
                 case CDN_INITDONE:                                          // |
                    code = _T("CDN_INITDONE");                               // |
                    break;                                                   // |
                 case CDN_SELCHANGE:                                         // |
                    code = _T("CDN_SELCHANGE");                              // |
                    break;                                                   // |
                 case CDN_FOLDERCHANGE:                                      // |
                    code = _T("CDN_FOLDERCHANGE");                           // |
                    break;                                                   // |
                 case CDN_SHAREVIOLATION:                                    // |
                    code = _T("CDN_SHAREVIOLATION");                         // |
                    break;                                                   // |
                 case CDN_HELP:                                              // |
                    code = _T("CDN_HELP");                                   // |
                    break;                                                   // |
                 case CDN_FILEOK:                                            // |
                    code = _T("CDN_FILEOK");                                 // |
                    break;                                                   // |
                 case CDN_TYPECHANGE:                                        // |
                    code = _T("CDN_TYPECHANGE");                             // |
                    break;                                                   // |
                 case CDN_INCLUDEITEM:                                       // |
                    code = _T("CDN_INCLUDEITEM");                            // |
                    break;                                                   // |
                 default:                                                    // |
                    code.Format(_T("code=0x%08x"), info->hdr.code);          // |
                    break;                                                   // |
                } /* code */                                                 // |
             TRACE1("Hook: WM_NOTIFY (code=%s)\n", code);                    // |
            } /* debugging */                                                // |
#endif //-----------------------------------------------------------------------+
            break;
         // These messages are not traced
         case WM_MOUSEMOVE:
         case WM_CTLCOLORSTATIC:
         case WM_SETCURSOR:
         case WM_NCHITTEST:
         case WM_SETFONT:
         case WM_MOUSEACTIVATE:
            break; 
         case WM_DESTROY:
            TRACE0("Hook: WM_DESTROY\n");
            break;
         case WM_COMMAND:
            TRACE3("Hook: WM_COMMAND wParam=(%d,,%d) lParam=0x%08x\n", HIWORD(wParam), LOWORD(wParam), lParam);
            { /* WM_COMMAND */
             LPOPENFILENAME ofn = (LPOPENFILENAME)::GetWindowLongPtr(hDlg, DWLP_USER);
             switch(LOWORD(wParam))
                { /* id */
                 case IDC_UTF8:
                 case IDC_UNICODE:
                    if(HIWORD(wParam) == BN_CLICKED)
                       ofn->lCustData = LOWORD(wParam);
                    break;
                } /* id */
            } /* WM_COMMAND */
            break;
         default:
             TRACE1("Hook: WM_0x%04x\n", msg);
             break;
        } /* msg */
     return 0;
    } // OpenFileExHook

/****************************************************************************
*                                 OpenFileEx
* Inputs:
*       BOOL save: TRUE to save, FALSE to open
*       CString & name: [in] Default filename
*                       [out] Place where filename will be put
*       const CString & ext: Default extension
*       UINT & option: Option for saving; 8 for UTF8, 16 for Unicode
*       UINT filterid: Stringtable ID of filter, 0 to have no filter
*       LPCTSTR titleid: Stringtable ID of title, 0 to use default title, MAKEINTRESOURCE(id) to use an ID
* Result: BOOL
*       TRUE if successful, name is defined
*       FALSE if error or cancelled, name is undefined
* Effect: 
*       Prompts for a filename
* Notes:
*       CFileDialog won't work because VS6 doesn't understand the
*       new OPENFILENAME struct which is larger in the new operating systems,
*       and fails horribly (access fault)       
****************************************************************************/

BOOL OpenFileEx(BOOL save, CString & name, const CString & ext, UINT & option, UINT filterid, LPCTSTR titleid)
   {
    TCHAR drive[MAX_PATH];
    TCHAR path[MAX_PATH];
    TCHAR initialdir[MAX_PATH];
    _tsplitpath(name, drive, path, NULL, NULL);
    _tmakepath(initialdir, drive, path, NULL, NULL);

    //****************
    // lStructSize
    //****************
    OPENFILENAME ofn = {sizeof(OPENFILENAME)};
    //****************
    // hwndOwner
    //****************
    ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
    //****************
    // hInstance
    //****************
    ofn.hInstance =  AfxGetInstanceHandle();

    //****************
    // lpstrFilter
    //****************

    LPTSTR filter0 = NULL;
    if(filterid != 0)
       { /* load filter */
        CString filter;
        filter.LoadString(filterid);

        filter0 = new TCHAR[filter.GetLength() + 1];
        if(filter0 == NULL)
           { /* failed */
            ::SetLastError(ERROR_NO_SYSTEM_RESOURCES);
            return FALSE;
           } /* failed */

        _tcscpy(filter0, filter);
        for(int i = 0; filter0[i] != _T('\0'); i++)
           { /* scan string */
            if(filter0[i] == _T('|'))
               filter0[i] = _T('\0');
           } /* scan string */
       } /* load filter */

    ofn.lpstrFilter = filter0;

    //****************
    // lpstrCustomFilter
    //****************
    // No custom filter

    //****************
    // nMaxCustFilter
    //****************
    // No count of custom filters

    //****************
    // nFilterIndex
    //****************
    // No filter indices

    //****************
    // lpstrFile
    //****************
    TCHAR filename[MAX_PATH];
    if(name.GetLength() < MAX_PATH)
       _tcscpy(filename, name);
    ofn.lpstrFile = filename;

    //****************
    // nMaxFile
    //****************
    ofn.nMaxFile = MAX_PATH;

    //****************
    // lpstrFileTitle
    //****************
    // No file title will be returned

    //****************
    // nMaxFileTitle
    //****************
    // No max needed since title is NULL

    //****************
    // lpstrInitialDir
    //****************
    ofn.lpstrInitialDir = initialdir;

    //****************
    // lpstrTitle
    //****************
    CString title;
    if(titleid != 0)
       { /* load title */
        if(titleid <= MAKEINTRESOURCE(65535))
           { /* load string */
            title.LoadString(LOWORD(titleid));
            ofn.lpstrTitle = (LPCTSTR)title;
           } /* load string */
        else
           { /* already string */
            ofn.lpstrTitle = titleid;
           } /* already string */
       } /* load title */

    //****************
    // Flags
    //****************
    ofn.Flags =
               // OFN_ALLOWMULTISELECT |
               (save ? OFN_CREATEPROMPT : 0) |
               //OFN_DONTADDTORECENT |
               OFN_ENABLEHOOK |
               // OFN_ENABLEINCLUDENOTIFY |
               OFN_ENABLESIZING |
               OFN_ENABLETEMPLATE |
               OFN_EXPLORER |
               // OFN_EXTENSIONDIFFERENT |
               (!save ? OFN_FILEMUSTEXIST : 0) |
               // OFN_FORCESHOWHIDDEN |
               OFN_HIDEREADONLY |
               // OFN_LONGNAMES |
               // OFN_NOCHANGEDIR |
               // OFN_NODEREFERENCELINKS |
               // OFN_NOLONGNAMES |
               // OFN_NONETWORKBUTTON |
               // OFN_NOREADONLYRETURN |
               // OFN_NOTESTFILECREATE |
               // OFN_NOVALIDATE |
               // OFN_OVERWRITEPROMPT |
               OFN_PATHMUSTEXIST |
               // OFN_READONLY |
               // OFN_SHAREAWARE |
               // OFN_SHOWHELP |
            0;

    //****************
    // nFileOffset (output)
    //****************


    //****************
    // nFileExtension (output)
    //****************

    //****************
    // lpstrDefExt
    //****************
    ofn.lpstrDefExt = ext;

    //****************
    // lCustData
    //****************
    ofn.lCustData = option;

    //****************
    // lpfnHook
    //****************
    ofn.lpfnHook = OpenFileExHook;
    

    //****************
    // lpTemplateName
    //****************
    ofn.lpTemplateName = MAKEINTRESOURCE(IDD_ENCODING);

    //****************
    // pvReserved
    //****************
    // reserved

    //****************
    // dwReserved
    //****************
    // reserved

    //****************
    // FlagsEx
    //****************
    // No extended flags

    //*****************************************************************************
    if(save)
       { /* save */
        if(!GetSaveFileName(&ofn))
           { /* save op failed */
            DWORD err = CommDlgExtendedError();
            delete filter0;
            return FALSE;
           } /* save op failed */
       } /* save */
    else
       { /* open */
        if(!GetOpenFileName(&ofn))
           { /* open failed */
            DWORD err = CommDlgExtendedError();
            delete filter0;
            return FALSE;
           } /* open failed */
       } /* open */

    switch(ofn.lCustData)
       { /* codepage */
        case 0:
           option = 0;
           break;
        case IDC_UNICODE:
           option = 16;
           break;
        case IDC_UTF8:
           option = 8;
           break;
       } /* codepage */

    name = ofn.lpstrFile;
    delete filter0;
    return TRUE;
   }
