// RTFHelp.cpp : implementation file
//
// Copyright  2005, Joseph M. Newcomer, All Rights Reserved
// You are free to redistribute FlounderClock providing all copyrights are left intact
// You must record any changes you make to this source by putting // and your
// initials at the end of each line you change. You may not remove any lines
// from this source; instead, put // in front of them and your initials at the end.
// You may not charge any fee for the redistribution, nor incorporate this code in
// any commercial product. This is freeware, and must remain so.
//
// Parts of this module are based on the COleRichEditCtrl by Mike O'Neill
// see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
#include "stdafx.h"
#include "resource.h"
#include "RTFHelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRTFHelp

CRTFHelp::CRTFHelp()
   {
    CallbackSet = FALSE;
   }

CRTFHelp::~CRTFHelp()
   {
   }


BEGIN_MESSAGE_MAP(CRTFHelp, CRichEditCtrl)
        //{{AFX_MSG_MAP(CRTFHelp)
                // NOTE - the ClassWizard will add and remove mapping macros here.
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRTFHelp message handlers

/****************************************************************************
*                         CRTFHelp::PreSubclassWindow
* Result: void
*       
* Effect: 
*       Creates a new interface object so images can be included in the
*       rich text control
****************************************************************************/

void CRTFHelp::PreSubclassWindow() 
   {
    CRichEditCtrl::PreSubclassWindow();
    ICallback = new IExRichEditOleCallback;
    CallbackSet = SetOLECallback(ICallback);
   }

/****************************************************************************
*          CRTFHelp::IEXRichEditOleCallback::IExRichEditOleCallback
* Effect: 
*       Constructor for the RichEditOleCallback interface
****************************************************************************/

CRTFHelp::IExRichEditOleCallback::IExRichEditOleCallback()
      {
      pStorage = NULL;
      m_iNumStorages = 0;
      m_dwRef = 0;

        // set up OLE storage

      HRESULT hResult = ::StgCreateDocfile(NULL,
                                           STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE /*| STGM_DELETEONRELEASE */|STGM_CREATE ,
                                           0, &pStorage );

      if ( pStorage == NULL ||
           hResult != S_OK )
         {
          AfxThrowOleException( hResult );
         }
      }

/****************************************************************************
*          CRTFHelp::IEXRichEditOleCallback::~IExRichEditOleCallback
* Effect: 
*       Destructor for the RichEditOleCallback interface
****************************************************************************/

CRTFHelp::IExRichEditOleCallback::~IExRichEditOleCallback()
   {
   }

/****************************************************************************
*               CRTFHelp::IExRichEditOleCallback::GetNewStorage
* Inputs:
*       LPSTORAGE * lplpstg: Pointer to storage pointer
* Result: HRESULT
*       
* Effect: 
*       Creates needed storage for a dynamic OLE object
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::GetNewStorage(LPSTORAGE* lplpstg)
   {
    m_iNumStorages++;
    WCHAR tName[50];
    swprintf(tName, L"REOLEStorage%d", m_iNumStorages);

    HRESULT hResult = pStorage->CreateStorage(tName, 
                                              STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE ,
                                              0, 0, lplpstg );

    if (hResult != S_OK )
       {
        ::AfxThrowOleException( hResult );
       }

    return hResult;
   } // CRTFHelp::IExRichEditOleCallback::GetNewStorage

/****************************************************************************
*              CRTFHelp::IExRichEditOleCallback::QueryInterface
* Inputs:
*       REFIID iid:
*       void ** ppvObject:
* Result: HRESULT
*       
* Effect: 
*       Returns an interface reference
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::QueryInterface(REFIID iid, void ** ppvObject)
   {
    HRESULT hr = S_OK;
    *ppvObject = NULL;

    if ( iid == IID_IUnknown ||
         iid == IID_IRichEditOleCallback )
       {
        *ppvObject = this;
        AddRef();
        hr = NOERROR;
       }
    else
       {
        hr = E_NOINTERFACE;
       }

    return hr;
   } // CRTFHelp::IExRichEditOleCallback::QueryInterface

/****************************************************************************
*                  CRTFHelp::IExRichEditOleCallback::AddRef
* Result: ULONG
*       The new reference count
* Effect: 
*       Increments the reference count
****************************************************************************/

ULONG STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::AddRef()
   {
   return ++m_dwRef;
   } // CRTFHelp::IExRichEditOleCallback::AddRef

/****************************************************************************
*                  CRTFHelp::IExRichEditOleCallback::Release
* Result: ULONG
*       The new reference count
* Effect: 
*       Decrements the reference count. If the reference count goes to 0,
*       the object is deleted
****************************************************************************/

ULONG STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::Release()
   {
    if ( --m_dwRef == 0 )
       {
        delete this;
        return 0;
       }

    return m_dwRef;
   } // CRTFHelp::IExRichEditOleCallback::Release


/****************************************************************************
*             CRTFHelp::IExRichEditOleCallback::GetInPlaceContext
* Inputs:
*       LPOLEINPLACEFRAME FAR *lplpFrame:
*       LPOLEINPLACEUIWINDOW FAR *lplpDoc:
*       LPOLEINPLACEFRAMEINFO lpFrameInfo:
* Result: HRESULT
*       
* Effect: 
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::GetInPlaceContext(
                             LPOLEINPLACEFRAME FAR *lplpFrame,
                             LPOLEINPLACEUIWINDOW FAR *lplpDoc,
                             LPOLEINPLACEFRAMEINFO lpFrameInfo)
   {
    return S_OK;
   }// CRTFHelp::IExRichEditOleCallback::GetInPlaceContext


/****************************************************************************
*              CRTFHelp::IExRichEditOleCallback::ShowContainerUI
* Inputs:
*       BOOL fShow: ignored
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::ShowContainerUI(BOOL fShow)
   {
    return S_OK;
   } // CRTFHelp::IExRichEditOleCallback::ShowContainerUI

/****************************************************************************
*             CRTFHelp::IExRichEditOleCallback::QueryInsertObject
* Inputs:
*       LPCLSID lpclsid: ignored
*       LPSTORAGE lpstg: ignored
*       LONG cp: ignored
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::QueryInsertObject(LPCLSID lpclsid, LPSTORAGE lpstg, LONG cp)
   {
    return S_OK;
   }


/****************************************************************************
*               CRTFHelp::IExRichEditOleCallback::DeleteObject
* Inputs:
*       LPOLEOBJECT lpoleobj:
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::DeleteObject(LPOLEOBJECT lpoleobj)
   {
    return S_OK;
   }

/****************************************************************************
*              CRTFHelp::IExRichEditOleCallback::QueryAcceptData
* Inputs:
*       all are ignored
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::QueryAcceptData(LPDATAOBJECT lpdataobj,
                                                     CLIPFORMAT FAR *lpcfFormat,
                                                     DWORD reco,
                                                     BOOL fReally,
                                                     HGLOBAL hMetaPict)
   {
    return S_OK;
   } // CRTFHelp::IExRichEditOleCallback::QueryAcceptData


/****************************************************************************
*           CRTFHelp::IExRichEditOleCallback::ContextSensitiveHelp
* Inputs:
*       BOOL fEnterMode: ignored
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::ContextSensitiveHelp(BOOL fEnterMode)
   {
    return S_OK;
   } // CRTFHelp::IExRichEditOleCallback::ContextSensitiveHelp

/****************************************************************************
*             CRTFHelp::IExRichEditOleCallback::GetClipboardData
* Inputs:
*       all are ignored
* Result: HRESULT
*       
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::GetClipboardData(CHARRANGE FAR *lpchrg, DWORD reco, LPDATAOBJECT FAR *lplpdataobj)
   {
    return S_OK;
   } // CRTFHelp::IExRichEditOleCallback::GetClipboardData

/****************************************************************************
*             CRTFHelp::IExRichEditOleCallback::GetDragDropEffect
* Inputs:
*       all are ignored
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::GetDragDropEffect(BOOL fDrag, DWORD grfKeyState, LPDWORD pdwEffect)
   {
    return S_OK;
   }

/****************************************************************************
*              CRTFHelp::IExRichEditOleCallback::GetContextMenu
* Inputs:
*       all are ignored
* Result: HRESULT
*       S_OK, always
* Effect: 
*       Does nothing.
*       This is OLE insertion magic. I don't pretend to understand it.
*       see http://www.codeproject.com/richedit/COleRichEditCtrl.asp
****************************************************************************/

HRESULT STDMETHODCALLTYPE 
   CRTFHelp::IExRichEditOleCallback::GetContextMenu(WORD seltyp,
                                                    LPOLEOBJECT lpoleobj,
                                                    CHARRANGE FAR *lpchrg,
                                                    HMENU FAR *lphmenu)
   {
    return S_OK;
   } // CRTFHelp::IExRichEditOleCallback::GetContextMenu
