#include "stdafx.h"
#include "resource.h"
#include "RegFontInfo.h"
#include "Registry.h"


/****************************************************************************
*                      RegFontInfo::Direction::ToString
* Result: CString
*       The direction string
****************************************************************************/

CString RegFontInfo::Direction::toString()
    {
     CString result;
     switch(value)
        { /* value */
         case Unknown:
            result.LoadString(IDS_DIRECTION_UNKNOWN);
            return  result;
         case RTL:
            result.LoadString(IDS_DIRECTION_RTL);
            return result;
         case LTR:
            result.LoadString(IDS_DIRECTION_LTR);
            return result;
        } /* value */
     return _T("?");
    } // RegFontInfo::Direction::ToString

/****************************************************************************
*                      RegFontInfo::Direction::MaxString
* Inputs:
*       CDC & dc: DC to use for measurement
* Result: CString
*       The widest string used for the types
* Effect: 
*       
****************************************************************************/

CString RegFontInfo::Direction::MaxString(CDC & dc)
    {
     CString result;
     int width = 0;
     for(int i = Unknown; i <= RTL; i++)
        { /* test each */
         CString s;
         s = RegFontInfo::Direction((dir)i).toString();
         int w = dc.GetTextExtent(s).cx;
         if(w > width)
            { /* wider */
             width = w;
             result = s;
            } /* wider */
        } /* test each */
     return result;
    } // RegFontInfo::Direction::MaxString

/****************************************************************************
*                        CFontMap::GetRegistryFontInfo
* Inputs:
*       LCID lcid: LCID to look up
* Result: RegFontInfo * 
*       The data found, or NULL if not found
****************************************************************************/

BOOL RegFontInfo::GetRegistryFontInfo(LCID lcid)
    {
     CString s;
     s.Format(_T("0x%04x"), lcid);
     return GetRegistryFontInfo(s);
    } // CFontMap::GetRegistryFontInfo

/****************************************************************************
*                      RegFontInfo::GetRegistryInfo
* Inputs:
*       const CString & key: Key in the form "0xnnnn"
* Result: BOOL
*       TRUE if found
*       FALSE if an error occurred
* Effect: 
*       Fills the info structure in from the registry data
****************************************************************************/

BOOL RegFontInfo::GetRegistryFontInfo(const CString & key)
    {
     CString hex;
     hex = key.Mid(2);
     lcid = _tcstoul(hex, 0, 16);

     CString FontMap;
     FontMap.LoadString(IDS_REGISTRY_FONTMAP);

     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn   does it exist?
     //----------------------------------------------------------------
     HKEY testkey = OpenKey(HKEY_CURRENT_USER, FontMap + _T("\\") + key);
     if(testkey == NULL)
        return FALSE;
     ::RegCloseKey(testkey);
     
     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn\Font Name  REG_SZ
     //----------------------------------------------------------------
     if(!GetRegistryString(HKEY_CURRENT_USER, FontMap + _T("\\") + key, IDS_REGISTRY_FONTNAME, fontname))
        { /* not found */
         if(::GetLastError() != ERROR_FILE_NOT_FOUND)
            return FALSE; // Something seriously wrong
         fontname = _T("");
        } /* not found */

     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn\Size Adjust
     //----------------------------------------------------------------
     if(!GetRegistryInt(HKEY_CURRENT_USER, key, IDS_REGISTRY_SIZE_ADJUST, (DWORD&)size))
        { /* not found */
         if(::GetLastError() != ERROR_FILE_NOT_FOUND)
            return FALSE; // something seriously wrong
         size = 0;
        } /* not found */

     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn\Direction
     //----------------------------------------------------------------
     BOOL dirLTR;
     if(!GetRegistryInt(HKEY_CURRENT_USER, key, IDS_REGISTRY_DIRECTION, (DWORD&)dirLTR))
        { /* not found */
         if(::GetLastError() != ERROR_FILE_NOT_FOUND)
            return FALSE; // something seriously wrong
         direction = RegFontInfo::Direction(RegFontInfo::Direction::Unknown);
        } /* not found */
     else
        { /* found */
         direction = dirLTR ? RegFontInfo::Direction(RegFontInfo::Direction::RTL) : RegFontInfo::Direction(RegFontInfo::Direction::LTR);
        } /* found */
     return TRUE;
    } // RegFontInfo::GetRegistryFontInfo

/****************************************************************************
*                      RegFontInfo::SetRegistryFontInfo
* Result: BOOL
*       TRUE if success
*       FALSE if error (state of registry undefined)
* Effect: 
*       Stores the RegFontInfo in the Registry
****************************************************************************/

BOOL RegFontInfo::SetRegistryFontInfo()
    {
     CString FontMap;
     FontMap.LoadString(IDS_REGISTRY_FONTMAP);
     CString key;
     key.Format(_T("0X%04X"), lcid);

     CString fontkey = FontMap + _T("\\") + key;

     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn\Font Name  REG_SZ
     //----------------------------------------------------------------
     if(!SetRegistryString(HKEY_CURRENT_USER, fontkey, IDS_REGISTRY_FONTNAME, fontname))
        { /* fontname error */
         return FALSE; // Something seriously wrong
        } /* fontname error */

     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn\Size Adjust
     //----------------------------------------------------------------
     if(!SetRegistryInt(HKEY_CURRENT_USER, fontkey, IDS_REGISTRY_SIZE_ADJUST, size))
        { /* size error */
         return FALSE; // something seriously wrong
        } /* size error */

     //----------------------------------------------------------------
     // HKCU\Software\LocaleExplorer\FontMap\0xnnnn\Direction
     //----------------------------------------------------------------
     if(!SetRegistryInt(HKEY_CURRENT_USER, fontkey, IDS_REGISTRY_DIRECTION, (int)direction))
        { /* direction error */
         return FALSE; // something seriously wrong
        } /* direction error */
     return TRUE;
    } // RegFontInfo::SetRegistryFontInfo

/****************************************************************************
*                        RegFontInfo::DeleteAllEntries
* Result: BOOL
*       TRUE if success, FALSE if error
* Effect: 
*       Deletes all the FontMap entries.
****************************************************************************/

BOOL RegFontInfo::DeleteAllEntries()
    {
     CString FontMap;
     FontMap.LoadString(IDS_REGISTRY_FONTMAP);
     DeleteRegistryKey(HKEY_CURRENT_USER, FontMap);
    } // RegFontInfo::DeleteAllEntries
