// RicheditCtrlEx.cpp : implementation file
//

#include "stdafx.h"
#include "RicheditCtrlEx.h"
#include <malloc.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRicheditCtrlEx

CRichEditCtrlEx::CRichEditCtrlEx()
{
}

CRichEditCtrlEx::~CRichEditCtrlEx()
{
}


BEGIN_MESSAGE_MAP(CRichEditCtrlEx, CRichEditCtrl)
        //{{AFX_MSG_MAP(CRichEditCtrlEx)
                // NOTE - the ClassWizard will add and remove mapping macros here.
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRicheditCtrlEx message handlers

/****************************************************************************
*                           CRichEditCtrlEx::GetTextRange
* Inputs:
*       int start:
*       int end:
*       CString & s:
* Result: int
*       Size of selection
* Effect: 
*       Retrieves the text in the range start..end to the string s
* Notes:
*       This would not be needed if the VS7 MFC is used. Hence it is
*       conditionally compiled only if _MFC_VER is < 0x0700
****************************************************************************/

#if _MFC_VER < 0x0700
int CRichEditCtrlEx::GetTextRange(int start, int end, CString & s)
    {
     if(start == end)
        { /* nothing */
         s = _T("");
         return 0;
        } /* nothing */

#ifdef _UNICODE // This is necessary because we fake out the Rich Edit 2.0 control
     TEXTRANGEW tr;
#else
     TEXTRANGEA tr;
#endif
     tr.chrg.cpMin = start;
     tr.chrg.cpMax = end;
     tr.lpstrText = s.GetBuffer(end - start + 2);
     int result = ::SendMessage(m_hWnd, EM_GETTEXTRANGE, 0, (LPARAM)&tr);
     s.ReleaseBuffer();
     return result;
    } // CRTFHelp::GetTextRange
#endif // MFC_VER < 0x0700

#if (_MFC_VER < 0x0700)
CString CRichEditCtrlEx::GetSelText() const
   {
   ASSERT(::IsWindow(m_hWnd));
   CHARRANGE cr;
   cr.cpMin = cr.cpMax = 0;
   ::SendMessage(m_hWnd, EM_EXGETSEL, 0, (LPARAM)&cr);
   LPTSTR lpsz = (LPTSTR)_alloca((cr.cpMax - cr.cpMin + 1)*sizeof(WCHAR));
   lpsz[0] = _T('\0');
   ::SendMessage(m_hWnd, EM_GETSELTEXT, 0, (LPARAM)lpsz);
   return lpsz;
   }
#endif // _MFC_VER < 0x0700

/****************************************************************************
*                       CRichEditCtrlEx::GetTextLength
* Result: long
*       THe text length
* Effect: 
*       Uses the GETTLEXTLENGTHEX technique
****************************************************************************/

#if _MFC_VER < 0x0700
long CRichEditCtrlEx::GetTextLength() const
    {
     ASSERT(::IsWindow(m_hWnd));
     GETTEXTLENGTHEX gt;
     gt.flags = GTL_NUMCHARS;
#ifdef _UNICODE
     gt.codepage = 1200;
#else
     gt.codepage = CP_ACP;
#endif
     return (long)::SendMessage(m_hWnd, EM_GETTEXTLENGTHEX, (WPARAM)&gt, 0);
    } // CRichEditCtrlEx::GetTextLength
#endif

/****************************************************************************
*                   CRichEditCtrlEx::SetSelectionCharFormat
* Inputs:
*       CHARFORMAT2 & cf: Character formatting
* Result: BOOL
*       TRUE if success, FALSE if error
* Effect: 
*       Sets character format 2 information
****************************************************************************/

#if _MFC_VER < 0x0700
BOOL CRichEditCtrlEx::SetSelectionCharFormat(CHARFORMAT2 & cf)
    {
     ASSERT(::IsWindow(m_hWnd));
     cf.cbSize = sizeof(CHARFORMAT2);
     return (BOOL)::SendMessage(m_hWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
    } // CRichEditCtrlEx::SetSelectionCharFormat
#endif

/****************************************************************************
*                   CRichEditCtrlEx::SetWindowTextNoNotify
* Inputs:
*       const CString & s: String to set
* Result: void
*
* Effect: 
*       Sets the text without doing a notification
****************************************************************************/

void CRichEditCtrlEx::SetWindowTextNoNotify(const CString & s)
    {
     DWORD mask = GetEventMask();
     SetEventMask(mask & ~ENM_CHANGE);
     CRichEditCtrl::SetWindowText(s);
     SetEventMask(mask);
    } // CRichEditCtrlEx::SetWindowTextNoNotify
    

/****************************************************************************
*                       CRichEditCtrlEx::SetSelNoNotify
* Inputs:
*       long low:
*       long high:
* Result: void
*       
* Effect: 
*       Sets the selection
****************************************************************************/

void CRichEditCtrlEx::SetSelNoNotify(long low, long high)
    {
     DWORD mask = CRichEditCtrl::GetEventMask();
     CRichEditCtrl::SetEventMask(mask & ~ENM_SELCHANGE);
     CRichEditCtrl::SetSel(low, high);
     CRichEditCtrl::SetEventMask(mask);
    } // CRichEditCtrlEx::SetSelNoNotify

/****************************************************************************
*                   CRichEditCtrlEx::GetSelectionCharFormat
* Inputs:
*       CHARFORMAT2 & cf: Character format
* Result: DWORD
*       The dwMask member of the CHARFORMAT structure
* Effect: 
*       Retrieves the character format
****************************************************************************/

DWORD CRichEditCtrlEx::GetSelectionCharFormat(CHARFORMAT2 & cf)
    {
     ASSERT(m_hWnd != NULL);
     return SendMessage(EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)(&cf));
    } // CRichEditCtrlEx::GetSelectionCharFormat
