// ShowGlyph.cpp : implementation file
//

#include "stdafx.h"
#include "ShowGlyph.h"
#include "gdiflush.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CShowGlyph, CWnd)
/////////////////////////////////////////////////////////////////////////////
// CShowGlyph

CShowGlyph::CShowGlyph()
   {
    CurrentStyle = None;
   }

CShowGlyph::~CShowGlyph()
{
}


BEGIN_MESSAGE_MAP(CShowGlyph, CWnd)
        ON_MESSAGE(WM_SETFONT, OnSetFont)
        ON_MESSAGE(WM_GETFONT, OnGetFont)
        //{{AFX_MSG_MAP(CShowGlyph)
        ON_WM_PAINT()
        ON_WM_ERASEBKGND()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CShowGlyph message handlers

/****************************************************************************
*                             CShowGlyph::OnPaint
* Result: void
*       
* Effect: 
*       Draws the glyph
****************************************************************************/

void CShowGlyph::OnPaint() 
   {
    CPaintDC dc(this); // device context for painting

    switch(CurrentStyle)
       { /* CurrentStyle */
        case None:
           return;
        case Char:
           { /* Char */
            CRect r;
            GetClientRect(&r);

            CFont * f = CFont::FromHandle(font);
#ifdef _DEBUG // ----------------------------------------------------------------+
            LOGFONT lf;                                                       // |
            f->GetLogFont(&lf);                                               // |
            dc.MoveTo(r.left, r.Height() / 2);                                // |
            dc.LineTo(r.right, r.Height() / 2);                               // |
            dc.MoveTo(r.Width() / 2, r.top);                                  // |
            dc.LineTo(r.Width() / 2, r.bottom);                               // |
            GDI_FLUSH();                                                      // |
#endif // ------------------------------------------------------------------------
            dc.SelectObject(f);
            dc.SetBkMode(TRANSPARENT);
            dc.SetTextAlign(TA_CENTER | TA_TOP);
            dc.TextOut(r.Width() / 2,  r.top, text);
            GDI_FLUSH();
           } /* Char */
           break;
        case Text:
           { /* Text */
            CFont * f = CFont::FromHandle(font);
            dc.SelectObject(f);
            CRect r;
            r.left = 0;
            r.top = 0;
            CSize sz = dc.GetTextExtent(text);
            r.right = sz.cx;
            r.bottom = sz.cy;
            CalcWindowRect(&r);
            SetWindowPos(NULL,
                         0,0, // no position change
                         r.Width(), r.Height(),
                         SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
            dc.SelectObject(f);
            dc.SetBkMode(TRANSPARENT);
            dc.SetTextAlign(TA_LEFT | TA_TOP);
            dc.TextOut(0, 0, text);
            GDI_FLUSH();
           } /* Text */
        }/* CurrentStyle */
    }

/****************************************************************************
*                          CShowGlyph::OnEraseBkgnd
* Inputs:
*       CDC * pDC: DC
* Result: BOOL
*       TRUE, always
* Effect: 
*       Paints the background
****************************************************************************/

BOOL CShowGlyph::OnEraseBkgnd(CDC* pDC) 
   {
    CRect r;
    GetClientRect(&r);
        
    pDC->FillSolidRect(&r, ::GetSysColor(COLOR_INFOBK));
    return TRUE;
   }

/****************************************************************************
*                             CShowGlyph::Create
* Inputs:
*       int x: screen position
*       int y: screen position
*       CWnd * parent: Parent of window
* Result: BOOL
*       TRUE if successful
*       FALSE if failure
* Effect: 
*       Creates the display window
****************************************************************************/

BOOL CShowGlyph::Create(int x, int y, CWnd * parent)
    {
     ASSERT(parent != NULL);
     if(parent == NULL)
        return FALSE;

     LPCTSTR infoclass = AfxRegisterWndClass(0, // no class styles
                                             NULL, // default arrow
                                             (HBRUSH)(COLOR_INFOBK + 1),
                                             NULL);
     ASSERT(infoclass != NULL);
     if(infoclass == NULL)
        return FALSE;

     CRect r;
     r.SetRectEmpty(); // rather small window to start
     BOOL result = CreateEx(WS_EX_NOACTIVATE,
                            infoclass,
                            NULL,
                            /* not WS_VISIBLE */ WS_POPUP | WS_BORDER,
                            r,
                            parent,
                            NULL);
     ASSERT(result);
     if(!result)
        return FALSE;


     return TRUE;
    } // CShowGlyph::Create

/****************************************************************************
*                            CShowGlyph::OnSetFont
* Inputs:
*       WPARAM: (WPARAM)(HFONT) font
*       LPARAM: (LPARAM)(BOOL) redraw option
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Saves the HFONT
****************************************************************************/

LRESULT CShowGlyph::OnSetFont(WPARAM wParam, LPARAM lParam)
    {
     font = (HFONT)wParam;
     if(lParam)
        { /* invalidate */
         Invalidate();
         RedrawWindow();
        } /* invalidate */
     return 0;
    } // CShowGlyph::OnSetFont

/****************************************************************************
*                            CShowGlyph::OnGetFont
* Inputs:
*       WPARAM: unused
*       LPARAM: used
* Result: LRESULT
*       (LRESULT)(HFONT) the font
****************************************************************************/

LRESULT CShowGlyph::OnGetFont(WPARAM, LPARAM)
    {
     return (LRESULT)font;
    } // CShowGlyph::OnGetFont

/****************************************************************************
*                             CShowGlyph::SetText
* Inputs:
*       LPCTSTR c: Desired string
* Result: void
*       
* Effect: 
*       Updates the character
****************************************************************************/

void CShowGlyph::SetText(LPCTSTR c)
    {
     if(c == text)
        return; // no change
     text = c;
     switch(CurrentStyle)
        { /* CurrentStyle */
         case Char:
            break;
         case Text:
            { /* Text */
            } /* Text */
        } /* CurrentStyle */
     Invalidate();
    } // CShowGlyph::SetText

/****************************************************************************
*                            CShowGlyph::SetStyle
* Inputs:
*       ShowStyle style: Style to use
*       CFont * fontPrototype
* Result: void
*       
* Effect: 
*       Sets the desired style
****************************************************************************/

void CShowGlyph::SetStyle(CShowGlyph::ShowStyle style, CFont * fontPrototype)
    {
     if(CurrentStyle == style)
        return; // already set

     ASSERT(fontPrototype != NULL);

     LOGFONT lf;
     fontPrototype->GetLogFont(&lf);

     CClientDC dc(this);

     switch(style)
        { /* style */
         case CShowGlyph::Char: // character display
            { /* ShowGlyph::Char */
             int w = dc.GetDeviceCaps(LOGPIXELSX) / 2;
             int h = dc.GetDeviceCaps(LOGPIXELSY) / 2;
             SetWindowPos(NULL,
                          0, 0, // no position chnage
                          w, h, // set new size
                          SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
             
             lf.lfHeight = (9 * h) / 10;
             lf.lfEscapement = 0;
             lf.lfOrientation = 0;
             ReplaceFont(&lf);
            } /* ShowGlyph::Char */
            break;

         case CShowGlyph::Text: // text display
            { /* ShowGlyph::Text */
             ReplaceFont(&lf); // just use the current font
            } /* ShowGlyph::Text */
            break;
        } /* style */


     // Set the style to be the requested style
     CurrentStyle = style;
    } // CShowGlyph::SetStyle

/****************************************************************************
*                           CShowGlyph::ReplaceFont
* Inputs:
*       LOGFONT * lf: Font specifications
* Result: void
*       
* Effect: 
*       Replaces the current font with the font defined by the LOGFONT structure
****************************************************************************/

void CShowGlyph::ReplaceFont(LOGFONT * lf)
    {
     CFont font; // local copy of new font
     font.CreateFontIndirect(lf);
     CFont * oldfont = GetFont();
     if(oldfont != NULL)
        oldfont->DeleteObject();

     SetFont(&font);
     font.Detach();
    } // CShowGlyph::ReplaceFont
