// StringDisplay.cpp : implementation file
//

#include "stdafx.h"
#include "StringDisplay.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStringDisplay

CStringDisplay::CStringDisplay()
   {
    UseSortKey = FALSE;
    SortCol = -1; // nonexistent
   }

CStringDisplay::~CStringDisplay()
{
}


BEGIN_MESSAGE_MAP(CStringDisplay, CListCtrlEx)
        //{{AFX_MSG_MAP(CStringDisplay)
        ON_WM_HSCROLL()
        //}}AFX_MSG_MAP
        ON_MESSAGE(WM_SETFONT, OnSetFont)
        ON_WM_MEASUREITEM_REFLECT()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStringDisplay message handlers

void CStringDisplay::DrawItem(LPDRAWITEMSTRUCT dis) 
   {
    //CListCtrlEx::DrawItem(dis);
    CRect r;
    CDC * dc = CDC::FromHandle(dis->hDC);
    int save = dc->SaveDC();
    LVCOLUMN colinfo = {0};
    colinfo.mask = LVCF_FMT | LVCF_WIDTH;
    COLORREF fill;
    COLORREF text;
    
    for(int col = 0; GetColumn(col, &colinfo); col++)
       { /* scan columns */
        //----------------------------------------------------------------
        // Compute the rectangle
        //----------------------------------------------------------------
        if(col > 0)
           { /* subitem */
            GetSubItemRect(dis->itemID, col, LVIR_BOUNDS, r);
           } /* subitem */
        else
           { /* main item */
            GetItemRect(dis->itemID, r, LVIR_BOUNDS);
           } /* main item */
        //----------------------------------------------------------------
        // Compute the colors for fill and text
        //----------------------------------------------------------------
        enum {FocusSelected, NoFocusSelected, NotSelected} ColorOption;
        if(dis->itemState & ODS_SELECTED)
           { /* selected */
            if(::GetFocus() == m_hWnd)
               { /* has focus */
                fill = ::GetSysColor(COLOR_HIGHLIGHT);
                text = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
                ColorOption = FocusSelected;
               } /* has focus */
            else
               { /* no focus */
                fill = ::GetSysColor(COLOR_GRAYTEXT);
                text = ::GetSysColor(COLOR_WINDOWTEXT);
                ColorOption = NoFocusSelected;
               } /* no focus */
           } /* selected */
        else
           { /* not selected */
            fill = ::GetSysColor(COLOR_WINDOW);
            text = ::GetSysColor(COLOR_WINDOWTEXT);
            ColorOption = NotSelected;
           } /* not selected */
        int pos = r.left;

        dc->SetTextColor(text);

        //----------------------------------------------------------------
        // Fill the background. Note the adjustment down and the expansion
        // This looks better
        //----------------------------------------------------------------
        CRect filler = r;
        filler.top += 1;
        filler.bottom += 1;

        dc->FillSolidRect(&filler, fill);

        //----------------------------------------------------------------
        // Compute the correct width of the column
        // Note that GetItemRect gives then ENTIRE rectangle, and
        // getting the subitem rect for item 0 does the same thing
        // So if we are in the first column, we ask for the second
        // column
        //----------------------------------------------------------------
        int width;
        if(col > 0)
           { /* second or higher */
            width = r.Width();
           } /* second or higher */
        else
           { /* special case 0 */
            if(GetSubItemRect(dis->itemID, col + 1, LVIR_BOUNDS, r))
               { /* got second */
                width = r.left;
               } /* got second */
            else
               { /* failed */
                ASSERT(FALSE);
                width = r.Width(); // probably wrong, shouldn't be here
               } /* failed */
           } /* special case 0 */

        //----------------------------------------------------------------
        // Now compute the alignment. Note that if there is 
        // horizontal scrolling, we have to adjust for the scrolling
        // position
        //----------------------------------------------------------------

        int offset = -GetScrollPos(SB_HORZ);
        
        int x = dis->rcItem.left + pos + offset;

        //----------------------------------------------------------------
        // Set up to honor the formatting alignment
        //----------------------------------------------------------------
        switch(colinfo.fmt & LVCFMT_JUSTIFYMASK)
           { /* fmt */
            case LVCFMT_CENTER:
               x = dis->rcItem.left + pos + width / 2 - offset;
               dc->SetTextAlign(TA_CENTER);
               break;
            case LVCFMT_RIGHT:
               x = dis->rcItem.left + pos + width - offset;
               dc->SetTextAlign(TA_RIGHT);
               break;
            case LVCFMT_LEFT:
            default:
               x = dis->rcItem.left + pos - offset;
               dc->SetTextAlign(TA_LEFT);
               break;
           } /* fmt */

        //----------------------------------------------------------------
        // Set a clipping region so we don't paint outside the lines...
        // Note that we have to add +1 to deal with the fact that all
        // drawing is up-to-but-not-including the endpoint, and we
        // want to include the endpoint
        //----------------------------------------------------------------
        CRect clip(dis->rcItem.left + pos - offset,
                   dis->rcItem.top,
                   dis->rcItem.left + pos - offset + width + 1,
                   dis->rcItem.bottom + 1);

        CRgn cliprgn;
        cliprgn.CreateRectRgn(clip.left, clip.top, clip.right, clip.bottom);
        dc->SelectClipRgn(&cliprgn);

        //----------------------------------------------------------------
        CString s = GetItemText(dis->itemID, col);

        //----------------------------------------------------------------
        // Format the text. If it is a sort key, we display it in colors
        // so it is more readable
        //----------------------------------------------------------------
        if(col == SortCol)
           { /* special highlighting */
            if(s.GetLength() > 0 && s[0] == _T('!'))
               { /* error string */
                int save2 = dc->SaveDC();
                s = s.Mid(1); // throw away initial ! that indicates an error
#define COLOR_ERROR_MESSAGE RGB(255, 0, 0)
                dc->SetTextColor(COLOR_ERROR_MESSAGE);
                dc->TextOut(x, dis->rcItem.top, s);
                dc->RestoreDC(save2);
               } /* error string */
            else
               { /* fixed-pitch formatting */
                CFont * font = GetFont();
                LOGFONT lf;
                font->GetLogFont(&lf);
                lf.lfWeight = FW_BOLD;
                lstrcpy(lf.lfFaceName, _T("Courier New"));

                CFont normal;
                normal.CreateFontIndirect(&lf);
                CFont bold;
                bold.CreateFontIndirect(&lf);
                int save2 = dc->SaveDC();

                s.TrimRight();
                
                if(UseSortKey)
                   { /* byte sequence string */
                    // The format is
                    // [character weights] 01 [diacritic weights] 01 [case weights] 01 [special weights] 00
                    // BLUE                BK GREEN               BK RED            BK MAGENTA           BK
#define COLOR_CHARACTER_WEIGHTS RGB(  0,   0, 255)
#define COLOR_DIACRITIC_WEIGHTS RGB(  0, 255,   0)
#define COLOR_CASE_WEIGHTS      RGB(255,   0,   0)
#define COLOR_SPECIAL_WEIGHTS   RGB(255,   0, 255)
                    typedef enum { CHARACTER_WEIGHTS, DIACRITIC_WEIGHTS, CASE_WEIGHTS, SPECIAL_WEIGHTS} states;
                    states state = CHARACTER_WEIGHTS;

                    COLORREF color = COLOR_CHARACTER_WEIGHTS;
                    int start = x;

                    while(!s.IsEmpty())
                       { /* extract */
                        s.TrimLeft();
                        CString t;
                        t = s.Left(3); // Extract leftmost three charactres (dd space)
                        if(!s.IsEmpty())
                           s = s.Mid(min(3,s.GetLength()));  // and remove from the string
                        long ch = _tcstoul(t, NULL, 16);
                        switch(ch)
                           { /* ch */
                            case 0x00: // end of string
                               color = RGB(0,0,0);
                               dc->SelectObject(&bold);
                               break;
                            case 0x01: // end of section
                               color = RGB(0,0,0);
                               dc->SelectObject(&bold);
                               switch(state)
                                  { /* 0x01 state */
                                   case CHARACTER_WEIGHTS:
                                      state = DIACRITIC_WEIGHTS;
                                      break;
                                   case DIACRITIC_WEIGHTS:
                                      state = CASE_WEIGHTS;
                                      break;
                                   case CASE_WEIGHTS:
                                      state = SPECIAL_WEIGHTS;
                                      break;
                                  } /* 0x01 state */
                               break;
                            default:
                               dc->SelectObject(&normal);
                               switch(state)
                                  { /* current state */
                                   case CHARACTER_WEIGHTS:
                                      color = COLOR_CHARACTER_WEIGHTS;
                                      break;
                                   case DIACRITIC_WEIGHTS:
                                      color = COLOR_DIACRITIC_WEIGHTS;
                                      break;
                                   case CASE_WEIGHTS:
                                      color = COLOR_CASE_WEIGHTS;
                                      break;
                                   case SPECIAL_WEIGHTS:
                                      color = COLOR_SPECIAL_WEIGHTS;
                                      break;
                                   default:
                                      ASSERT(FALSE);
                                      break;
                                  } /* current state */
                               break;
                           } /* ch */
                        // Now output the character
                        // The states are
                        // Focus   Selected     fill    text
                        //--------------------------------------
                        //   Y         Y        select  COLOR_HIGHLIGHTTEXT
                        //   N         Y        window  color
                        //   Y         N        unselect color
                        //   N         N        unselect color
                        switch(ColorOption)
                           { /* ColorOption */
                            case FocusSelected:
                               dc->SetTextColor(text);
                               break;
                            case NoFocusSelected:
                               dc->SetTextColor(text);
                               break;
                            case NotSelected:
                               dc->SetTextColor(color);
                               break;
                           } /* ColorOption */

                        // Now handle the clipping-on-the-right
                        BOOL stop = FALSE;
                        CSize sz = dc->GetTextExtent(t + _T("..."));
                        if(x - start + sz.cx >= width)
                           { /* too long */
                            t = _T("...");
                            stop = TRUE;
                           } /* too long */
                        dc->TextOut(x, dis->rcItem.top, t);
                        x += dc->GetTextExtent(t).cx;
                        if(stop)
                           break;
                       } /* extract */

                    dc->RestoreDC(save2);
                   } /* byte sequence string */
                else
                   { /* expanded string */
                    dc->SelectObject(&bold);
                    dc->TextOut(x, dis->rcItem.top, s);
                   } /* expanded string */
               } /* fixed-pitch formatting */
           } /* special highlighting */
        else
           { /* normal text */
            if(col == 0 && s.Left(2) == _T("0x"))
               { /* hex value */
                CFont fixed;
                LOGFONT lf;
                CFont * font = GetFont();
                font->GetLogFont(&lf);
                lstrcpy(lf.lfFaceName, _T("Courier New"));
                fixed.CreateFontIndirect(&lf);
                
                int save = dc->SaveDC();
                dc->SelectObject(&fixed);
                dc->TextOut(x, dis->rcItem.top, s);
                dc->RestoreDC(save);
               } /* hex value */
            else
               { /* vanilla text */
                dc->TextOut(x, dis->rcItem.top, s);
               } /* vanilla text */
           } /* normal text */
        //----------------------------------------------------------------
        // Adjust the position for the next column
        //----------------------------------------------------------------
        pos += width;
       } /* scan columns */

    dc->RestoreDC(save);
   }

/****************************************************************************
*                         CStringDisplay::SetSortKey
* Inputs:
*       BOOL sk: Sortkey flag; TRUE to format as sort key, FALSE as text
*       int col: Column to be formatted
* Result: void
*       
* Effect: 
*       Forces a refresh
****************************************************************************/

void CStringDisplay::SetSortKey(BOOL sk, int col)
    {
     BOOL changed = FALSE;
     changed |= sk != UseSortKey;
     changed |= col != SortCol;
     
     UseSortKey = sk; SortCol = col;
     if(changed)
        Invalidate();
    } // CStringDisplay::SetSortKey

/****************************************************************************
*                          CStringDisplay::OnHScroll
* Inputs:
*       UINT nSBCode:
*       UINT nPos:
*       CScrollBar * pScrollBar:
* Result: void
*       
* Effect: 
*       For an owner-draw control, horizontal scrolling produces very strange
*       appearances unless we redraw the entire control
****************************************************************************/

void CStringDisplay::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
   {
    Invalidate();
    CListCtrlEx::OnHScroll(nSBCode, nPos, pScrollBar);
   }

/****************************************************************************
*                          CStringDisplay::OnSetFont
* Inputs:
*       WPARAM wParam: passed through implicitly via Default()
*       LPARAM lParam: passed through implicitly via Default()
* Result: LRESULT
*       passed back
* Effect: 
*       Forces a recomputation of the item height. When DefWindowProc
*       handles the WM_WINDOWPOSCHANGED message, it generates a
*       WM_MEASUREITEM
* Notes:
*       See Uwe Keim's article on www.codeproject.com on solving the
*       item height problem
****************************************************************************/

LRESULT CStringDisplay::OnSetFont(WPARAM, LPARAM)
    {
     LRESULT result = Default();

     CRect r;
     GetWindowRect(&r);

     WINDOWPOS wp;
     wp.hwnd = m_hWnd;
     wp.cx = r.Width();
     wp.cy = r.Height();
     wp.flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER;
     SendMessage(WM_WINDOWPOSCHANGED, 0, (LPARAM)&wp);
#ifdef _DEBUG //----------------------------------------------------------------+
     LOGFONT lf;                                                             // |
     GetFont()->GetLogFont(&lf);                                             // |
#endif //-----------------------------------------------------------------------+
     return result;
    } // CStringDisplay::OnSetFont

/****************************************************************************
*                         CStringDisplay::MeasureItem
* Inputs:
*       LPMEASUREITEMSTRUCT mis: Structure to fill in
* Result: void
*       
* Effect: 
*       Computes the item height
****************************************************************************/

void CStringDisplay::MeasureItem(LPMEASUREITEMSTRUCT mis)
    {
     LOGFONT lf;
     GetFont()->GetLogFont(&lf);
     mis->itemHeight = abs(lf.lfHeight) + 4;
    } // CStringDisplay::MeasureItem
