#include "stdafx.h"
#include "TranslateDigits.h"

/****************************************************************************
*                               TranslateDigits
* Inputs:
*       LCID lcid: Locale ID
*       CString & digits: Digit string
* Result: BOOL
*       TRUE if string was changed
*       FALSE if not changed, or there was an error
*       If FALSE, and ::GetLastError() is ERROR_SUCCESS, it means there
*       were no digits changed
*       If FALSE, and ::GetLastError() is NOT ERROR_SUCCESS, it means
*       the attempt to get the translated digits failed
* Effect: 
*       Replaces the digits 0..9 in the input string with the locale-specific
*       digitis
****************************************************************************/

BOOL TranslateDigits(LCID lcid, CString & digits, SCRIPT_DIGITSUBSTITUTE * digitinfo)
    {
#ifdef _UNICODE
#if 1
     BOOL ok = TRUE;
     DWORD err = ::GetLastError();
     CString snativedigits_data;
     { /* get LOCALE_SNATIVEDIGITS */
      int length = GetLocaleInfo(lcid, LOCALE_SNATIVEDIGITS, NULL, 0);
      LPTSTR p = snativedigits_data.GetBuffer(length);
      if(!::GetLocaleInfo(lcid, LOCALE_SNATIVEDIGITS, p, length))
         { /* failed */
          err = ::GetLastError();
          snativedigits_data.ReleaseBuffer();
          snativedigits_data = _T("0123456789");
          ok = FALSE;
         } /* failed */
      else
         { /* success */
          snativedigits_data.ReleaseBuffer();
          err = ERROR_SUCCESS;
         } /* success */
     } /* get LOCALE_SNATIVEDIGITS */

     // Replace the digits
     BOOL change = FALSE;
     for(int i = 0; i < digits.GetLength(); i++)
        { /* replace digits */
         TCHAR ch = digits[i];
         static const CString digitcharacters = _T("0123456789");
         int p = digitcharacters.Find(ch);
         if(p >= 0)
            { /* replace */
             if(ch != snativedigits_data[p])
                { /* different */
                 digits.Replace(ch, snativedigits_data[p]);
                 change = TRUE;
                } /* different */
            } /* replace */
        } /* replace digits */
     ok &= change;
     ::SetLastError(err);
     return ok;
#else
     SCRIPT_CONTROL scriptcontrol;
     SCRIPT_STATE scriptstate;
     HRESULT result = ::ScriptApplyDigitSubstitution(digitinfo, &scriptcontrol, &scriptstate);
     int maxitems = 2; // start with 2
     SCRIPT_ITEM * items;
     HRESULT converted;
     int actualitems;
     while(TRUE)
        { /* try to convert */
         items = new SCRIPT_ITEM[maxitems];
         converted = ::ScriptItemize((LPCWSTR)digits, digits.GetLength(), maxitems, &scriptcontrol, &scriptstate, items, &actualitems);
         if(converted == S_OK)
            break;
         delete [] items;
         if(converted == E_OUTOFMEMORY)
            { /* reallocate */
             maxitems *= 2; // try again
             continue;
            } /* reallocate */
         break;
        } /* try to convert */
     if(converted != S_OK)
         { /* failed */
          delete [] items;
          return FALSE;
         } /* failed */

     return TRUE;
#endif
#else // _UNICODE
     return TRUE;
#endif // _UNICODE
    } // TranslateDigits
