// Unicode.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "Unicode.h"
#include "msg.h"
#include "DeferredMessage.h"
#include "Query.h"
#include "RegVars.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CUnicode, CFunction)

//****************************************************************
// Code points display columns
//****************************************************************
#define COL_FIRST 0
#define COL_LAST  1
#define COL_BLOCK 2

//****************************************************************
// UCD display information
//****************************************************************
#define COL_CODEPOINT                 0
#define COL_NAME                      1
#define COL_GENERAL_CATEGORY          2
#define COL_CANONICAL_COMBINING_CLASS 3
#define COL_BIDI_CLASS                4
#define COL_DECOMPOSITION_TYPE        5
#define COL_NUMERIC_TYPE              6
#define COL_NUMERIC_DIGIT             7
#define COL_NUMERIC_VALUE             8
#define COL_BIDI_MIRRORED             9
#define COL_UNICODE_1_NAME           10
#define COL_ISO_COMMENT              11
#define COL_UPPERCASE_MAPPING        12
#define COL_LOWERCASE_MAPPING        13
#define COL_TITLECASE_MAPPING        14

/****************************************************************************
*                              UWM_ADD_ELEMENTS
* Inputs:
*       WPARAM: count of elements to add
*       LPARAM: unused
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Handles asynchronous updates
****************************************************************************/

#define UWM_ADD_ELEMENTS_MSG _T("UWM_ADD_ELEMENTS-{53287F9C-C24C-4be4-9884-DACCE71CF3D7}")

DECLARE_MESSAGE(UWM_ADD_ELEMENTS)

/////////////////////////////////////////////////////////////////////////////
// CUnicode dialog


CUnicode::CUnicode(CWnd* pParent /*=NULL*/)
        : CFunction(CUnicode::IDD, pParent)
{
        //{{AFX_DATA_INIT(CUnicode)
                // NOTE: the ClassWizard will add member initialization here
        //}}AFX_DATA_INIT
}


void CUnicode::DoDataExchange(CDataExchange* pDX)
{
        CFunction::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CUnicode)
        DDX_Control(pDX, IDC_PROGRESS, c_Progress);
        DDX_Control(pDX, IDC_FULLWIDTH, c_FullWidth);
        DDX_Control(pDX, IDC_CUSTOM, c_Custom);
        DDX_Control(pDX, IDC_CHOOSE_CUSTOM, c_ChooseCustom);
        DDX_Control(pDX, IDC_BESTFIT, c_BestFit);
        DDX_Control(pDX, IDC_UNICODE_INFO, c_UnicodeInfo);
        DDX_Control(pDX, IDC_CODEPOINTS, c_CodePoints);
        //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CUnicode, CFunction)
        ON_NOTIFY(HDN_ITEMCLICK, IDC_UNICODE_INFO, OnItemclickUnicodeInfo)
//ON_NOTIFY(HDN_ITEMCLICKW, IDC_UNICODE_INFO, OnItemclickUnicodeInfo)
//        ON_NOTIFY(HDN_ITEMCLICKA, 0, OnItemclickUnicodeInfo)
//        ON_NOTIFY(HDN_ITEMCLICKW, 0, OnItemclickUnicodeInfo)
//        ON_NOTIFY(HDN_ITEMCLICK, 0, OnItemclickUnicodeInfo)

        //{{AFX_MSG_MAP(CUnicode)
        ON_NOTIFY(NM_DBLCLK, IDC_CODEPOINTS, OnDblclkCodepoints)
        ON_BN_CLICKED(IDC_CUSTOM, OnCustom)
        ON_BN_CLICKED(IDC_BESTFIT, OnBestfit)
        ON_BN_CLICKED(IDC_FULLWIDTH, OnFullwidth)
        ON_BN_CLICKED(IDC_CHOOSE_CUSTOM, OnChooseCustom)
        ON_WM_SIZE()
        //}}AFX_MSG_MAP
        ON_REGISTERED_MESSAGE(UWM_ADD_ELEMENTS, OnAddElements)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUnicode message handlers

/****************************************************************************
*                          CUnicode::LoadCodepoints
* Result: void
*       
* Effect: 
*       Loads the code points display
* Notes:
*       Input is a resource of the form
*       
*       <empty line>
*       # <comment>
*       first..last; block
****************************************************************************/

void CUnicode::LoadCodepoints()
    {
     c_CodePoints.InsertColumnEx(COL_FIRST, IDS_CODEPOINT_FIRST, LVCFMT_LEFT, _T("________"));
     c_CodePoints.InsertColumnEx(COL_LAST,  IDS_CODEPOINT_LAST,  LVCFMT_LEFT, _T("________"));
     c_CodePoints.InsertColumnEx(COL_BLOCK, IDS_CODEPOINT_BLOCK, LVCFMT_LEFT, -1);

     HRSRC resource;
     resource = ::FindResource(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_BLOCKS), _T("BLOCKS"));
     if(resource == NULL)
        { /* failed */
         DWORD err = ::GetLastError();
        // TODO: Report error here
         return;
        } /* failed */
     HGLOBAL h = ::LoadResource(AfxGetInstanceHandle(), resource);
     if(h == NULL)
        { /* load failed */
         DWORD err = ::GetLastError();
        // TODO: Report error here
         return;
        } /* load failed */

     LPCSTR p = (LPCSTR)::LockResource(h); // note this is an 8-bit resource!
     if(p == NULL)
        { /* lock failed */
         DWORD err = ::GetLastError();
         // TODO: report error here
         return;
        } /* lock failed */

     CString blocks(p);

     for(int i = 0; ;)
        { /* add each */
         int n = blocks.Find(_T("\r\n"));
         if(n < 0)
            break; // out of data
         
         CString line;
         line = blocks.Left(n);
         blocks = blocks.Mid(n + 2);

         // # comment
         // <empty>
         // first..last; block
         line.TrimLeft();
         line.TrimRight();
         if(line.IsEmpty())
            continue; // blank line, ignore
         if(line[0] == _T('#'))
            continue;  // comment line
         
         CString first;
         CString last;
         CString block;

         n = line.Find(_T(".."));
         if(n < 0)
            continue; // illegal line
         first = line.Left(n);
         line = line.Mid(n + 2);
         n = line.Find(_T(";"));
         if(n < 0)
            continue; // illegal line
         last = line.Left(n);
         block = line.Mid(n + 1);
         
         first.TrimLeft();
         first.TrimRight();
         last.TrimLeft();
         last.TrimRight();
         block.TrimLeft();
         block.TrimRight();

         c_CodePoints.InsertItem(i, first);
         c_CodePoints.SetItemText(i, COL_LAST, last);
         c_CodePoints.SetItemText(i, COL_BLOCK, block);
         c_CodePoints.SetItemData(i, _tcstoul(first, NULL, 16));
         i++;
        } /* add each */
    } // CUnicode::LoadCodepoints

#define COL_CODEPOINT                 0
#define COL_NAME                      1
#define COL_GENERAL_CATEGORY          2
#define COL_CANONICAL_COMBINING_CLASS 3
#define COL_BIDI_CLASS                4
#define COL_DECOMPOSITION_TYPE        5
#define COL_NUMERIC_TYPE              6
#define COL_NUMERIC_DIGIT             7
#define COL_NUMERIC_VALUE             8
#define COL_BIDI_MIRRORED             9
#define COL_UNICODE_1_NAME           10
#define COL_ISO_COMMENT              11
#define COL_UPPERCASE_MAPPING        12
#define COL_LOWERCASE_MAPPING        13
#define COL_TITLECASE_MAPPING        14
#define COL_GLYPH                    15
#define COL_MAX                      15 // highest column # supported

typedef enum {Fixed, Zero, LowCompression, MediumCompression, HighCompression, SuperHighCompression } CompressionOptions;
static const CompressionOptions BestFitChoices[] = {
   Fixed,           // Codepoint
   HighCompression,  // Name
   Fixed, // Genercal category
   Fixed, // Canonical combining class
   Fixed, // BIDI class
   SuperHighCompression,  // Decomposition type
   Fixed,   // numeric type
   Fixed,   // numeric digit
   Fixed,   // numeric value
   Fixed,   // BiDi mirrored
   Zero,    // Unicode 1.0 name
   HighCompression,  // ISO Comment
   Fixed,   // uppercase mapping
   Fixed,   // lowercase mapping
   Fixed,   // titlecase mapping
   Fixed    // glyph
};

/****************************************************************************
*                             CUnicode::LoadInfo
* Result: void
*       
* Effect: 
*       Loads the Unicode information
****************************************************************************/

void CUnicode::LoadInfo()
    {
     c_Header.SubclassWindow(c_UnicodeInfo.GetHeaderCtrl()->m_hWnd);
     ::SetWindowLong(c_Header.m_hWnd, GWL_ID, c_UnicodeInfo.GetDlgCtrlID());
                            

     
// This macro assumes the header text will be rotated 90, left aligned.  If it is desired to have
// a different orientation, an Explicit SetTextAlign can follow the use of this macro.
// Similarly, it assumes the body text will be left aligned; call the
// SetColumnAlignment method to change its alignmnent

#define ADDCOL(field) {                                                                                       \
                       c_UnicodeInfo.InsertColumnEx(COL_##field, IDS_UNICODE_##field, LVCFMT_LEFT, _T("___")); \
                       c_Header.SetTextAlign(COL_##field, DT_CENTER | DT_90);                                  \
                      }

     ADDCOL(CODEPOINT);
     c_Header.SetTextAlign(COL_CODEPOINT, DT_LEFT | DT_BOTTOM | DT_0);

     ADDCOL(NAME);
     c_Header.SetTextAlign(COL_NAME, DT_LEFT | DT_BOTTOM | DT_0);

     ADDCOL(GENERAL_CATEGORY);
     ADDCOL(CANONICAL_COMBINING_CLASS);
     c_UnicodeInfo.SetColumnAlignment(COL_CANONICAL_COMBINING_CLASS, LVCFMT_CENTER);

     ADDCOL(BIDI_CLASS);

     ADDCOL(DECOMPOSITION_TYPE);
     c_Header.SetTextAlign(COL_DECOMPOSITION_TYPE, DT_LEFT | DT_BOTTOM | DT_0);

     ADDCOL(NUMERIC_TYPE);
     ADDCOL(NUMERIC_DIGIT);

     ADDCOL(NUMERIC_VALUE);
     c_UnicodeInfo.SetColumnAlignment(COL_NUMERIC_VALUE, LVCFMT_RIGHT);

     ADDCOL(BIDI_MIRRORED);
     c_UnicodeInfo.SetColumnAlignment(COL_BIDI_MIRRORED, LVCFMT_CENTER);

     ADDCOL(UNICODE_1_NAME);
     c_Header.SetTextAlign(COL_UNICODE_1_NAME, DT_LEFT | DT_BOTTOM | DT_0);

     ADDCOL(ISO_COMMENT);
     ADDCOL(UPPERCASE_MAPPING);
     ADDCOL(LOWERCASE_MAPPING);
     ADDCOL(TITLECASE_MAPPING);
     ADDCOL(GLYPH);
     c_UnicodeInfo.SetColumnAlignment(COL_GLYPH, LVCFMT_CENTER);
     
     if(!unicode.Load())
        { /* failed */
         ASSERT(FALSE);
         return;
        } /* failed */

#define NELEMENTS 10

     c_Progress.SetRange32(0, unicode.GetRange());
     c_Progress.SetPos(0);
     CurrentPos = unicode.GetFirstCode();
     EnqueueMessage(this, UWM_ADD_ELEMENTS, NELEMENTS);
    } // CUnicode::LoadInfo

/****************************************************************************
*                           CUnicode::OnAddElements
* Inputs:
*       WPARAM wParam: Number of elements to add
*       LPARAM lParam: ingored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       
****************************************************************************/

LRESULT CUnicode::OnAddElements(WPARAM wParam, LPARAM)
    {
     int count = (int)wParam;
     CClientDC dc(&c_UnicodeInfo);

     int borders = dc.GetTextExtent(CString(_T("  "))).cx;

     while(CurrentPos != NULL)
        { /* has code points */
         if(count <= 0)
            { /* go away for a while */
             EnqueueMessage(this, UWM_ADD_ELEMENTS, NELEMENTS);
             c_Progress.SetPos(unicode.GetPos(CurrentPos));
             return 0;
            } /* go away for a while */

         count--;  // decrement count
         LPCSTR code = unicode.GetNextCode(CurrentPos);
         if(code == NULL)
            break;
      
         UnicodeData::Substring codepoint = unicode.GetField(COL_CODEPOINT, code);
         CString cp = codepoint.GetString();
         int n = c_UnicodeInfo.GetItemCount();

         n = c_UnicodeInfo.InsertItem(n, cp);
         c_UnicodeInfo.SetItemData(n, (DWORD_PTR)code);
         CSize sz = dc.GetTextExtent(cp);
         c_UnicodeInfo.SetPeakColumnWidth(COL_CODEPOINT, sz.cx + borders);

         LVITEM subitem = {0};
         subitem.mask = LVIF_TEXT;
         subitem.iItem = n;

         UnicodeData::Substring name = unicode.GetField(COL_NAME, code);
         CString nm = name.GetString();
         sz = dc.GetTextExtent(nm);
         c_UnicodeInfo.SetPeakColumnWidth(COL_NAME, sz.cx + borders);

         subitem.pszText = (LPTSTR)(LPCTSTR)nm;
         subitem.iSubItem = COL_NAME;
         c_UnicodeInfo.SetItem(&subitem);

         UnicodeData::Substring css = unicode.GetField(COL_GENERAL_CATEGORY, code);
         for(int i = COL_GENERAL_CATEGORY; i <= COL_TITLECASE_MAPPING; i++)
            { /* scan columns */
             if(css.GetLength() > 0)
                { /* add element */
                 CString s = css.GetString();
                 sz = dc.GetTextExtent(s);
                 c_UnicodeInfo.SetPeakColumnWidth(i, sz.cx + borders);
                 subitem.pszText = (LPTSTR)(LPCTSTR)s;
                 subitem.iSubItem = i;
                 c_UnicodeInfo.SetItem(&subitem);
                } /* add element */
             css = unicode.GetNextField(css);
            } /* scan columns */

         UINT ch = _tcstoul(cp, NULL, 16);
         if(ch <= 0xFFFF)
            { /* set glyph */
             CString s((TCHAR)ch);
             sz = dc.GetTextExtent(s);
             c_UnicodeInfo.SetPeakColumnWidth(COL_GLYPH, sz.cx + 2 * borders);
             subitem.pszText = (LPTSTR)(LPCTSTR)s;
             subitem.iSubItem = COL_GLYPH;
             c_UnicodeInfo.SetItem(&subitem);
            } /* set glyph */
        } /* has code points */

#if 0
     CRect r;
     GetClientRect(&r);

     CRect w;
     c_UnicodeInfo.GetWindowRect(&w);
     ScreenToClient(&w);
     c_UnicodeInfo.SetWindowPos(NULL,
                                0, 0,  // no move
                                r.Width() - w.left,
                                w.Height(),
                                SWP_NOMOVE | SWP_NOZORDER);
#endif
     c_UnicodeInfo.SetGlyphColumn(COL_GLYPH);
     
     c_UnicodeInfo.EnableWindow(TRUE);
     c_CodePoints.EnableWindow(TRUE);
     c_UnicodeInfo.Invalidate(); // force redraw to get correct colors
     c_Header.Invalidate(); // force redraw to kill artifacts of resizing

     c_Custom.EnableWindow(TRUE);
     c_FullWidth.EnableWindow(TRUE);
     c_BestFit.EnableWindow(TRUE);
     c_Progress.ShowWindow(SW_HIDE);

     fullwidths.SetSize(COL_MAX + 1);
     for(int i = 0; i <= COL_MAX; i++)
        { /* save widths */
         fullwidths[i] = c_UnicodeInfo.GetColumnWidth(i);
        } /* save widths */
     return 0;
    } // CUnicode::OnAddElements

/****************************************************************************
*                           CUnicode::OnInitDialog
* Result: BOOL
*       TRUE, always
* Effect: 
*       Initializes the dialog
****************************************************************************/

BOOL CUnicode::OnInitDialog() 
   {
    CFunction::OnInitDialog();
        
    // TODO: Add extra initialization here
    LoadCodepoints();
    c_UnicodeInfo.EnableWindow(FALSE);
    c_CodePoints.EnableWindow(FALSE);
    c_Custom.EnableWindow(FALSE);
    c_FullWidth.EnableWindow(FALSE);
    c_BestFit.EnableWindow(FALSE);
    c_Progress.ShowWindow(SW_SHOW);

    RegistryDWordArray cols(IDS_REGISTRY_CUSTOM);
    if(!cols.load())
       custom.SetSize(0);
    else
       { /* have data */
        custom.SetSize(COL_MAX + 1);
        for(int i = 0; i < cols.value.GetSize(); i++)
           { /* copy values */
            custom[i] = (int)cols.value[i];
           } /* copy values */
       } /* have data */

    CheckRadioButton(IDC_FULLWIDTH, IDC_CUSTOM, IDC_FULLWIDTH);
    LoadInfo();

    updateControls();

    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
   }

/****************************************************************************
*                              CUnicode::IndexOK
* Inputs:
*       CWnd * wnd: Window to index
* Result: BOOL
*       TRUE to index it, FALSE to skip it
****************************************************************************/

BOOL CUnicode::IndexOK(CWnd * wnd)
    {
     return FALSE; // right now, all pages are non-indexable
    } // CUnicode::IndexOK

/****************************************************************************
*                              CUnicode::Refresh
* Result: void
*       
* Effect: 
*       For this page, does nothing but the superclass issues a warning so
*       we need to override here
****************************************************************************/

void CUnicode::Refresh()
    {
     if(!initialized)
        return;

     EnableQuery(QUERY_DWORD, FALSE);
     EnableQuery(QUERY_NOUSEROVERRIDE, FALSE);
     EnableQuery(QUERY_USE_CP_ACP, FALSE);

     QueryTemplate(_T(""));
     SetCode(_T(""), FALSE);
    } // CUnicode::Refresh

/****************************************************************************
*                            CUnicode::Initialize
* Result: void
*       
* Effect: 
*       Initializes
****************************************************************************/

void CUnicode::Initialize()
    {
     // Placeholder here
    } // CUnicode::Initialize

/****************************************************************************
*                         CUnicode::SetFontToControls
* Result: void
*       
* Effect: 
*       Does nothing for now
****************************************************************************/

void CUnicode::SetFontToControls()
    {
     CFont * f = c_Header.GetFont();
     c_CodePoints.SetFont(&UnicodeFont);
     c_UnicodeInfo.SetFont(&UnicodeFont);
     c_Header.SetFont(f);
    } // CUnicode::SetFontToControls

/****************************************************************************
*                        CUnicode::OnDblclkCodepoints
* Inputs:
*       NMHDR * pNMHDR:
*       LRESULT * pResult:
* Result: void
*       
* Effect: 
*       Selects the code point index in the UnicodeInfo display
****************************************************************************/

void CUnicode::OnDblclkCodepoints(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    *pResult = 0;

    
    POSITION pos = c_CodePoints.GetFirstSelectedItemPosition();
    if(pos == NULL)
       return;  // should not happen

    // This is a single-selection control
    int n = c_CodePoints.GetNextSelectedItem(pos);

    UINT cp = c_CodePoints.GetItemData(n);

    int newsel = SearchUnicodeInfo(cp);
    if(newsel < 0)
       return;
    
    POSITION oldpos = c_UnicodeInfo.GetFirstSelectedItemPosition();
    if(oldpos != NULL)
       { /* deselect */
        int oldsel = c_UnicodeInfo.GetNextSelectedItem(oldpos);
        c_UnicodeInfo.SetItemState(oldsel, 0, LVIS_SELECTED);
       } /* deselect */

    c_UnicodeInfo.SetItemState(newsel, LVIS_SELECTED, LVIS_SELECTED);
    c_UnicodeInfo.EnsureVisible(newsel, FALSE);
    c_UnicodeInfo.SetFocus();
   }

/****************************************************************************
*                         CUnicode::SearchUnicodeInfo
* Inputs:
*       UINT cp: Desired codepoint
* Result: int
*       First codepoint index that is >= cp
* Notes: 
*       This is basically binary search, but we will settle for "near" rather
*       that "equal", e.g., if the value is 0x0500, we can return 0x0530
*       if that is the first codepoint above the range given
****************************************************************************/

int CUnicode::SearchUnicodeInfo(UINT cp)
    {
    // Now do a binary-search to locate the item
    // Note that unlike bsearch, this will return the
    // first codepoint that is >= the desired search value
     int lo = 0;
     int num = c_UnicodeInfo.GetItemCount();
     int hi = num;
     int mid;
     int half;
     int best = -1;

     while(lo <= hi)
        { /* scan halves */
         if(half = num / 2)
            { /* can go down */
             mid = lo + (num & 1 ? half : (half - 1));
            // Get the value
             CString t = c_UnicodeInfo.GetItemText(mid, COL_CODEPOINT);
             UINT codepoint = _tcstoul(t, NULL, 16); // convert to integer
             TRACE(_T("[%d] %s=%d cp=0x%04x (%d)\n"), mid, t, codepoint, cp, cp);
             if(cp == codepoint)
                return mid;  // exact hit
             best = mid;
             if(cp < codepoint)
                { /* go backward */
                 hi = mid - 1;
                 num = (num & 1) ? half : half - 1;
                } /* go backward */
             else
                { /* go forward */
                 lo = mid + 1;
                 num = half;
                } /* go forward */
            } /* can go down */
         else
            { /* no more choices */
             CString t = c_UnicodeInfo.GetItemText(lo, COL_CODEPOINT);
             UINT codepoint = _tcstoul(t, NULL, 16); // convert to integer
             TRACE(_T("[%d]* %s=%d cp=0x%04x (%d)\n"), mid, t, codepoint, cp, cp);
             if(codepoint < cp && lo < c_UnicodeInfo.GetItemCount() - 1)
                return lo + 1;
             else
                return lo;
             return best;
            } /* no more choices */
        } /* scan halves */
     return -1; // error
    } // CUnicode::SearchUnicodeInfo


/****************************************************************************
*                             CUnicode::OnCustom
* Result: void
*       
* Effect: 
*       Chooses the user-defined custom column widths
****************************************************************************/

void CUnicode::OnCustom() 
   {
    if(custom.GetSize() > 0)
       { /* use custom */
        for(int i = 0; i < custom.GetSize(); i++)
           { /* resize */
            c_UnicodeInfo.SetColumnWidth(i, custom[i]);
           } /* resize */
       } /* use custom */
    updateControls();
   }

/****************************************************************************
*                             CUnicode::SumWidths
* Result: int
*       The sum of all the best-fit widths
****************************************************************************/

int CUnicode::SumWidths()
    {
     int sum = 0;
     for(int i = 0; i <= COL_MAX; i++)
        { /* compute width */
         sum += bestwidths[i];
        } /* compute width */
     return sum;
    } // CUnicode::SumWidths

/****************************************************************************
*                           CUnicode::AdjustWidths
* Inputs:
*       int width: The desired width
* Result: void
*       
* Effect: 
*       Computes a new adjusted width
****************************************************************************/

void CUnicode::AdjustWidths(int width)
    {
     int sum = SumWidths();
     if(sum <= width)
        return;

#define LOW_COMPRESSION        1
#define MEDIUM_COMPRESSION     3
#define HIGH_COMPRESSION       9
#define SUPERHIGH_COMPRESSION 18

#define COMPRESS_IT(compression) \
                                 bestwidths[i] = max(0, bestwidths[i] - (compression)); \
                                 sum = SumWidths();  \
                                 if(sum < width) \
                                    { /* adjust it */  \
                                     int delta = width - sum; \
                                     sum += width - sum; \
                                     bestwidths[i] += delta; \
                                    } /* adjust it */

     while(TRUE)
        { /* adjust widths */
         for(int i = 0; i <= COL_MAX; i++)
            { /* drop each */
             switch(BestFitChoices[i])
                { /* adjust */
                 case Fixed:
                    continue;  // ignore this one
                 case Zero:
                    COMPRESS_IT(bestwidths[i]);
                    break;
                 case LowCompression:
                    COMPRESS_IT(LOW_COMPRESSION);
                    break;
                 case MediumCompression:
                    COMPRESS_IT(MEDIUM_COMPRESSION);
                    break;
                 case HighCompression:
                    COMPRESS_IT(HIGH_COMPRESSION);
                    break;
                 case SuperHighCompression:
                    COMPRESS_IT(SUPERHIGH_COMPRESSION);
                    break;
                } /* adjust */
             if(sum <= width)
                return;
            } /* drop each */
        } /* adjust widths */
    } // CUnicode::AdjustWidths

/****************************************************************************
*                             CUnicode::OnBestFit
* Result: void
*       
* Effect: 
*       Tries to fit everything into the window
****************************************************************************/

void CUnicode::OnBestfit() 
   {
    CRect r;
    c_UnicodeInfo.GetClientRect(&r); // get max size

    DWORD width = r.Width();

    bestwidths.Copy(fullwidths);

    AdjustWidths(width);

    for(int i = 0; i <= COL_MAX; i++)
       { /* change widths */
        c_UnicodeInfo.SetColumnWidth(i, bestwidths[i]);
        TRACE(_T("col[%d]=%d\n"), i, bestwidths[i]);
       } /* change widths */
    updateControls();
   }

/****************************************************************************
*                            CUnicode::OnFullwidth
* Result: void
*       
* Effect: 
*       Reverts to the original widths
****************************************************************************/

void CUnicode::OnFullwidth() 
   {
    for(int i = 0; i <= COL_MAX; i++)
       { /* set widths */
        c_UnicodeInfo.SetColumnWidth(i, fullwidths[i]);
       } /* set widths */
    updateControls();
   }

/****************************************************************************
*                          CUnicode::OnChooseCustom
* Result: void
*       
* Effect: 
*       Chooses custom widths
****************************************************************************/

void CUnicode::OnChooseCustom() 
   {
    custom.SetSize(COL_MAX + 1);
    for(int i = 0; i <= COL_MAX; i++)
       { /* store current */
        custom[i] = c_UnicodeInfo.GetColumnWidth(i);
       } /* store current */

    switch(AfxMessageBox(IDS_QUERY_SET_SIZES, MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2))
       { /* response */
        case IDNO:
           return;
        case IDYES:
           break;
         } /* response */

    for(int k = 0; k <= COL_MAX; k++)
       { /* set values */
        custom[k] = c_UnicodeInfo.GetColumnWidth(k);
       } /* set values */

    // Now save them
    RegistryDWordArray cols(IDS_REGISTRY_CUSTOM);
    cols.value.SetSize(custom.GetSize());
    for(int n = 0; n < custom.GetSize(); n++)
       cols.value[n] = (DWORD)custom[n];
    cols.store();
   }

/****************************************************************************
*                          CUnicode::updateControls
* Result: void
*       
* Effect: 
*       Updates the control states
****************************************************************************/

void CUnicode::updateControls()
    {
     c_ChooseCustom.EnableWindow(c_Custom.GetCheck() == BST_CHECKED);
    } // CUnicode::updateControls

/****************************************************************************
*                              CUnicode::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Resizes the Unicode display
****************************************************************************/

void CUnicode::OnSize(UINT nType, int cx, int cy) 
   {
    CFunction::OnSize(nType, cx, cy);
        
    if(c_UnicodeInfo.GetSafeHwnd() != NULL)
       { /* can resize */
        CRect r;
        c_UnicodeInfo.GetWindowRect(&r);
        ScreenToClient(&r);
        c_UnicodeInfo.SetWindowPos(NULL,
                                   0, 0, // no position change
                                   cx, cy - r.top,
                                   SWP_NOMOVE | SWP_NOZORDER);
       } /* can resize */
   }

/****************************************************************************
*                     CUnicode::OnItemclickedUnicodeInfo
* Inputs:
*       NMHDR pNMHDR
*       LRESULT * pResult: set to 0, always
* Result: void
*       
* Effect: 
*       Finds the next non-empty subitem in the control
****************************************************************************/

void CUnicode::OnItemclickUnicodeInfo(NMHDR* pNMHDR, LRESULT* pResult) 
   {
    HD_NOTIFY *hdn = (HD_NOTIFY *) pNMHDR;

    *pResult = 0;

    if(hdn->iButton != 0) // 0 means left button
       return;

    // The left button has been clicked
    int col = hdn->iItem;  // column
    
    int selection = 0; // first selection
    POSITION pos = c_UnicodeInfo.GetFirstSelectedItemPosition();
    while(pos != NULL)
       { /* scan */
        selection = c_UnicodeInfo.GetNextSelectedItem(pos);
       } /* scan */

    for(int i = selection + 1; i < c_UnicodeInfo.GetItemCount(); i++)
       { /* scan */
        CString s = c_UnicodeInfo.GetItemText(i, col);
        if(!s.IsEmpty())
           { /* found nonblank */
            c_UnicodeInfo.SetItemState(selection, 0, LVIS_SELECTED); // unselect current selection
            c_UnicodeInfo.SetItemState(i, LVIS_SELECTED, LVIS_SELECTED); // select latest one
            c_UnicodeInfo.EnsureVisible(i, FALSE);
            break;
           } /* found nonblank */
       } /* scan */

   }
