// UnicodeDisplay.cpp : implementation file
//

#include "stdafx.h"
#include "UnicodeDisplay.h"
#include "msg.h"
#include "gdiflush.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/****************************************************************************
*                              UWM_UPDATE_NEEDED
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Posted by the udpate logic if it sees the position has changed
*       and a character display is in effect
****************************************************************************/

#define UWM_UPDATE_NEEDED_MSG _T("UWM_UPDATE_NEEDED-{0F11D4A6-78B8-4358-BA2E-8B0267982D34}")

DECLARE_MESSAGE(UWM_UPDATE_NEEDED)


/////////////////////////////////////////////////////////////////////////////
// CUnicodeDisplay

CUnicodeDisplay::CUnicodeDisplay()
{
 visible = FALSE;
 oldtop = -1;
}

CUnicodeDisplay::~CUnicodeDisplay()
{
}


BEGIN_MESSAGE_MAP(CUnicodeDisplay, CListCtrlEx)
        ON_MESSAGE(WM_MOUSELEAVE, OnMouseLeave)
        ON_REGISTERED_MESSAGE(UWM_UPDATE_NEEDED, OnUpdateNeeded)
        //{{AFX_MSG_MAP(CUnicodeDisplay)
        ON_WM_MOUSEMOVE()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUnicodeDisplay message handlers

/****************************************************************************
*                          CUnicodeDisplay::DrawItem
* Inputs:
*       LPDRAWITEMSTRUCT dis
* Result: void
*       
* Effect: 
*       Draws the contents
****************************************************************************/

void CUnicodeDisplay::DrawItem(LPDRAWITEMSTRUCT dis)
    {
     CRect r;
     CDC * dc = CDC::FromHandle(dis->hDC);
     int save = dc->SaveDC();
     LVCOLUMN colinfo = {0};
     colinfo.mask = LVCF_FMT | LVCF_WIDTH;
     COLORREF fill;
     COLORREF text;
     UINT dtformat = 0;

     int newtop = GetTopIndex();
     if(newtop != oldtop)
        { /* need update */
         PostMessage(UWM_UPDATE_NEEDED);
         oldtop = newtop;
        } /* need update */

     int gap = 2 * ::GetSystemMetrics(SM_CXBORDER); 
     int offset = -GetScrollPos(SB_HORZ);

     //----------------------------------------------------------------
     // Compute the colors for fill and text
     //----------------------------------------------------------------
     if(dis->itemState & ODS_SELECTED)
        { /* selected */
         if(::GetFocus() == m_hWnd)
            { /* has focus */
             fill = ::GetSysColor(COLOR_HIGHLIGHT);
             text = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
            } /* has focus */
         else
            { /* no focus */
             fill = ::GetSysColor(COLOR_GRAYTEXT);
             text = ::GetSysColor(COLOR_WINDOWTEXT);
            } /* no focus */
        } /* selected */
     else
        { /* not selected */
#define BAND1_COLOR RGB(192,255,192)
#define BAND2_COLOR RGB(155,255,155)
#define DISABLED_BAND1_COLOR RGB(192,192,192)
#define DISABLED_BAND2_COLOR RGB(155,155,155)
         COLORREF band1;
         COLORREF band2;
         if(IsWindowEnabled())
            { /* enabled */
             text = ::GetSysColor(COLOR_WINDOWTEXT);
             band1 = BAND1_COLOR;
             band2 = BAND2_COLOR;
            } /* enabled */
         else
            { /* disabled */
             text = RGB(128,128,128);
             band1 = DISABLED_BAND1_COLOR;
             band2 = DISABLED_BAND2_COLOR;
            } /* disabled */

         switch(dis->itemID % 6)
            { /* color selection */
             case 0:
             case 1:
             case 2:
                fill = band1;
                break;
             case 3:
             case 4:
             case 5:
                fill = band2;
                break;
            } /* color selection */

        } /* not selected */

     for(int col = 0; GetColumn(col, &colinfo); col++)
        { /* scan columns */
         //----------------------------------------------------------------
         // Compute the rectangle
         //----------------------------------------------------------------
         GetSubItemRectEx(dis->itemID, col, LVIR_BOUNDS, r);
         int pos = r.left;

         dc->SetTextColor(text);

         //----------------------------------------------------------------
         // Fill the background. Note the adjustment down and the expansion
         // This looks better
         //----------------------------------------------------------------
         CRect filler = r;
         //filler.top += 1;
         filler.bottom += 2;

         dc->FillSolidRect(&filler, fill);
         GDI_FLUSH();

         //----------------------------------------------------------------
         // Compute the correct width of the column
         // Note that GetItemRect gives then ENTIRE rectangle, and
         // getting the subitem rect for item 0 does the same thing
         // So if we are in the first column, we ask for the second
         // column
         //----------------------------------------------------------------
         int width = r.Width();

         //----------------------------------------------------------------
         // Now compute the alignment. Note that if there is 
         // horizontal scrolling, we have to adjust for the scrolling
         // position
         //----------------------------------------------------------------

         int x; // = dis->rcItem.left + pos + offset;

         //----------------------------------------------------------------
         // Set up to honor the formatting alignment
         //----------------------------------------------------------------
         switch(colinfo.fmt & LVCFMT_JUSTIFYMASK)
            { /* fmt */
             case LVCFMT_CENTER:
                x = dis->rcItem.left + pos + width / 2 - offset;
                dc->SetTextAlign(TA_CENTER);
                dtformat = DT_CENTER;
                break;
             case LVCFMT_RIGHT:
                x = dis->rcItem.left + pos + width - offset - gap;
                dc->SetTextAlign(TA_RIGHT);
                dtformat = DT_RIGHT;
                break;
             case LVCFMT_LEFT:
             default:
                x = dis->rcItem.left + pos - offset;
                dc->SetTextAlign(TA_LEFT);
                dtformat = DT_LEFT;
                break;
            } /* fmt */

         //----------------------------------------------------------------
         // Set a clipping region so we don't paint outside the lines...
         // Note that we have to add +1 to deal with the fact that all
         // drawing is up-to-but-not-including the endpoint, and we
         // want to include the endpoint
         //----------------------------------------------------------------

         CRect clip(r.left, r.top, r.right - gap, r.bottom + 1);

         CRgn cliprgn;
         cliprgn.CreateRectRgn(clip.left, clip.top, clip.right, clip.bottom);
         { /* draw clipped */
          int save2 = dc->SaveDC();
          dc->SelectClipRgn(&cliprgn);

          //----------------------------------------------------------------
          // There appears to be a serious bug in the DrawText API, in that
          // it gives erroneous positioning when anything other than DT_LEFT
          // is specified!  For example, a sequence of right-justified
          // numbers oves out offset by the number of digits, such as
          //             8
          //             9
          //           10
          //           12
          //           50
          //         100
          //         500
          //       1000
          //       1000
          //       5000
          //     10000
          // Of course, this makes no sense; I've single-stepped through this
          // code and I'm passing in the correct parameters, and it still gets
          // it wrong.  But I want the automatic ellipses, so what I've chosen
          // to do here is use DrawText only when DT_LEFT is selected
          // For reasons not entirely understood, I get the wrong computation
          // returned for GetItemRect(0), so I special case col == 0 as well
         
          CString s = GetItemText(dis->itemID, col);
          CRect dtrect = clip;
          dtrect.left += offset;
          dtrect.right += offset;
          dtrect.top--;
         
          if(col > 0 && dtformat == DT_LEFT)
             dc->DrawText(s, &dtrect, dtformat | DT_WORD_ELLIPSIS);
          else
             dc->TextOut(x, dis->rcItem.top - 1, s);
          dc->RestoreDC(save2);
         } /* draw clipped */
         //pos += width;
        } /* scan columns */
     dc->RestoreDC(save);
    } // CUnicodeDisplay::DrawItem

/****************************************************************************
*                        CUnicodeDisplay::OnMouseMove
* Inputs:
*       UINT nFlags: ignored
*       CPoint point: ignored
* Result: void
*       
* Effect: 
*       If the mouse is moving over the glyph column pops up a nice big
*       window to display the glyph
****************************************************************************/

void CUnicodeDisplay::OnMouseMove(UINT nFlags, CPoint point) 
   {
    ActivatePopupDisplay(point);
    CListCtrlEx::OnMouseMove(nFlags, point);
   }

/****************************************************************************
*                    CUnicodeDisplay::ActivatePopupDisplay
* Inputs:
*       CPoint point: Point where mouse is found, in client coordinates
* Result: void
*       
* Effect: 
*       Activates the glyph wndow
****************************************************************************/

void CUnicodeDisplay::ActivatePopupDisplay(CPoint point)
    {
     if(GlyphColumn != -1)
        { /* have glyph */
         int i = ItemFromPoint(point);
         if(i < 0)
            { /* not found */
             HideCharacter();
             return; // not in point
            } /* not found */

        // Now further refine it 
         CRect g;
         GetSubItemRect(i, GlyphColumn, LVIR_BOUNDS, g);
#ifdef _DEBUG //----------------------------------------------------------------------------------------------------+
         { /* debug */                                                                                           // |
          TRACE(_T("g=[%d, %d, %d, %d] point=[%d, %d]\n"), g.left, g.top, g.right, g.bottom, point.x, point.y);  // |
         } /* debug */                                                                                           // |
#endif //-----------------------------------------------------------------------------------------------------------+
         if(g.PtInRect(point))
            { /* glyph hit */
             CString glyph = GetItemText(i, GlyphColumn);
             ShowCharacter(point, glyph);
             return;
            } /* glyph hit */

         LVCOLUMN data = {LVCF_WIDTH};
         CClientDC dc(this);
         CFont * f = GetFont();
         dc.SelectObject(f);
         for(int col = 0; GetColumn(col, &data); col++)
            { /* scan columns */
             CRect r;
             if(!GetSubItemRectEx(i, col, LVIR_BOUNDS, r))
                return;
             if(!r.PtInRect(point))
                continue;
             // Otherwise, see if we are in something that requires flyover help
             int w = data.cx;
             if(w  > 0)
                { /* has real column */
                 CString text = GetItemText(i, col);
                 CSize sz = dc.GetTextExtent(text);
                 if(sz.cx > w)
                    { /* show text */
                     ShowText(point, text);
                     return;
                    } /* show text */
                } /* has real column */
            } /* scan columns */
         HideCharacter();
        } /* have glyph */
    } // CUnicodeDisplay::ActivatePopupDisplay

/****************************************************************************
*                        CUnicodeDisplay::OnMouseLeave
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Hides the popup window
****************************************************************************/

LRESULT CUnicodeDisplay::OnMouseLeave(WPARAM, LPARAM)
    {
     HideCharacter();
     return 0;
    } // CUnicodeDisplay::OnMouseLeave

/****************************************************************************
*                         CUnicodeDisplay::HidePopup
* Result: void
*       
* Effect: 
*       Hides the popup window
****************************************************************************/

void CUnicodeDisplay::HidePopup()
    {
     if(!visible)
        return;
     if(popup.GetSafeHwnd() == NULL)
        return;

     popup.ShowWindow(SW_HIDE);  // in case it was showing
     TRACE(_T("hide\n"));
     visible = FALSE;
    } // CUnicodeDisplay::HidePopup

/****************************************************************************
*                       CUnicodeDisplay::HideCharacter
* Result: void
*       
* Effect: 
*       Hides the character display
****************************************************************************/

void CUnicodeDisplay::HideCharacter()
    {
     HidePopup();
    } // CUnicodeDisplay::HideCharacter

/****************************************************************************
*                        CUnicodeDisplay::PreShowPopup
* Inputs:
*       CPoint point: Place to display the window
* Result: void
*       
* Effect: 
*       Pops up the window if it is needed
****************************************************************************/

BOOL CUnicodeDisplay::PreShowPopup(CPoint point)
    {
     if(popup.GetSafeHwnd() == NULL)
        { /* create popup */
         if(!popup.Create(0, 0, this))
            return FALSE; // error
         visible = FALSE;
        } /* create popup */
     return TRUE;
    } // CUnicodeDisplay::PreShowPopup

/****************************************************************************
*                       CUnicodeDisplay::PostShowPopup
* Inputs:
*       CPoint point: Desired point
* Result: void
*       
* Effect: 
*       Displays the font
****************************************************************************/

void CUnicodeDisplay::PostShowPopup(CPoint point)
    {
     CPoint pt = point;
     pt.x -= ::GetSystemMetrics(SM_CXCURSOR);
     pt.y += ::GetSystemMetrics(SM_CYCURSOR);
     ClientToScreen(&pt);

     if(!visible)
        { /* need to show */
         popup.ShowWindow(SW_SHOWNA); // in case it isn't already shown
         TRACKMOUSEEVENT tme = {sizeof(TRACKMOUSEEVENT), TME_LEAVE, m_hWnd, 0};
         popup.SetWindowPos(NULL, pt.x, pt.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
         visible = TRUE;
#ifdef _DEBUG //----------------------------------------------------------------+
         { /* debug */                                                       // |
          TRACE(_T("show\n"));                                               // |
          CRect w;                                                           // |
          popup.GetWindowRect(&w);                                           // |
          TRACE(_T("glyph [%d, %d]\n"), w.left, w.top);                      // |
         } /* debug */                                                       // |
#endif //-----------------------------------------------------------------------+
         VERIFY(TrackMouseEvent(&tme));
        } /* need to show */
     else
        { /* just move it */
         popup.SetWindowPos(NULL, pt.x, pt.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
        } /* just move it */
    } // CUnicodeDisplay::PostShowPopup

/****************************************************************************
*                          CUnicodeDisplay::ShowText
* Inputs:
*       CPoint point:
*        const CString & text:
* Result: void
*       
* Effect: 
*       Pops up the text
****************************************************************************/

void CUnicodeDisplay::ShowText(CPoint point, const CString & text)
    {
     if(!PreShowPopup(point))
        return;

     CFont * f = GetFont();
     ASSERT(f != NULL);

     // At this point, we know we have a valid glyph window, and a
     // valid glyph
     popup.SetText(text); // no effect if character is the one that is defined
     popup.SetStyle(CShowGlyph::Text, f);

     PostShowPopup(point);
    } // CUnicodeDisplay::ShowText

/****************************************************************************
*                       CUnicodeDisplay::ShowCharacter
* Inputs:
*       CPoint point: Point at which to display it
*       const CString & glyph: Glyph
* Result: void
*       
* Effect: 
*       Displays the character window
****************************************************************************/

void CUnicodeDisplay::ShowCharacter(CPoint point, const CString & glyph)
    {
     if(glyph.IsEmpty())
        return;

     if(!PreShowPopup(point))
        return;

     // we now have a window, so we can set its font

     CFont * f = GetFont();
     ASSERT(f != NULL);

     // At this point, we know we have a valid glyph window, and a
     // valid glyph
     popup.SetText(glyph); // no effect if character is the one that is defined
     popup.SetStyle(CShowGlyph::Char, f);

     PostShowPopup(point);
    } // CUnicodeDisplay::ShowCharacter

/****************************************************************************
*                   CUnicodeDisplay::ReActivatePopupDisplay
* Result: void
*       
* Effect: 
*       Reactivates the glyph display
****************************************************************************/

void CUnicodeDisplay::ReActivatePopupDisplay()
    {
     CPoint point;
     GetCursorPos(&point);
     ScreenToClient(&point);
     ActivatePopupDisplay(point);
    } // CUnicodeDisplay::ReActivatePopupDisplay

/****************************************************************************
*                       CUnicodeDisplay::OnUpdateNeeded
* Inputs:
*       WPARAM: ignored
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Forces an update if the character window is visible
****************************************************************************/

LRESULT CUnicodeDisplay::OnUpdateNeeded(WPARAM, LPARAM)
    {
     ReActivatePopupDisplay();
     return 0;
    } // CUnicodeDisplay::OnUpdateNeeded
