// VerticalHeader.cpp : implementation file
//

#include "stdafx.h"
//#include "resource.h"
#include "VerticalHeader.h"
#include "gdiflush.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVerticalHeader

CVerticalHeader::CVerticalHeader()
{
}

CVerticalHeader::~CVerticalHeader()
{
}


BEGIN_MESSAGE_MAP(CVerticalHeader, CHeaderCtrl)
        //{{AFX_MSG_MAP(CVerticalHeader)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/****************************************************************************
*                        CVerticalHeader::GetItemText
* Inputs:
*       UINT col: Column to retrive text for
*       CString & s: Destination
* Result: void
*       
* Effect: 
*       Reads the text
****************************************************************************/

void CVerticalHeader::GetItemText(UINT col, CString & s)
    {
     HDITEM item = {0};
     item.pszText = s.GetBuffer(MAX_PATH);
     item.cchTextMax = MAX_PATH;
     item.mask = HDI_TEXT;
     GetItem(col, &item);
     s.ReleaseBuffer();
    } // CVerticalHeader::GetItemText

/****************************************************************************
*                        CVerticalHeader::GetItemData
* Inputs:
*       UINT col: Column
* Result: LPARAM
*       The item data
****************************************************************************/

LPARAM CVerticalHeader::GetItemData(UINT col)
    {
     HDITEM item = {0};
     item.mask = HDI_LPARAM;
     GetItem(col, &item);
     return item.lParam;
    } // CVerticalHeader::GetItemData

/////////////////////////////////////////////////////////////////////////////
// CVerticalHeader message handlers

void CVerticalHeader::DrawItem(LPDRAWITEMSTRUCT dis) 
   {
    CDC * dc = CDC::FromHandle(dis->hDC);
    CRect rect = dis->rcItem;
        
    int save = dc->SaveDC();
    
    CString s;
    GetItemText(dis->itemID, s);

    TRACE2("CVerticalHeader::DrawItem: rect=(%d, %d, ", rect.left, rect.top);
    TRACE3("%d, %d) \"%s\"\n", rect.right, rect.bottom, s);
    
    LPARAM flags = GetItemData(dis->itemID);
    
    int x;
    int y;
    CSize sz;
    UINT align = 0; 
    int gap = 2 * ::GetSystemMetrics(SM_CXBORDER);
    int vgap = dc->GetTextExtent(_T("  ")).cx; 
    
    switch(flags & DT_ROTATEMASK)
       { /* flags */
        case DT_0:
           dc->SelectObject(&font0);
           TRACE(_T("font0\n"));
           sz = dc->GetTextExtent(s);
           
           switch(flags & DT_HORZ)
              { /* horz */
               case DT_LEFT:
                  x = rect.left + gap;
                  align |= TA_LEFT;
                  break;
               case DT_CENTER:
                  x = rect.Width() / 2;
                  align |= TA_CENTER;
                  break;
               case DT_RIGHT:
                  x = rect.right - gap;
                  align |= TA_RIGHT;
                  break;
              } /* horz */
           switch(flags & DT_VERT) 
              { /* vert */
               case DT_TOP:
                  align |= TA_TOP;
                  y = rect.top + gap;
                  break;
               case DT_VCENTER:
                  align |= TA_BOTTOM;
                  y = rect.bottom - rect.Height() / 2 + sz.cx / 2;
                  break;
               case DT_BOTTOM:
                  align |= TA_BOTTOM;
                  y = rect.bottom - gap;
                  break;
              } /* vert */
           dc->SetTextAlign(align);
           break;
        case DT_90:
           dc->SelectObject(&font90);
           TRACE(_T("font90\n"));
           sz = dc->GetTextExtent(s);
           switch(flags & DT_HORZ)
              { /* horz90 */
               case DT_LEFT:
                  align |= TA_LEFT;
                  x = rect.left + gap;
                  break;
               case DT_CENTER:
                  align |= TA_LEFT;
                  x = rect.left + rect.Width() / 2 - sz.cy / 2;
                  break;
               case DT_RIGHT:
                  align |= TA_RIGHT;
                  y = rect.top + gap;
                  break;
              } /* horz90 */
           switch(flags & DT_VERT)
              { /* vert90 */
               case DT_BOTTOM:
                  align |= TA_BOTTOM;
                  y = rect.bottom - gap - vgap;
                  break;
               case DT_VCENTER:
                  align |= TA_CENTER;
                  y = rect.bottom - rect.Height() / 2 + sz.cx / 2;
                  break;
               case DT_TOP:
                  align |= TA_RIGHT;
                  y = rect.top + gap;
                  break;
              } /* vert90 */
           break;
        case DT_180:
        case DT_270:
           ASSERT(FALSE); // not yet supported
           break;
        default:
           ASSERT(FALSE);
           break;
       } /* flags */

    // If text extent is > r.Width but < r.Height(), and
    // alignment is DT_0, make it DT_90
    // This is just a heuristic

    sz = dc->GetTextExtent(s);
    if(sz.cx > rect.Width() && ((flags & DT_ROTATEMASK) == DT_0))
       { /* maybe rotate */
        dc->SelectObject(&font90);
        align = TA_TOP | TA_LEFT;
        y = rect.bottom - gap ;
        x = rect.left + gap;
       } /* maybe rotate */

#ifdef _DEBUG //----------------------------------------------------------------------+
    LOGFONT lf; // we want to see font details when debugging                      // |
    CFont * f = CFont::FromHandle((HFONT)::GetCurrentObject(dc->m_hDC, OBJ_FONT)); // |
    f->GetLogFont(&lf);                                                            // |
#endif //-----------------------------------------------------------------------------+
    CRgn rgn;
    rgn.CreateRectRgn(rect.left, rect.top, rect.right, rect.bottom);

    dc->SelectClipRgn(&rgn);
    
    TRACE3("TextOut(%d, %d, \"%s\")\n", x, y, s);
    //dc->DrawText(s, rect, flags
    dc->SetTextAlign(align);
    dc->TextOut(x, y, s);
    GDI_FLUSH();

    dc->RestoreDC(save);
    //CHeaderCtrl::DrawItem(dis);
   }

/****************************************************************************
*                           CVerticalHeader::SetTextAlign
* Inputs:
*       UINT col: Column to SetTextAlign
*       UINT options: alignment flags
* Result: void
*       
* Effect: 
*       Marks the column as being rotated by a specific amount, and saves
*       the drawing request as part of the LPARAM
* Notes:
*       It is impossible to change the height of the header without doing a
*       SetFont of the font that determines the height. Therefore, we do a
*       SetFont of a fake font which is solely to handle this bizarre case.
*       This requires that a SetTextAlign be done on every column or the
*       default font will be used, which will look weird.
****************************************************************************/

void CVerticalHeader::SetTextAlign(UINT col, UINT options)
    {
     HDITEM item = {0};
     CString text;
     item.pszText = text.GetBuffer(MAX_PATH);
     item.cchTextMax = MAX_PATH;
     item.mask = HDI_FORMAT | HDI_TEXT;
     GetItem(col, &item);
     text.ReleaseBuffer();

     // We have retrieved the text and the current format information

     // Force it to be owner-draw, and remove the text flag since we do
     // not need to set the text back
     item.fmt |= HDF_OWNERDRAW;
     item.mask &= ~HDI_TEXT;
     

     int gap = 2 * ::GetSystemMetrics(SM_CXBORDER) * 3;

     CClientDC dc(this);
     dc.SelectObject(&font0); // Start with our current 0-degree font
     CSize sz = dc.GetTextExtent(text);
      
     item.cxy = sz.cy + gap;
     item.mask |= HDI_WIDTH | HDI_LPARAM;
     item.lParam = options;
     SetItem(col, &item);

     int height;
     LOGFONT lf;

     switch(options & DT_ROTATEMASK)
        { /* figure out height */
         case DT_90:
         case DT_270:
            height = sz.cx + gap;
            fontfake.GetLogFont(&lf);
            break;
         case DT_0:
         case DT_180:
            height = sz.cy + gap;
            font0.GetLogFont(&lf);
            break;
        } /* figure out height */

     if(abs(lf.lfHeight) < height)
        { /* create new font */
         if(lf.lfHeight < 0)
            { /* negative */
             lf.lfHeight = -height;
             lf.lfHeight -= 2 * gap; // fudge factor empirically determined
            } /* negative */
         else
            { /* positive */
             lf.lfHeight = height;
             lf.lfHeight += 2 * gap; // fudge factor empirically determined
            } /* positive */

         fontfake.DeleteObject();
         fontfake.CreateFontIndirect(&lf);
         SetFont(&fontfake);
        } /* create new font */
    } // CVerticalHeader::SetTextAlign

/****************************************************************************
*                     CVerticalHeader::PreSubclassWindow
* Result: void
*       
* Effect: 
*       Handles the font creation.
* Notes:
*       THis currently uses "Arial" but we might consider either "Tahoma"
*       or figuring out the default system font.
*       Note that MS Sans Serif fonts don't seem to rotate well
****************************************************************************/

void CVerticalHeader::PreSubclassWindow() 
   {
    CFont * f = GetFont();
    LOGFONT lf;
    f->GetLogFont(&lf);
    lstrcpy(lf.lfFaceName, _T("Arial"));
    VERIFY(font0.CreateFontIndirect(&lf));
    fontfake.CreateFontIndirect(&lf);
    lf.lfOrientation = 900;
    lf.lfEscapement = lf.lfOrientation;
    VERIFY(font90.CreateFontIndirect(&lf));
    CHeaderCtrl::PreSubclassWindow();
   }

