// WhatIsThisEdit.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "WhatIsThisEdit.h"
#include "Help.h"
#include "lctypemsg.h"
#include "GetCaption.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CWhatIsThisEdit, CNoNotifyEdit)
/////////////////////////////////////////////////////////////////////////////
// CWhatIsThisEdit

CWhatIsThisEdit::CWhatIsThisEdit()
   {
    MultiSz = FALSE;
    HelpCursor = ::LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_HELP_IBEAM));
   }

CWhatIsThisEdit::~CWhatIsThisEdit()
{
}


BEGIN_MESSAGE_MAP(CWhatIsThisEdit, CNoNotifyEdit)
        //{{AFX_MSG_MAP(CWhatIsThisEdit)
        ON_WM_CONTEXTMENU()
        ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
        ON_COMMAND(ID_EDIT_CUT, OnEditCut)
        ON_COMMAND(ID_EDIT_DELETE, OnEditDelete)
        ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
        ON_COMMAND(ID_EDIT_SELECTALL, OnEditSelectall)
        ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
        ON_COMMAND(IDM_WHAT_IS_THIS, OnWhatIsThis)
        ON_WM_RBUTTONDOWN()
        ON_COMMAND(IDM_DEBUG_INFO, OnDebugInfo)
        ON_WM_SETCURSOR()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWhatIsThisEdit message handlers

void CWhatIsThisEdit::OnContextMenu(CWnd* pWnd, CPoint point) 
   {
    CMenu menu;
    
    int low;
    int high;
    CNoNotifyEdit::GetSel(low, high);
    BOOL hasSelection = (low < high);

    BOOL pasteable = ::IsClipboardFormatAvailable(CF_TEXT);
    BOOL readonly = (GetStyle() & ES_READONLY) != 0;

    int length = GetWindowTextLength();
    BOOL empty = length == 0;
    BOOL allSelected = (low == 0 && high == length);

    CString caption = ::GetCaption(this);
    BOOL what = (CHelp::FindHelpResource(caption) != NULL);

    menu.LoadMenu(IDR_EDIT_POPUP);
    menu.EnableMenuItem(ID_EDIT_UNDO,     MF_BYCOMMAND | (!readonly && CNoNotifyEdit::CanUndo() ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(ID_EDIT_CUT,      MF_BYCOMMAND | (!readonly && hasSelection     ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(ID_EDIT_COPY,     MF_BYCOMMAND | (hasSelection                  ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(ID_EDIT_PASTE,    MF_BYCOMMAND | (!readonly && pasteable        ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(ID_EDIT_DELETE,   MF_BYCOMMAND | (!readonly && hasSelection     ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(ID_EDIT_SELECTALL,MF_BYCOMMAND | (!empty    && !allSelected     ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
    menu.EnableMenuItem(IDM_WHAT_IS_THIS, MF_BYCOMMAND | (what                          ? MF_ENABLED : (MF_DISABLED | MF_GRAYED)));
#ifndef _DEBUG
    menu.DeleteMenu(IDM_DEBUG_INFO, MF_BYCOMMAND);
    //menu.DeleteMenu(IDM_HEX, MF_BYCOMMAND);
#endif
    CMenu * popup = menu.GetSubMenu(0);

    popup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this); 
}

/****************************************************************************
*                         CWhatIsThisEdit::OnEditCopy
* Result: void
*       
* Effect: 
*       Copies to clipboard
****************************************************************************/

void CWhatIsThisEdit::OnEditCopy() 
   {
    SendMessage(WM_COPY);
   }

/****************************************************************************
*                         CWhatIsThisEdit::OnEditCut
* Result: void
*       
* Effect: 
*       Cuts current selection to clipboard
****************************************************************************/

void CWhatIsThisEdit::OnEditCut() 
   {
    SendMessage(WM_CUT);
   }

/****************************************************************************
*                        CWhatIsThisEdit::OnEditDelete
* Result: void
*       
* Effect: 
*       Clears the text
****************************************************************************/

void CWhatIsThisEdit::OnEditDelete() 
   {
    SendMessage(WM_CLEAR);
   }

/****************************************************************************
*                        CWhatIsThisEdit::OnEditPaste
* Result: void
*       
* Effect: 
*       Pastes from the clipboard into the edit control
****************************************************************************/

void CWhatIsThisEdit::OnEditPaste() 
   {
    SendMessage(WM_PASTE);
   }

/****************************************************************************
*                      CWhatIsThisEdit::OnEditSelectall
* Result: void
*       
* Effect: 
*       Selects the entire contents
****************************************************************************/

void CWhatIsThisEdit::OnEditSelectall()
    {
     SendMessage(EM_SETSEL, 0, -1);
    } // CWhatIsThisEdit::OnEditSelectall

/****************************************************************************
*                         CWhatIsThisEdit::OnEditUndo
* Result: void
*       
* Effect: 
*       Undoes the last change
****************************************************************************/

void CWhatIsThisEdit::OnEditUndo() 
   {
    SendMessage(WM_UNDO);
   }

/****************************************************************************
*                        CWhatIsThisEdit::OnWhatIsThis
* Result: void
*       
* Effect: 
*       Pops up the explanation window
****************************************************************************/

void CWhatIsThisEdit::OnWhatIsThis() 
   {
    CString s = ::GetCaption(this);
    CHelp dlg;
    dlg.m_HelpName = s;
    dlg.DoModal();
   }

/****************************************************************************
*                       CWhatIsThisEdit::OnRButtonDown
* Inputs:
*       UINT nFlags:
*       CPoint point: 
* Result: void
*       
* Effect: 
*       Sets the focus and passes the message on (which eventually generates
*       a WM_CONTEXTMENU)
****************************************************************************/

void CWhatIsThisEdit::OnRButtonDown(UINT nFlags, CPoint point) 
   {
    SetFocus();
    CNoNotifyEdit::OnRButtonDown(nFlags, point);
   }

/****************************************************************************
*                        CWhatIsThisEdit::OnDebugInfo
* Result: void
*       
* Effect: 
*       Pops up whatever currently is the debug info of concern
****************************************************************************/

void CWhatIsThisEdit::OnDebugInfo() 
   {
#ifdef _DEBUG //---------------------------------------------------------------------------+
    CFont * f;                                                                          // |
    f = GetFont();                                                                      // |
    LOGFONT lf;                                                                         // |
    f->GetLogFont(&lf);                                                                 // |
    CString msg;                                                                        // |
    msg.Format(_T("lfFaceName \"%s\"\nlfFontHeight %d"), lf.lfFaceName, lf.lfHeight);   // |
    CString text;                                                                       // |
    CEdit::GetWindowText(text);                                                         // |
    CString sep = _T("\r\n");                                                           // |
    for(int i = 0; i < text.GetLength(); i++)                                           // |
       { /* convert */                                                                  // |
        CString t;                                                                      // |
#ifdef _UNICODE                                                                         // |
        t.Format(_T("%s%04X"), sep, (WCHAR)text[i]);                                    // |
#else                                                                                   // |
        t.Format(_T("%s%02x"), sep, (BYTE)text[i]);                                     // |
#endif                                                                                  // |
        sep = _T(" ");                                                                  // |
        msg += t;                                                                       // |
       } /* convert */                                                                  // |
    AfxMessageBox(msg, MB_ICONINFORMATION | MB_OK);                                     // |
#endif  //---------------------------------------------------------------------------------+
}

/****************************************************************************
*                        CWhatIsThisEdit::OnSetCursor
* Inputs:
*       CWnd * pWnd: ignroed
*       UINT nHitTest: ignored
*       UINT message: ignored
* Result: BOOL
*       TRUE, always
* Effect: 
*       Indicates there is help (potentially) available for this item
****************************************************************************/

BOOL CWhatIsThisEdit::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
   {
   if(nHitTest == HTCLIENT)
       { /* in client area */
        ::SetCursor(HelpCursor);
        return TRUE;
       } /* in client area */
    // TODO: Add your message handler code here and/or call default
        
    return CNoNotifyEdit::OnSetCursor(pWnd, nHitTest, message);
   }
