#include "stdafx.h"
#include "GetCaption.h"
/****************************************************************************
*                                   GetCaption
* Inputs:
*       CWnd * wnd: Window whose caption is to be returned
* Result: CString
*       Accompanying caption
* Notes:
*       * If there is a caption window, that is, a static window which precedes
*         this window, its text is used.
*       * If the preceding window does not exist, the GWLP_USERDATA field
*         is used as the specification of the window whose caption is to be used
*       * If the GWLP_USERDATA field is NULL, the string #dddd where dddd is
*         the decimal value of GetCtrlID
*
*       If there are any "&" characters in the caption, they are deleted
****************************************************************************/

CString GetCaption(CWnd * wnd)
   {
    CString s;
    CWnd * captionwnd = wnd->GetWindow(GW_HWNDPREV); // get previous caption
    if(captionwnd != NULL)
       { /* is static? */
        TCHAR classname[MAX_PATH];
        if(!::GetClassName(captionwnd->m_hWnd, classname, MAX_PATH))
           { /* GetClssName failed */
            captionwnd = NULL;
           } /* GetClssName failed */
        else
           { /* got class name */
            if(_tcsicmp(classname, _T("static")) != 0)
               { /* not static class */
                captionwnd = NULL;
               } /* not static class */
           } /* got class name */
        
       } /* is static? */

    // If we get here, then either we have a NULL window because
    //   (a) there was no preceding window
    //   (b) the preceding window was not a static window
    //   (c) the classname of the preceding window couldn't be read
    // or we have the reference to a STATIC class caption window
    //
    // If we have a NULL window, then the GWLP_USERDATA field of the
    // desired window might be the handle of the caption window
    
    if(captionwnd == NULL)
       { /* special case */
        HWND prev = (HWND)::GetWindowLongPtr(wnd->m_hWnd, GWLP_USERDATA);
        if(prev == NULL)
           { /* create #ddd */
            s.Format(_T("#%d"), wnd->GetDlgCtrlID());
            return s;
           } /* create #ddd */
        captionwnd = CWnd::FromHandle(prev);
       } /* special case */

    captionwnd->GetWindowText(s);
    s.Replace(_T("&"), _T(""));
    return s;
   } // GetCaption
