#ifdef EXPLORERDLG
#define QuerySource() this
#else
#define QuerySource() TabParent    // The parent of the tab control that holds this dialog
#endif

#include "LangType.h"

typedef enum {
   QUERY_LOCALE,   // (LRESULT)(LCID)
   QUERY_NOUSEROVERRIDE, // (LRESULT)(DWORD)
   QUERY_USE_CP_ACP,     // (LRESULT)(DWORD)
   QUERY_LANGUAGE,       // (LRESULT)(LangType)
   QUERY_DWORD,          // (LRESULT)(BOOL)
   QUERY_TEMPLATE,       // (LRESULT)(LPCTSTR)
   QUERY_VARIABLE,       // (LRESULT)(LPCTSTR)
   QUERY_DIGITINFO,      // (LRESULT)(SCRIPT_DIGITSUBSTITUTE *)
} QueryType;

#define QueryLocale()             ((LCID)QuerySource()->SendMessage(UWM_QUERY, (WPARAM)QUERY_LOCALE))
#define QueryLocaleFrom(x)        ((LCID)((x)->SendMessage(UWM_QUERY, (WPARAM)QUERY_LOCALE)))
#define MISSING_LCID ((LCID)-1)

#define QueryDigitInfo()    ((SCRIPT_DIGITSUBSTITUTE *)(QuerySource()->SendMessage(UWM_QUERY, (WPARAM)QUERY_DIGITINFO)))
#define EnableQuery(q,b)    ((void)                    (QuerySource()->SendMessage(UWM_ENABLE_QUERY, (WPARAM)(q), (LPARAM)(b))))
#define QueryFlag(q)        ((DWORD)                    QuerySource()->SendMessage(UWM_QUERY,        (WPARAM)(q)))
#define QueryLanguage()     ((LangType)                 QuerySource()->SendMessage(UWM_QUERY,        (WPARAM)QUERY_LANGUAGE))
#define QueryDWORD()        ((BOOL)                     QuerySource()->SendMessage(UWM_QUERY,        (WPARAM)QUERY_DWORD))
#define QueryTemplate(name) ((LPCTSTR)                  QuerySource()->SendMessage(UWM_QUERY,        (WPARAM)QUERY_TEMPLATE, (LPARAM)(LPCTSTR)(name)))
#define QueryVariable()     ((LPCTSTR)                  QuerySource()->SendMessage(UWM_QUERY,        (WPARAM)QUERY_VARIABLE))

#define SetVariable(s)      ((void)                     QuerySource()->SendMessage(UWM_SET_VARIABLE, (WPARAM)(LPCTSTR)(s)))
#define SetCode(s, e)       ((void)                     QuerySource()->SendMessage(UWM_SET_CODE,     (WPARAM)(LPCTSTR)(s), (LPARAM)(e)))

/****************************************************************************
*                                  UWM_QUERY
* Inputs:
*       WPARAM: (WPARAM)(QueryType)
*       LPARAM: not used
* Result: LRESULT
*       (LRESULT)(as specified by QueryType comments)
* Effect: 
*       Returns a value based on the query request
****************************************************************************/

#define UWM_QUERY_MSG _T("UWM_QUERY-{96B9AB6C-F310-49d3-8EE3-B7702C0A1CF3}")


/****************************************************************************
*                              UWM_ENABLE_QUERY
* Inputs:
*       WPARAM: (WPARAM)(QueryType) Query to be enabled or disabled
*       LPARAM: (LPARAM)(BOOL) TRUE to enable, FALSE to disable
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Enables/disables the query. If disabled, the UWM_QUERY will always
*       be FALSE
****************************************************************************/

#define UWM_ENABLE_QUERY_MSG _T("UWM_ENABLE_QUERY-{96B9AB6C-F310-49d3-8EE3-B7702C0A1CF3}")

// These messages are sent by the child dialogs to the main dialog to
// request actions or query state

/****************************************************************************
*                                UWM_SET_CODE
* Inputs:
*       WPARAM: (WPARAM)(LPCTSTR) String to store
*       LPARAM: (LPARAM)(BOOL) whether to enable the control for copy or not
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Sets the code string in the control.
* Notes:
*       Sent by a child dialog to the main dialog to update the code sample
*       The CString * is managed by the sender and must NOT be deleted by
*       the recipient
****************************************************************************/

#define UWM_SET_CODE_MSG _T("UWM_SET_CODE-{96B9AB6C-F310-49d3-8EE3-B7702C0A1CF3}")



/****************************************************************************
*                              UWM_SET_VARIABLE
* Inputs:
*       WPARAM: (WPARAM)(LPCTSTR) new name of variable to set
*       LPARAM: ignored
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Updates the "variable" edit control
****************************************************************************/

#define UWM_SET_VARIABLE_MSG _T("UWM_SET_VARIABLE-{96B9AB6C-F310-49d3-8EE3-B7702C0A1CF3}")
// Note that this defines a unique static UINT in each module in which it is included

#include "msg.h"

DECLARE_MESSAGE(UWM_QUERY)
DECLARE_MESSAGE(UWM_SET_VARIABLE)
DECLARE_MESSAGE(UWM_ENABLE_QUERY)
DECLARE_MESSAGE(UWM_SET_CODE)
